/*  
 * ia32_ioctl_prototype.h 
 * 
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
 
#include <ia32x_utils.h>


/* from /usr/include/linux/fs.h */
#define BLKGETSIZE _IO(0x12,96)	/* return device size */
#define BLKRAGET   _IO(0x12,99) /* get current read ahead setting */
#define BLKFRAGET  _IO(0x12,101)/* get filesystem (mm/filemap.c) read-ahead */
#define BLKSECTGET _IO(0x12,103)/* get max sectors per request (ll_rw_blk.c) */
#define BLKBSZGET  _IOR(0x12,112,sizeof(int))
#define BLKBSZSET  _IOW(0x12,113,sizeof(int))
#define BLKGETSIZE64 _IOR(0x12,114,size_t)


/* from /usr/include/linux/msdos_fs.h */
/*
 * ioctl commands
 */
#define	VFAT_IOCTL_READDIR_BOTH		_IOR('r', 1, struct dirent [2])
#define	VFAT_IOCTL_READDIR_SHORT	_IOR('r', 2, struct dirent [2])


/* from /usr/include/linux/raw.h */
#define RAW_SETBIND	_IO( 0xac, 0 )
#define RAW_GETBIND	_IO( 0xac, 1 )


/* from /usr/include/linux/if.h */
struct ifmap 
{
	unsigned long mem_start;
	unsigned long mem_end;
	unsigned short base_addr; 
	unsigned char irq;
	unsigned char dma;
	unsigned char port;
	/* 3 bytes spare */
};

struct ifreq 
{
#define IFHWADDRLEN	6
#define	IFNAMSIZ	16
	union
	{
		char	ifrn_name[IFNAMSIZ];		/* if name, e.g. "en0" */
	} ifr_ifrn;
	
	union {
		struct	sockaddr ifru_addr;
		struct	sockaddr ifru_dstaddr;
		struct	sockaddr ifru_broadaddr;
		struct	sockaddr ifru_netmask;
		struct  sockaddr ifru_hwaddr;
		short	ifru_flags;
		int	ifru_ivalue;
		int	ifru_mtu;
		struct  ifmap ifru_map;
		char	ifru_slave[IFNAMSIZ];	/* Just fits the size */
		char	ifru_newname[IFNAMSIZ];
		char *	ifru_data;
	} ifr_ifru;
};

struct ifconf 
{
	int	ifc_len;			/* size of buffer	*/
	union 
	{
		char *			ifcu_buf;
		struct	ifreq 		*ifcu_req;
	} ifc_ifcu;
};

#define	ifc_buf	ifc_ifcu.ifcu_buf		/* buffer address	*/
#define	ifc_req	ifc_ifcu.ifcu_req		/* array of structures	*/


/* from /usr/include/linux/hdreg.h */
#define HDIO_GETGEO		0x0301	/* get device geometry */
#define HDIO_GET_UNMASKINTR     0x0302  /* get current unmask setting */
#define HDIO_GET_MULTCOUNT      0x0304  /* get current IDE blockmode setting */
#define HDIO_GET_KEEPSETTINGS   0x0308  /* get keep-settings-on-reset flag */
#define HDIO_GET_32BIT          0x0309  /* get current io_32bit setting */
#define HDIO_GET_NOWERR         0x030a  /* get ignore-write-error flag */
#define HDIO_GET_DMA            0x030b  /* get use-dma flag */
#define HDIO_GET_NICE           0x030c  /* get nice flags */


struct hd_geometry {
      unsigned char heads;
      unsigned char sectors;
      unsigned short cylinders;
      unsigned long start;
};


/* from /usr/include/linux/reiserfs_fs.h */
#define REISERFS_IOC_UNPACK             _IOW(0xCD,1,long)


/* from /usr/include/linux/ext2_fs.h */
#define EXT2_IOC_GETFLAGS               _IOR('f', 1, long)
#define EXT2_IOC_SETFLAGS               _IOW('f', 2, long)
#define EXT2_IOC_GETVERSION             _IOR('v', 1, long)
#define EXT2_IOC_SETVERSION             _IOW('v', 2, long)


/* from /usr/include/linux/sockios.h */
#define SIOCGSTAMP      0x8906          /* Get stamp */
#define SIOCADDRT       0x890B          /* add routing table entry      */
#define SIOCDELRT       0x890C          /* delete routing table entry   */
#define SIOCGIFFLAGS    0x8913          /* get flags                    */
#define SIOCSIFFLAGS    0x8914          /* set flags                    */
#define SIOCGIFADDR     0x8915          /* get PA address               */
#define SIOCSIFADDR     0x8916          /* set PA address               */
#define SIOCGIFDSTADDR  0x8917          /* get remote PA address        */
#define SIOCSIFDSTADDR  0x8918          /* set remote PA address        */
#define SIOCGIFBRDADDR  0x8919          /* get broadcast PA address     */
#define SIOCSIFBRDADDR  0x891a          /* set broadcast PA address     */
#define SIOCGIFNETMASK  0x891b          /* get network PA mask          */
#define SIOCSIFNETMASK  0x891c          /* set network PA mask          */
#define SIOCGIFMETRIC   0x891d          /* get metric                   */
#define SIOCSIFMETRIC   0x891e          /* set metric                   */
#define SIOCGIFMEM      0x891f          /* get memory address (BSD)     */
#define SIOCSIFMEM      0x8920          /* set memory address (BSD)     */
#define SIOCGIFMTU      0x8921          /* get MTU size                 */
#define SIOCSIFMTU      0x8922          /* set MTU size                 */
#define SIOCSIFHWADDR   0x8924          /* set hardware address         */
#define SIOCGIFHWADDR   0x8927          /* Get hardware address         */
#define SIOCADDMULTI    0x8931          /* Multicast address lists      */
#define SIOCDELMULTI    0x8932
#define SIOCGIFINDEX    0x8933          /* name -> if_index mapping     */
#define SIOCSIFPFLAGS   0x8934          /* set/get extended flags set   */
#define SIOCGIFPFLAGS   0x8935
#define SIOCGIFTXQLEN   0x8942          /* Get the tx queue length      */
#define SIOCSIFTXQLEN   0x8943          /* Set the tx queue length      */
#define SIOCETHTOOL     0x8946          /* Ethtool interface            */
#define SIOCGIFMAP      0x8970          /* Get device parameters        */
#define SIOCSIFMAP      0x8971          /* Set device parameters        */
#define SIOCBONDENSLAVE 0x8990          /* enslave a device to the bond */
#define SIOCBONDRELEASE 0x8991          /* release a slave from the bond*/
#define SIOCBONDSETHWADDR      0x8992   /* set the hw addr of the bond  */
#define SIOCBONDSLAVEINFOQUERY 0x8993   /* rtn info about slave state   */
#define SIOCBONDINFOQUERY      0x8994   /* rtn info about bond state    */
#define SIOCBONDCHANGEACTIVE   0x8995   /* update to a new active slave */
#define SIOCDEVPRIVATE  0x89F0		/* to 89FF */


/* from /usr/include/linux/blkpg.h */
#define BLKPG      _IO(0x12,105)

/* The argument structure */
struct blkpg_ioctl_arg {
        int op;
        int flags;
        int datalen;
        void *data;
};

/* The subfunctions (for the op field) */
#define BLKPG_ADD_PARTITION     1
#define BLKPG_DEL_PARTITION     2

/* Sizes of name fields. Unused at present. */
#define BLKPG_DEVNAMELTH        64
#define BLKPG_VOLNAMELTH        64

/* The data structure for ADD_PARTITION and DEL_PARTITION */
struct blkpg_partition {
        long long start;                /* starting offset in bytes */
        long long length;               /* length in bytes */
        int pno;                        /* partition number */
        char devname[BLKPG_DEVNAMELTH]; /* partition name, like sda5 or c0d1p2,
                                           to be used in kernel messages */
        char volname[BLKPG_VOLNAMELTH]; /* volume label */
};


/* from /usr/include/linux/route.h */
struct rtentry
{
        unsigned long   rt_pad1;
        struct sockaddr rt_dst;         /* target address               */
        struct sockaddr rt_gateway;     /* gateway addr (RTF_GATEWAY)   */
        struct sockaddr rt_genmask;     /* target network mask (IP)     */
        unsigned short  rt_flags;
        short           rt_pad2;
        unsigned long   rt_pad3;
        void            *rt_pad4;
        short           rt_metric;      /* +1 for binary compatibility! */
        char            *rt_dev;        /* forcing the device at add    */
        unsigned long   rt_mtu;         /* per route MTU/Window         */
#define rt_mss  rt_mtu                  /* Compatibility :-(            */
        unsigned long   rt_window;      /* Window clamping              */
        unsigned short  rt_irtt;        /* Initial RTT                  */
};


/* from /usr/include/linux/auto_fs.h */
#define AUTOFS_IOC_SETTIMEOUT _IOWR(0x93,0x64,unsigned long)


/* from /usr/include/linux/loop.h */
#define LOOP_SET_STATUS 0x4C02
#define LOOP_GET_STATUS 0x4C03

#define LO_NAME_SIZE    64
#define LO_KEY_SIZE     32

struct loop_info {
        int             lo_number;      /* ioctl r/o */
        unsigned int    lo_device;      /* ioctl r/o */
        unsigned long   lo_inode;       /* ioctl r/o */
        unsigned int    lo_rdevice;     /* ioctl r/o */
        int             lo_offset;
        int             lo_encrypt_type;
        int             lo_encrypt_key_size;    /* ioctl w/o */
        int             lo_flags;       /* ioctl r/o */
        char            lo_name[LO_NAME_SIZE];
        unsigned char   lo_encrypt_key[LO_KEY_SIZE]; /* ioctl w/o */
        unsigned long   lo_init[2];
        char            reserved[4];
};

/* from /usr/include/linux/mtd/mtd.h */
struct mtd_oob_buf {
        __u32	start;
        __u32	ength;
        unsigned char *ptr;
};

#define MEMWRITEOOB             _IOWR('M', 3, struct mtd_oob_buf)
#define MEMREADOOB              _IOWR('M', 4, struct mtd_oob_buf)

/* from /usr/include/linux/serial.h */
struct serial_struct {
        int     type;
        int     line;
        unsigned int    port;
        int     irq;
        int     flags;
        int     xmit_fifo_size;
        int     custom_divisor;
        int     baud_base;
        unsigned short  close_delay;
        char    io_type;
        char    reserved_char[1];
        int     hub6;
        unsigned short  closing_wait; /* time to wait before closing */
        unsigned short  closing_wait2; /* no longer used... */
        unsigned char   *iomem_base;
        unsigned short  iomem_reg_shift;
        unsigned int    port_high;
        int     reserved[1];
};

/* from /usr/include/linux/filter.h */
struct sock_filter      /* Filter block */
{
        __u16   code;   /* Actual filter code */
        __u8    jt;     /* Jump true */
        __u8    jf;     /* Jump false */
        __u32   k;      /* Generic multiuse field */
};

struct sock_fprog       /* Required for SO_ATTACH_FILTER. */
{
        unsigned short          len;    /* Number of filter blocks */
        struct sock_filter      *filter;
};

/* from /usr/include/linux/ppp_defs.h */
struct ppp_idle {
    time_t xmit_idle;           /* time since last NP packet sent */
    time_t recv_idle;           /* time since last NP packet received */
};

/* from /usr/include/linux/if_ppp.h */
struct ppp_option_data {
        __u8    *ptr;
        __u32   length;
        int     transmit;
};

#define PPPIOCGIDLE     _IOR('t', 63, struct ppp_idle) /* get idle time */
#define PPPIOCSACTIVE   _IOW('t', 70, struct sock_fprog) /* set active filt */
#define PPPIOCSPASS     _IOW('t', 71, struct sock_fprog) /* set pass filter */
#define PPPIOCSCOMPRESS _IOW('t', 77, struct ppp_option_data)

/* from /usr/include/linux/fb.h */
#define FBIOGET_FSCREENINFO     0x4602
#define FBIOGETCMAP             0x4604
#define FBIOPUTCMAP             0x4605

struct fb_fix_screeninfo {
        char id[16];                    /* identification string eg "TT Builtin" */
        unsigned long smem_start;       /* Start of frame buffer mem */
                                        /* (physical address) */
        __u32 smem_len;                 /* Length of frame buffer mem */
        __u32 type;                     /* see FB_TYPE_*                */
        __u32 type_aux;                 /* Interleave for interleaved Planes */
        __u32 visual;                   /* see FB_VISUAL_*              */
        __u16 xpanstep;                 /* zero if no hardware panning  */
        __u16 ypanstep;                 /* zero if no hardware panning  */
        __u16 ywrapstep;                /* zero if no hardware ywrap    */
        __u32 line_length;              /* length of a line in bytes    */
        unsigned long mmio_start;       /* Start of Memory Mapped I/O   */
                                        /* (physical address) */
        __u32 mmio_len;                 /* Length of Memory Mapped I/O  */
        __u32 accel;                    /* Type of acceleration available */
        __u16 reserved[3];              /* Reserved for future compatibility */
};

struct fb_cmap {
        __u32 start;                    /* First entry  */
        __u32 len;                      /* Number of entries */
        __u16 *red;                     /* Red values   */
        __u16 *green;
        __u16 *blue;
        __u16 *transp;                  /* transparency, can be NULL */
};

/* from /usr/include/linux/kd.h */
#define GIO_UNIMAP      0x4B66  /* get unicode-to-font mapping from kernel */
#define PIO_UNIMAP      0x4B67  /* put unicode-to-font mapping in kernel */
#define GIO_FONTX       0x4B6B  /* get font using struct consolefontdesc */
#define PIO_FONTX       0x4B6C  /* set font using struct consolefontdesc */
#define KDFONTOP        0x4B72  /* font operations */


struct unipair {
        unsigned short unicode;
        unsigned short fontpos;
};
struct unimapdesc {
        unsigned short entry_ct;
        struct unipair *entries;
};

struct consolefontdesc {
        unsigned short charcount;       /* characters in font (256 or 512) */
        unsigned short charheight;      /* scan lines per character (1-32) */
        char *chardata;                 /* font data in expanded form */
};

struct console_font_op {
        unsigned int op;        /* operation code KD_FONT_OP_* */
        unsigned int flags;     /* KD_FONT_FLAG_* */
        unsigned int width, height;     /* font size */
        unsigned int charcount;
        unsigned char *data;    /* font data with height fixed to 32 */
};

/* from /usr/include/linux/mtio.h */
struct  mtpos {
        long    mt_blkno;       /* current block number */
};

struct	mtget {
	long	mt_type;	/* type of magtape device */
	long	mt_resid;	/* residual count: (not sure)
				 *	number of bytes ignored, or
				 *	number of files not skipped, or
				 *	number of records not skipped.
				 */
	/* the following registers are device dependent */
	long	mt_dsreg;	/* status register */
	long	mt_gstat;	/* generic (device independent) status */
	long	mt_erreg;	/* error register */
	/* The next two fields are not always used */
	__kernel_daddr_t mt_fileno;	/* number of current file on tape */
	__kernel_daddr_t mt_blkno;	/* current block number */
};

struct mtconfiginfo {
        long    mt_type;        /* drive type */
        long    ifc_type;       /* interface card type */
        unsigned short  irqnr;  /* IRQ number to use */
        unsigned short  dmanr;  /* DMA channel to use */
        unsigned short  port;   /* IO port base address */

        unsigned long   debug;  /* debugging flags */

        unsigned        have_dens:1;
        unsigned        have_bsf:1;
        unsigned        have_fsr:1;
        unsigned        have_bsr:1;
        unsigned        have_eod:1;
        unsigned        have_seek:1;
        unsigned        have_tell:1;
        unsigned        have_ras1:1;
        unsigned        have_ras2:1;
        unsigned        have_ras3:1;
        unsigned        have_qfa:1;

        unsigned        pad1:5;
        char            reserved[10];
};

#define MTIOCGET        _IOR('m', 2, struct mtget)      /* get tape status */
#define MTIOCPOS        _IOR('m', 3, struct mtpos)      /* get tape position */
#define MTIOCGETCONFIG  _IOR('m', 4, struct mtconfiginfo) /* get tape config */
#define MTIOCSETCONFIG  _IOW('m', 5, struct mtconfiginfo) /* set tape config */


#ifndef TIOCGSERIAL
#define TIOCGSERIAL        0x541E
#endif

#ifndef TIOCSSERIAL
#define TIOCSSERIAL        0x541F
#endif

/* from /usr/include/linux/cdrom.h */
#define CDROMREADAUDIO          0x530e /* (struct cdrom_read_audio) */
#define CDROM_SEND_PACKET       0x5393  /* send a packet to the drive */

struct cdrom_msf0
{
        __u8    minute;
        __u8    second;
        __u8    frame;
};

union cdrom_addr
{
        struct cdrom_msf0       msf;
        int                     lba;
};

struct cdrom_read_audio
{
        union cdrom_addr addr; /* frame address */
        __u8 addr_format;    /* CDROM_LBA or CDROM_MSF */
        int nframes;           /* number of 2352-byte-frames to read at once */
        __u8 *buf;           /* frame buffer (size: nframes*2352 bytes) */
};

#define CDROM_PACKET_SIZE       12

struct cdrom_generic_command
{
        unsigned char           cmd[CDROM_PACKET_SIZE];
        unsigned char           *buffer;
        unsigned int            buflen;
        int                     stat;
        void                    *sense; /* struct request_sense* */
        unsigned char           data_direction;
        int                     quiet;
        int                     timeout;
        void                    *reserved[1];
};

/* from /usr/include/linux/scsi/sg.h */
#define SG_IO 0x2285   /* similar effect as write() followed by read() */

typedef struct sg_iovec
{
  void * iov_base;            /* Starting address  */
  size_t iov_len;             /* Length in bytes  */
} sg_iovec_t;


typedef struct sg_io_hdr
{
  int interface_id;           /* [i] 'S' for SCSI generic (required) */
  int dxfer_direction;        /* [i] data transfer direction  */
  unsigned char cmd_len;      /* [i] SCSI command length ( <= 16 bytes) */
  unsigned char mx_sb_len;    /* [i] max length to write to sbp */
  unsigned short int iovec_count; /* [i] 0 implies no scatter gather */
  unsigned int dxfer_len;     /* [i] byte count of data transfer */
  void * dxferp;              /* [i], [*io] points to data transfer memory
                                 or scatter gather list */
  unsigned char * cmdp;       /* [i], [*i] points to command to perform */
  unsigned char * sbp;        /* [i], [*o] points to sense_buffer memory */
  unsigned int timeout;       /* [i] MAX_UINT->no timeout (unit: millisec) */
  unsigned int flags;         /* [i] 0 -> default, see SG_FLAG... */
  int pack_id;                /* [i->o] unused internally (normally) */
  void * usr_ptr;             /* [i->o] unused internally */
  unsigned char status;       /* [o] scsi status */
  unsigned char masked_status;/* [o] shifted, masked scsi status */
  unsigned char msg_status;   /* [o] messaging level data (optional) */
  unsigned char sb_len_wr;    /* [o] byte count actually written to sbp */
  unsigned short int host_status; /* [o] errors from host adapter */
  unsigned short int driver_status;/* [o] errors from software driver */
  int resid;                  /* [o] dxfer_len - actual_transferred */
  unsigned int duration;      /* [o] time taken by cmd (unit: millisec) */
  unsigned int info;          /* [o] auxiliary information */
} sg_io_hdr_t;


/* from /usr/include/linux/scsi/usbdevice_fs.h */
struct usbdevfs_ctrltransfer {
        __u8 requesttype;
        __u8 request;
        __u16 value;
        __u16 index;
        __u16 length;
        __u32 timeout;  /* in milliseconds */
        void *data;
};

struct usbdevfs_bulktransfer {
        unsigned int ep;
        unsigned int len;
        unsigned int timeout; /* in milliseconds */
        void *data;
};

struct usbdevfs_disconnectsignal {
        unsigned int signr;
        void *context;
};

#define USBDEVFS_CONTROL           _IOWR('U', 0, struct usbdevfs_ctrltransfer)
#define USBDEVFS_BULK              _IOWR('U', 2, struct usbdevfs_bulktransfer)
#define USBDEVFS_REAPURB           _IOW('U', 12, void *)
#define USBDEVFS_REAPURBNDELAY     _IOW('U', 13, void *)
#define USBDEVFS_DISCSIGNAL        _IOR('U', 14, struct usbdevfs_disconnectsignal)

/* from /usr/include/linux/scsi/videodev.h */
struct video_tuner
{
        int tuner;
        char name[32];
        ulong rangelow, rangehigh;      /* Tuner range */
        __u32 flags;
#define VIDEO_TUNER_PAL         1
#define VIDEO_TUNER_NTSC        2
#define VIDEO_TUNER_SECAM       4
#define VIDEO_TUNER_LOW         8       /* Uses KHz not MHz */
#define VIDEO_TUNER_NORM        16      /* Tuner can set norm */
#define VIDEO_TUNER_STEREO_ON   128     /* Tuner is seeing stereo */
#define VIDEO_TUNER_RDS_ON      256     /* Tuner is seeing an RDS datastream */
#define VIDEO_TUNER_MBS_ON      512     /* Tuner is seeing an MBS datastream */
        __u16 mode;                     /* PAL/NTSC/SECAM/OTHER */
#define VIDEO_MODE_PAL          0
#define VIDEO_MODE_NTSC         1
#define VIDEO_MODE_SECAM        2
#define VIDEO_MODE_AUTO         3
        __u16 signal;                   /* Signal strength 16bit scale */
};

struct video_clip
{
        __s32   x,y;
        __s32   width, height;
        struct  video_clip *next;       /* For user use/driver use only */
};

struct video_window
{
        __u32   x,y;                    /* Position of window */
        __u32   width,height;           /* Its size */
        __u32   chromakey;
        __u32   flags;
        struct  video_clip *clips;      /* Set only */
        int     clipcount;
#define VIDEO_WINDOW_INTERLACE  1
#define VIDEO_WINDOW_CHROMAKEY  16      /* Overlay by chromakey */
#define VIDEO_CLIP_BITMAP       -1
/* bitmap is 1024x625, a '1' bit represents a clipped pixel */
#define VIDEO_CLIPMAP_SIZE      (128 * 625)
};

struct video_buffer
{
        void    *base;
        int     height,width;
        int     depth;
        int     bytesperline;
};

#define VIDIOCGTUNER            _IOWR('v',4,struct video_tuner)         /* Get tuner abilities */
#define VIDIOCSTUNER            _IOW('v',5,struct video_tuner)          /* Tune the tuner for the current channel */
#define VIDIOCGWIN              _IOR('v',9, struct video_window)        /* Get the video overlay window */
#define VIDIOCSWIN              _IOW('v',10, struct video_window)       /* Set the video overlay window - passes clip list for hardware smarts , chromakey etc */
#define VIDIOCGFBUF             _IOR('v',11, struct video_buffer)       /* Get frame buffer */
#define VIDIOCSFBUF             _IOW('v',12, struct video_buffer)       /* Set frame buffer - root only */
#define VIDIOCGFREQ             _IOR('v',14, unsigned long)             /* Set tuner */
#define VIDIOCSFREQ             _IOW('v',15, unsigned long)             /* Set tuner */

/* from /usr/include/linux/scsi/atm.h */
struct atmif_sioc {
    int number;
    int length;
    void *arg;
};

/* from /usr/include/linux/scsi/atmioc.h */
#define ATMIOC_PHYTYP     0x10 /* PHY dev type ioctls, unique per PHY type */
#define ATMIOC_SARCOM     0x50 /* SAR device common ioctls, globally unique */
#define ATMIOC_ITF        0x80 /* Interface ioctls, globally unique */

/* from /usr/include/linux/scsi/atmdev.h */
struct atm_iobuf {
        int length;
        void *buffer;
};

#define ATM_GETLINKRATE _IOW('a',ATMIOC_ITF+1,struct atmif_sioc)
                                        /* get link rate */
#define ATM_GETNAMES    _IOW('a',ATMIOC_ITF+3,struct atm_iobuf)
                                        /* get interface names (numbers) */
#define ATM_GETTYPE     _IOW('a',ATMIOC_ITF+4,struct atmif_sioc)
                                        /* get interface type name */
#define ATM_GETESI      _IOW('a',ATMIOC_ITF+5,struct atmif_sioc)
                                        /* get interface ESI */
#define ATM_GETADDR     _IOW('a',ATMIOC_ITF+6,struct atmif_sioc)
                                        /* get itf's local ATM addr. list */
#define ATM_RSTADDR     _IOW('a',ATMIOC_ITF+7,struct atmif_sioc)
                                        /* reset itf's ATM address list */
#define ATM_ADDADDR     _IOW('a',ATMIOC_ITF+8,struct atmif_sioc)
                                        /* add a local ATM address */
#define ATM_DELADDR     _IOW('a',ATMIOC_ITF+9,struct atmif_sioc)
                                        /* remove a local ATM address */
#define ATM_GETCIRANGE  _IOW('a',ATMIOC_ITF+10,struct atmif_sioc)
                                        /* get connection identifier range */
#define ATM_SETCIRANGE  _IOW('a',ATMIOC_ITF+11,struct atmif_sioc)
                                        /* set connection identifier range */
#define ATM_SETESI      _IOW('a',ATMIOC_ITF+12,struct atmif_sioc)
                                        /* set interface ESI */
#define ATM_SETESIF     _IOW('a',ATMIOC_ITF+13,struct atmif_sioc)
                                        /* force interface ESI */
#define ATM_GETSTAT     _IOW('a',ATMIOC_SARCOM+0,struct atmif_sioc)
                                        /* get AAL layer statistics */
#define ATM_GETSTATZ    _IOW('a',ATMIOC_SARCOM+1,struct atmif_sioc)
                                        /* get AAL layer statistics and zero */
#define ATM_GETLOOP     _IOW('a',ATMIOC_SARCOM+2,struct atmif_sioc)
                                        /* get loopback mode */
#define ATM_SETLOOP     _IOW('a',ATMIOC_SARCOM+3,struct atmif_sioc)
                                        /* set loopback mode */
#define ATM_QUERYLOOP   _IOW('a',ATMIOC_SARCOM+4,struct atmif_sioc)
                                        /* query supported loopback modes */

/* from /usr/include/linux/scsi/sonet.h */
#define __SONET_ITEMS \
    __HANDLE_ITEM(section_bip);         /* section parity errors (B1) */ \
    __HANDLE_ITEM(line_bip);            /* line parity errors (B2) */ \
    __HANDLE_ITEM(path_bip);            /* path parity errors (B3) */ \
    __HANDLE_ITEM(line_febe);           /* line parity errors at remote */ \
    __HANDLE_ITEM(path_febe);           /* path parity errors at remote */ \
    __HANDLE_ITEM(corr_hcs);            /* correctable header errors */ \
    __HANDLE_ITEM(uncorr_hcs);          /* uncorrectable header errors */ \
    __HANDLE_ITEM(tx_cells);            /* cells sent */ \
    __HANDLE_ITEM(rx_cells);            /* cells received */

struct sonet_stats {
#define __HANDLE_ITEM(i) int i
        __SONET_ITEMS
#undef __HANDLE_ITEM
} __attribute__ ((packed));

#define SONET_FRSENSE_SIZE 6            /* C1[3],H1[3] (0xff for unknown) */

#define SONET_GETSTAT   _IOR('a',ATMIOC_PHYTYP,struct sonet_stats)
                                        /* get statistics */
#define SONET_GETSTATZ  _IOR('a',ATMIOC_PHYTYP+1,struct sonet_stats)
                                        /* ... and zero counters */
#define SONET_SETDIAG   _IOWR('a',ATMIOC_PHYTYP+2,int)
                                        /* set error insertion */
#define SONET_CLRDIAG   _IOWR('a',ATMIOC_PHYTYP+3,int)
                                        /* clear error insertion */
#define SONET_GETDIAG   _IOR('a',ATMIOC_PHYTYP+4,int)
                                        /* query error insertion */
#define SONET_SETFRAMING _IO('a',ATMIOC_PHYTYP+5)
                                        /* set framing mode (SONET/SDH) */
#define SONET_GETFRAMING _IOR('a',ATMIOC_PHYTYP+6,int)
                                        /* get framing mode */
#define SONET_GETFRSENSE _IOR('a',ATMIOC_PHYTYP+7, \
  unsigned char[SONET_FRSENSE_SIZE])    /* get framing sense information */



struct ifmap32 {
	__u32 mem_start;
	__u32 mem_end;
	__u16 base_addr;
	__u8 irq;
	__u8 dma;
	__u8 port;
};

struct ifreq32 {
	union {
		char	ifrn_name[IFNAMSIZ];
	} ifr_ifrn;
	union {
		struct sockaddr	ifru_addr;
		struct sockaddr	ifru_dstaddr;
		struct sockaddr	ifru_broadaddr;
		struct sockaddr	ifru_netmask;
		struct sockaddr	ifru_hwaddr;
		__s16	ifru_flags;
		__s32	ifru_ivalue;
		__s32	ifru_mtu;
		struct ifmap32	ifru_map;
		char	ifru_slave[IFNAMSIZ];
		char	ifru_newname[IFNAMSIZ];
		__u32	ifru_data;
	} ifr_ifru;
};

struct ifconf32 {
	__u32	ifc_len;
	__u32	ifcbuf;
};

struct hd_geometry32 {
	unsigned char  heads;
	unsigned char  sectors;
	unsigned short cylinders;
	__u32          start;
};

struct raw_config_request {
	__s32 raw_minor;
	__u64 block_major;
	__u64 block_minor;
};
struct raw_config_request32 {
	__s32 raw_minor;
	__u64 block_major;
	__u64 block_minor;
} __attribute__((packed));

struct blkpg_ioctl_arg32 {
        __s32	op;
        __s32	flags;
        __s32	datalen;
        __u32	data;
};

struct blkpg_partition32 {
        long long start;                /* starting offset in bytes */
        long long length;               /* length in bytes */
        int pno;                        /* partition number */
        char devname[BLKPG_DEVNAMELTH]; /* partition name, like sda5 or c0d1p2,
                                           to be used in kernel messages */
        char volname[BLKPG_VOLNAMELTH]; /* volume label */
} __attribute__((packed));

struct rtentry32 {
        __u32	rt_pad1;
        struct sockaddr rt_dst;         /* target address               */
        struct sockaddr rt_gateway;     /* gateway addr (RTF_GATEWAY)   */
        struct sockaddr rt_genmask;     /* target network mask (IP)     */
        __u16	rt_flags;
        __s16	rt_pad2;
        __u32	rt_pad3;
        __u32	rt_pad4;
        __s16	rt_metric;	/* +1 for binary compatibility! */
        __u32	rt_dev;		/* forcing the device at add    */
        __u32	rt_mtu;		/* per route MTU/Window         */
#define rt_mss  rt_mtu		/* Compatibility :-(            */
        __u32	rt_window;	/* Window clamping              */
        __u16	rt_irtt;	/* Initial RTT                  */
} __attribute__((packed));

struct loop_info32 {
        __s32	lo_number;      /* ioctl r/o */
        __u16	lo_device;      /* ioctl r/o */
        __u32	lo_inode;       /* ioctl r/o */
        __u16	lo_rdevice;     /* ioctl r/o */
        __s32	lo_offset;
        __s32	lo_encrypt_type;
        __s32	lo_encrypt_key_size;    /* ioctl w/o */
        __s32	lo_flags;       /* ioctl r/o */
        char	lo_name[LO_NAME_SIZE];
        unsigned char	lo_encrypt_key[LO_KEY_SIZE]; /* ioctl w/o */
        __u32	lo_init[2];
        char	reserved[4];
} __attribute__((packed));

struct mtd_oob_buf32 {
        __u32	start;
        __u32	ength;
        __u32	ptr;
} __attribute__((packed));


struct serial_struct32 {
        __s32	type;
        __s32	line;
        __u32	port;
        __s32	irq;
        __s32	flags;
        __s32	xmit_fifo_size;
        __s32	custom_divisor;
        __s32	baud_base;
        __u16	close_delay;
        char	io_type;
        char	reserved_char[1];
        __s32	hub6;
        __u16	closing_wait; /* time to wait before closing */
        __u16	closing_wait2; /* no longer used... */
        __u32	iomem_base;
        __u16	iomem_reg_shift;
        __u32	port_high;
        __s32	reserved[1];
} __attribute__((packed));

struct sock_fprog32
{
        __u16   len;
        __u32   filter;
};

struct ppp_idle32 {
        __s32   xmit_idle;
        __s32   recv_idle;
};

struct ppp_option_data32 {
        __u32   ptr;
        __u32   length;
        __s32   transmit;
};

struct fb_fix_screeninfo32 {
        char id[16];                    /* identification string eg "TT Builtin" */
        __u32 smem_start;               /* Start of frame buffer mem */
                                        /* (physical address) */
        __u32 smem_len;                 /* Length of frame buffer mem */
        __u32 type;                     /* see FB_TYPE_*                */
        __u32 type_aux;                 /* Interleave for interleaved Planes */
        __u32 visual;                   /* see FB_VISUAL_*              */
        __u16 xpanstep;                 /* zero if no hardware panning  */
        __u16 ypanstep;                 /* zero if no hardware panning  */
        __u16 ywrapstep;                /* zero if no hardware ywrap    */
        __u32 line_length;              /* length of a line in bytes    */
        __u32 mmio_start;               /* Start of Memory Mapped I/O   */
                                        /* (physical address) */
        __u32 mmio_len;                 /* Length of Memory Mapped I/O  */
        __u32 accel;                    /* Type of acceleration available */
        __u16 reserved[3];              /* Reserved for future compatibility */
} __attribute__((packed));

struct fb_cmap32 {
        __u32 start;                    /* First entry  */
        __u32 len;                      /* Number of entries */
        __u32 red;                     /* Red values   */
        __u32 green;
        __u32 blue;
        __u32 transp;                  /* transparency, can be NULL */
};

struct unimapdesc32 {
        __u16 entry_ct;
        __u32 entries;
};

struct consolefontdesc32 {
        __u16 charcount;
        __u16 charheight;
        __u32 chardata;
};

struct console_font_op32 {
        __u32 op;
        __u32 flags;
        __u32 width, height;
        __u32 charcount;
        __u32 data;
};

struct  mtpos32 {
        __s32    mt_blkno;
};

struct  mtget32 {
        __s32    mt_type;
        __s32    mt_resid;
        __s32    mt_dsreg;
        __s32    mt_gstat;
        __s32    mt_erreg;
        __kernel_daddr_t    mt_fileno;
        __kernel_daddr_t    mt_blkno;
};

struct mtconfiginfo32 {
        __s32    mt_type;        
        __s32    ifc_type;       
        __s16    irqnr;  
        __s16    dmanr;  
        __s16    port;   

        __u32    debug; 

        __u32    have_dens:1;
        __u32    have_bsf:1;
        __u32    have_fsr:1;
        __u32    have_bsr:1;
        __u32    have_eod:1;
        __u32    have_seek:1;
        __u32    have_tell:1;
        __u32    have_ras1:1;
        __u32    have_ras2:1;
        __u32    have_ras3:1;
        __u32    have_qfa:1;

        __u32    pad1:5;
        char     reserved[10];
};

struct cdrom_read_audio32 {
        union cdrom_addr addr;
        __u8 addr_format;    
        __s32 nframes;        
        __u32 buf;         
};

struct cdrom_generic_command32 {
        unsigned char    cmd[CDROM_PACKET_SIZE];
        __u32            buffer;
        __u32            buflen;
        __s32            stat;
        __u32            sense; /* struct request_sense* */
        unsigned char    data_direction;
        __s32            quiet;
        __s32            timeout;
        __u32            reserved[1];
};

typedef struct sg_iovec32 {
        __u32    iov_base;
        __u32    iov_len; 
} sg_iovec_t32;

typedef struct sg_io_hdr32 {
        __s32 interface_id;           
        __s32 dxfer_direction;        
        unsigned char cmd_len;      
        unsigned char mx_sb_len;    
        __u16 iovec_count; 
        __u32 dxfer_len; 
        __u32 dxferp;          
        __u32 cmdp;   
        __u32 sbp;    
        __u32 timeout;   
        __u32 flags;     
        __s32 pack_id;            
        __u32 usr_ptr;         
        unsigned char status;   
        unsigned char masked_status;
        unsigned char msg_status;   
        unsigned char sb_len_wr;    
        __u16 host_status; 
        __u16 driver_status;
        __s32 resid;                  
        __u32 duration;     
        __u32 info;         
} sg_io_hdr_t32;

struct usbdevfs_ctrltransfer32 {
        __u8 requesttype;
        __u8 request;
        __u16 value;
        __u16 index;
        __u16 length;
        __u32 timeout;
        __u32 data;
};

struct usbdevfs_bulktransfer32 {
        __u32 ep;
        __u32 len;
        __u32 timeout;
        __u32 data;
};

struct usbdevfs_disconnectsignal32 {
        __u32 signr;
        __u32 context;
};

struct video_tuner32 {
        __s32 tuner;
        char name[32];
        __u32 rangelow, rangehigh;
        __u32 flags;
        __u16 mode;   
        __u16 signal; 
};

struct video_clip32 {
        __s32 x,y;
        __s32 width, height;
        __u32 next;
};

struct video_window32 {
        __u32 x,y;              
        __u32 width,height;     
        __u32 chromakey;
        __u32 flags;
        __u32 clips;
        __s32 clipcount;
};

struct video_buffer32 {
        __u32 base;
        __s32 height,width;
        __s32 depth;
        __s32 bytesperline;
};

struct atmif_sioc32 {
        __s32 number;
        __s32 length;
        __u32 arg;
};

struct atm_iobuf32 {
        __s32 length;
        __u32 buffer;
};



#define VFAT_IOCTL_READDIR_BOTH32	_IOR('r', 1, struct dirent32[2])
#define VFAT_IOCTL_READDIR_SHORT32	_IOR('r', 2, struct dirent32[2])
#define REISERFS_IOC_UNPACK32           _IOW(0xCD, 1, __s32)
#define EXT2_IOC_GETFLAGS32             _IOR('f', 1, __s32)
#define EXT2_IOC_SETFLAGS32             _IOW('f', 2, __s32)
#define EXT2_IOC_GETVERSION32           _IOR('v', 1, __s32)
#define EXT2_IOC_SETVERSION32           _IOW('v', 2, __s32)
#define BLKBSZGET32			_IOR(0x12, 112, __s32)
#define BLKBSZSET32			_IOW(0x12, 113, __s32)
#define BLKGETSIZE64_32			_IOR(0x12, 114, __s32)
#define AUTOFS_IOC_SETTIMEOUT32		_IOWR(0x93, 0x64, __u32)
#define MEMWRITEOOB32			_IOWR('M', 3, struct mtd_oob_buf32)
#define MEMREADOOB32			_IOWR('M', 4, struct mtd_oob_buf32)
#define PPPIOCGIDLE32			_IOR('t', 63, struct ppp_idle32)
#define PPPIOCSPASS32			_IOW('t', 71, struct sock_fprog32)
#define PPPIOCSACTIVE32			_IOW('t', 70, struct sock_fprog32)
#define PPPIOCSCOMPRESS32		_IOW('t', 77, struct ppp_option_data32)
#define MTIOCGET32			_IOR('m', 2, struct mtget32)
#define MTIOCPOS32			_IOR('m', 3, struct mtpos32)
#define MTIOCGETCONFIG32		_IOR('m', 4, struct mtconfiginfo32)
#define MTIOCSETCONFIG32		_IOW('m', 5, struct mtconfiginfo32)
#define USBDEVFS_CONTROL32		_IOWR('U', 0, struct usbdevfs_ctrltransfer32)
#define USBDEVFS_BULK32			_IOWR('U', 2, struct usbdevfs_bulktransfer32)
#define USBDEVFS_REAPURB32		_IOW('U', 12, __u32)
#define USBDEVFS_REAPURBNDELAY32	_IOW('U', 13, __u32)
#define USBDEVFS_DISCSIGNAL32		_IOR('U', 14, struct usbdevfs_disconnectsignal32)
#define VIDIOCGTUNER32			_IOWR('v',4,struct video_tuner32)  
#define VIDIOCSTUNER32			_IOW('v',5,struct video_tuner32)   
#define VIDIOCGWIN32			_IOR('v',9, struct video_window32) 
#define VIDIOCSWIN32			_IOW('v',10, struct video_window32)
#define VIDIOCGFBUF32			_IOR('v',11, struct video_buffer32)
#define VIDIOCSFBUF32			_IOW('v',12, struct video_buffer32)
#define VIDIOCGFREQ32			_IOR('v',14, __u32)      
#define VIDIOCSFREQ32			_IOW('v',15, __u32)      
#define ATM_GETLINKRATE32		_IOW('a',ATMIOC_ITF+1,struct atmif_sioc32)
#define ATM_GETNAMES32			_IOW('a',ATMIOC_ITF+3,struct atm_iobuf32)
#define ATM_GETTYPE32			_IOW('a',ATMIOC_ITF+4,struct atmif_sioc32)
#define ATM_GETESI32			_IOW('a',ATMIOC_ITF+5,struct atmif_sioc32)
#define ATM_GETADDR32			_IOW('a',ATMIOC_ITF+6,struct atmif_sioc32)
#define ATM_RSTADDR32			_IOW('a',ATMIOC_ITF+7,struct atmif_sioc32)
#define ATM_ADDADDR32			_IOW('a',ATMIOC_ITF+8,struct atmif_sioc32)
#define ATM_DELADDR32			_IOW('a',ATMIOC_ITF+9,struct atmif_sioc32)
#define ATM_GETCIRANGE32		_IOW('a',ATMIOC_ITF+10,struct atmif_sioc32)
#define ATM_SETCIRANGE32		_IOW('a',ATMIOC_ITF+11,struct atmif_sioc32)
#define ATM_SETESI32			_IOW('a',ATMIOC_ITF+12,struct atmif_sioc32)
#define ATM_SETESIF32			_IOW('a',ATMIOC_ITF+13,struct atmif_sioc32)
#define ATM_GETSTAT32			_IOW('a',ATMIOC_SARCOM+0,struct atmif_sioc32)
#define ATM_GETSTATZ32			_IOW('a',ATMIOC_SARCOM+1,struct atmif_sioc32)
#define ATM_GETLOOP32			_IOW('a',ATMIOC_SARCOM+2,struct atmif_sioc32)
#define ATM_SETLOOP32			_IOW('a',ATMIOC_SARCOM+3,struct atmif_sioc32)
#define ATM_QUERYLOOP32			_IOW('a',ATMIOC_SARCOM+4,struct atmif_sioc32)


#define IOCTL_ARRAY_DEFAULT_SIZE 113
static inline unsigned long ioctl_hash_value (unsigned cmd)
{
	return (cmd % IOCTL_ARRAY_DEFAULT_SIZE);
}

#define UNUSED_IOCTL	1

