/*
 * ia32_ipc.c
 *
 * wrapper functions for syscalls that deal with ipc. 
 * common to both internal and external wrappers.
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Gordon Jin <gordon.jin@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>
#include "user32.h"


long HIDDEN remove_shm_region (unsigned long addr);

#define PERM_CONV(a)                                       \
	id64_ds32.a##_perm.key = id64_ds.a##_perm.key;     \
	id64_ds32.a##_perm.uid = id64_ds.a##_perm.uid;     \
	id64_ds32.a##_perm.gid = id64_ds.a##_perm.gid;     \
	id64_ds32.a##_perm.cuid = id64_ds.a##_perm.cuid;   \
	id64_ds32.a##_perm.cgid = id64_ds.a##_perm.cgid;   \
	id64_ds32.a##_perm.mode = id64_ds.a##_perm.mode;   \
	id64_ds32.a##_perm.seq = id64_ds.a##_perm.seq;     

#define OLD_PERM_CONV(a)                                   \
	id_ds32.a##_perm.key = id64_ds.a##_perm.key;       \
	id_ds32.a##_perm.uid = id64_ds.a##_perm.uid;       \
	id_ds32.a##_perm.gid = id64_ds.a##_perm.gid;       \
	id_ds32.a##_perm.cuid = id64_ds.a##_perm.cuid;     \
	id_ds32.a##_perm.cgid = id64_ds.a##_perm.cgid;     \
	id_ds32.a##_perm.mode = id64_ds.a##_perm.mode;     \
	id_ds32.a##_perm.seq = id64_ds.a##_perm.seq;     



struct msgbuf32 {
	__s32 mtype; 
	char  mtext[1]; 
};

struct ipc_perm32 {
	key_t      key;
	__uid_t32  uid;
	__gid_t32  gid;
	__uid_t32  cuid;
	__gid_t32  cgid;
	__mode_t32 mode;
	unsigned short seq;
};

struct ipc64_perm32 {
	key_t key;
	__uid32_t32 uid;
	__gid32_t32 gid;
	__uid32_t32 cuid;
	__gid32_t32 cgid;
	__mode_t32  mode;
	unsigned short __pad1;
	unsigned short seq;
	unsigned short __pad2;
	unsigned int unused1;
	unsigned int unused2;
};

struct semid_ds32 {
	struct ipc_perm32 sem_perm;    /* permissions .. see ipc.h */
	__time_t32 sem_otime;          /* last semop time */
	__time_t32 sem_ctime;          /* last change time */
	__u32 sem_base;                /* ptr to first semaphore in array */
	__u32 sem_pending;             /* pending operations to be processed */
	__u32 sem_pending_last;        /* last pending operation */
	__u32 undo;                    /* undo requests on this array */
	unsigned short  sem_nsems;     /* no. of semaphores in array */
};

struct semid64_ds32 {
	struct ipc64_perm32 sem_perm;
	__time_t32 sem_otime;
	unsigned int __unused1;
	__time_t32 sem_ctime;
	unsigned int __unused2;
	unsigned int sem_nsems;
	unsigned int __unused3;
	unsigned int __unused4;
};

struct msqid_ds32 {
	struct ipc_perm32 msg_perm;
	__u32 msg_first;
	__u32 msg_last;
	__time_t32 msg_stime;
	__time_t32 msg_rtime;
	__time_t32 msg_ctime;
	__u32 wwait;
	__u32 rwait;
	unsigned short msg_cbytes;
	unsigned short msg_qnum;
	unsigned short msg_qbytes;
	__ipc_pid_t32 msg_lspid;
	__ipc_pid_t32 msg_lrpid;
};

struct msqid64_ds32 {
	struct ipc64_perm32 msg_perm;
	__time_t32 msg_stime;
	unsigned int __unused1;
	__time_t32 msg_rtime;
	unsigned int __unused2;
	__time_t32 msg_ctime;
	unsigned int __unused3;
	unsigned int msg_cbytes;
	unsigned int msg_qnum;
	unsigned int msg_qbytes;
	__pid_t32 msg_lspid;
	__pid_t32 msg_lrpid;
	unsigned int __unused4;
	unsigned int __unused5;
};

struct shmid_ds32 {
	struct ipc_perm32 shm_perm;
	int shm_segsz;
	__time_t32 shm_atime;
	__time_t32 shm_dtime;
	__time_t32 shm_ctime;
	__ipc_pid_t32 shm_cpid;
	__ipc_pid_t32 shm_lpid;
	unsigned short shm_nattch;
};

struct shmid64_ds32 {
	struct ipc64_perm32 shm_perm;
	__size_t32 shm_segsz;
	__time_t32 shm_atime;
	unsigned int __unused1;
	__time_t32 shm_dtime;
	unsigned int __unused2;
	__time_t32 shm_ctime;
	unsigned int __unused3;
	__pid_t32 shm_cpid;
	__pid_t32 shm_lpid;
	unsigned int shm_nattch;
	unsigned int __unused4;
	unsigned int __unused5;
};

struct shminfo64_32 {
	unsigned int shmmax;
	unsigned int shmmin;
	unsigned int shmmni;
	unsigned int shmseg;
	unsigned int shmall;
	unsigned int __unused1;
	unsigned int __unused2;
	unsigned int __unused3;
	unsigned int __unused4;
};

struct shm_info32 {
	int used_ids;
	__u32 shm_tot; 
	__u32 shm_rss; 
	__u32 shm_swp;
	__u32 swap_attempts;
	__u32  swap_successes;
};

struct ipc_kludge32 {
	__u32 msgp;
	__s32 msgtyp;
};

#define IS_IPC_64(cmd)                      \
({                                          \
	int _r = (cmd & IPC_64) ? 1 : 0;    \
	if (_r)                             \
		cmd &= ~IPC_64;             \
	_r;                                 \
})
 

static inline int ipc_parse_version32 (int *cmd)
{
	if (*cmd & IPC_64) {
		*cmd ^= IPC_64;
		return IPC_64;
	} else {
		return IPC_OLD;
	}
}

static int
semctl32 (int semid, int semnum, int cmd, void *arg)
{
	int ipc64;
	int newcmd;
	int retval;

	union semun semun64;

	__u32 pad;

	struct semid64_ds id64_ds;

 	newcmd = cmd;	
	ipc64 = IS_IPC_64 (newcmd);

	if (!arg) {
		retval = -EINVAL;
		goto out;
	}

	if (g_safe_memcpy (&pad, arg, sizeof(pad)) < 0) {
		retval = -EFAULT;
		goto out;
	}


	switch (newcmd) {
	    case IPC_RMID:
	    case IPC_SET:
	    case IPC_INFO:
	    case GETPID:
	    case GETVAL:
	    case GETALL:
	    case GETNCNT:
	    case GETZCNT:
	    case SETALL:
	    case SEM_INFO:
		semun64.__pad = I2P (pad);
		retval = SYSCALL (semctl, (semid, semnum, newcmd, semun64));
		break;

	    case SETVAL:
		semun64.val = (int)pad;
		retval = SYSCALL (semctl, (semid, semnum, newcmd, semun64));
		break;

	    case IPC_STAT:
	    case SEM_STAT:
		semun64.__pad = &id64_ds;
		retval = SYSCALL (semctl, (semid, semnum, newcmd, semun64));

		if (IS_SYS_ERROR (retval)) {
			break;
		}
		
		ASSERT_HIGH_ZERO(id64_ds.sem_otime);
		ASSERT_HIGH_ZERO(id64_ds.sem_ctime);
		ASSERT_HIGH_ZERO(id64_ds.sem_nsems);

		if (ipc64) {
			struct semid64_ds32 id64_ds32;

			PERM_CONV(sem);

			id64_ds32.sem_otime = id64_ds.sem_otime;
			id64_ds32.sem_ctime = id64_ds.sem_ctime;
			id64_ds32.sem_nsems = id64_ds.sem_nsems;
			if (g_safe_memcpy (I2P(pad), &id64_ds32, sizeof(id64_ds32)) < 0) {
				retval = -EFAULT;
				break;
			}
		} else {
			struct semid_ds32 id_ds32;

			OLD_PERM_CONV(sem);

			id_ds32.sem_otime = id64_ds.sem_otime;
			id_ds32.sem_ctime = id64_ds.sem_ctime;
			id_ds32.sem_nsems = id64_ds.sem_nsems;
			if (g_safe_memcpy (I2P(pad), &id_ds32, sizeof(id_ds32)) < 0) {
				retval = -EFAULT;
				break;
			}
		}
		break;

	    default:
		retval = -EINVAL;
		break;
	}

out:
	return retval;
}

static int
do_msgsnd32 (int msgid, struct msgbuf32 *msg32, __u32 msgsz, int msgflag)
{
	struct msgbuf *msg64;
	int    retval;
	int    mtype32;

	if (msgsz > MSGMAX) {
		return -EINVAL;
	}

	msg64 = (struct msgbuf *)malloc (msgsz + sizeof(long));
	if (!msg64) {
		return -ENOMEM;
	}

	if (g_safe_memcpy (&mtype32, &msg32->mtype, sizeof(msg32->mtype)) < 0) {
		retval = -EFAULT;
		goto out;
	}
	msg64->mtype = (long)mtype32;

	if (g_safe_memcpy (&msg64->mtext, &msg32->mtext, msgsz) < 0) {
		retval = -EFAULT;
		goto out;
	}
	retval = LONG_SYSCALL4 (msgsnd, msgid, msg64, msgsz, msgflag);

out:
	free (msg64);
	return retval;
}

static int
do_msgrcv32 (int msgid, struct msgbuf32 *msg32, __u32 msgsz, int msgtyp, int msgflag)
{
	struct msgbuf *msg64;
	int    retval;
	int    mtype32;

	if (msgsz > MSGMAX) {
		return -EINVAL;
	}

	msg64 = (struct msgbuf *)malloc (msgsz + sizeof(long));
	if (!msg64) {
		return -ENOMEM;
	}
	
	retval = LONG_SYSCALL5 (msgrcv, msgid, msg64, msgsz, msgtyp, msgflag);
	if (retval < 0) {
		goto out;
	}

	ASSERT_HIGH_ZERO(msg64->mtype);
	mtype32 = (int)msg64->mtype;

	if (g_safe_memcpy (&msg32->mtype, &mtype32, sizeof(msg32->mtype)) < 0) {
		retval = -EFAULT;
		goto out;
	}
	
	if (g_safe_memcpy (&msg32->mtext, &msg64->mtext, retval) < 0) {
		retval = -EFAULT;
	}
out:
	free (msg64);
	return retval;
}

static int
msgctl32 (int msqid, int cmd, struct msqid64_ds32 *buf)
{
	int    retval;
	int    newcmd;
	int    ipc64;

	/* 64-bit kernel data structure */
	struct msqid64_ds   id64_ds;

	/* IA-32 kernel data structure */
	struct msqid64_ds32 id64_ds32;
	struct msqid_ds32   id_ds32;

	newcmd = cmd;

	ipc64 = IS_IPC_64 (newcmd);

	switch (newcmd) {
	    case IPC_INFO:
	    case MSG_INFO:
		/* for IPC_INFO/MSG_INFO, buf is pointer to msginfo, it's same
		   on IA-32 and Itanium Linux kernel */
		retval = SYSCALL (msgctl, (msqid, newcmd, (void *)buf));
		break;
	    case IPC_RMID:
		/* buf is not used by the kernel */
		retval = SYSCALL (msgctl, (msqid, newcmd, (void *)buf));
		break;
		
	    case IPC_SET:
		/* for IPC_SET only uid, gid, mode in msg_perm and msg_qbytes  
		   are used by kernel, so we do only necessary conversion */
		if (ipc64) {
			if (g_safe_memcpy (&id64_ds32, buf, sizeof(id64_ds32)) < 0) {
				retval = -EFAULT;
				break;
			}
		} else {
			if (g_safe_memcpy (&id_ds32, buf, sizeof(id_ds32)) < 0) {
				retval = -EFAULT;
				break;
			}
		}

		id64_ds.msg_perm.uid = ipc64?
				       id64_ds32.msg_perm.uid : id_ds32.msg_perm.uid;
		id64_ds.msg_perm.gid = ipc64?
				       id64_ds32.msg_perm.gid : id_ds32.msg_perm.gid;

		id64_ds.msg_perm.mode = ipc64?
				        id64_ds32.msg_perm.mode : id_ds32.msg_perm.mode;

		id64_ds.msg_qbytes = ipc64? 
				     id64_ds32.msg_qbytes : id_ds32.msg_qbytes;

		retval = SYSCALL (msgctl, (msqid, newcmd, &id64_ds));
		break;

	    case IPC_STAT:
	    case MSG_STAT:
		
		retval = SYSCALL (msgctl, (msqid, newcmd, (void *)&id64_ds));
	
		if (ipc64) {

			PERM_CONV(msg);

			id64_ds32.msg_stime = id64_ds.msg_stime;
			id64_ds32.msg_rtime = id64_ds.msg_rtime;
			id64_ds32.msg_ctime = id64_ds.msg_ctime;
			id64_ds32.msg_cbytes = id64_ds.msg_cbytes;
			id64_ds32.msg_qnum = id64_ds.msg_qnum;
			id64_ds32.msg_qbytes = id64_ds.msg_qbytes;
			id64_ds32.msg_lspid = id64_ds.msg_lspid;
			id64_ds32.msg_lrpid = id64_ds.msg_lrpid;

			if (g_safe_memcpy (buf, &id64_ds32, sizeof(id64_ds32)) < 0) {
				retval = -EFAULT;
			}
		} else {
			OLD_PERM_CONV(msg);

			id_ds32.msg_stime = id64_ds.msg_stime;
			id_ds32.msg_rtime = id64_ds.msg_rtime;
			id_ds32.msg_ctime = id64_ds.msg_ctime;
			id_ds32.msg_cbytes = id64_ds.msg_cbytes;
			id_ds32.msg_qnum = id64_ds.msg_qnum;
			id_ds32.msg_qbytes = id64_ds.msg_qbytes;
			id_ds32.msg_lspid = id64_ds.msg_lspid;
			id_ds32.msg_lrpid = id64_ds.msg_lrpid;
			if (g_safe_memcpy (buf, &id_ds32, sizeof(id_ds32)) < 0) {
				retval = -EFAULT;
			}
		}
		break;
	    default:
		retval = -EINVAL;
		break;
	}

	return retval;
}


static int
shmctl32 (int shmid, int cmd, struct shmid64_ds32 *buf)
{
	int newcmd = cmd;
	int retval;
	int ipc64;

		
	/* 64-bit kernel data structure */
	struct shmid64_ds      id64_ds;

	/* IA-32 kernel data structure */
	struct shmid64_ds32    id64_ds32;    /* for IPC_64 */
	struct shmid_ds32      id_ds32;      /* for backward compatibility support */

	ipc64 = IS_IPC_64 (newcmd);

	switch (newcmd) {
	    case IPC_INFO:
	    {
		/* IA-32 kerenel data structure */
		struct shminfo64_32 info64_32; /* for IPC_64 */
		struct shminfo      info_32;   /* for backward compatibility support */

		/* 64-bit kernel data structure */
		struct shminfo64    info64; 

		retval = SYSCALL (shmctl, (shmid, newcmd, &info64));

		if (retval < 0) {
			break;
		}
		if (ipc64) {
			memset (&info64_32, 0, sizeof(info64_32));
			info64_32.shmmax = info64.shmmax;
			info64_32.shmmin = info64.shmmin;
			info64_32.shmmni = info64.shmmni;
			info64_32.shmseg = info64.shmseg;
			info64_32.shmall = info64.shmall;
	
			if (g_safe_memcpy (buf, &info64_32, sizeof(info64_32)) < 0) {
				retval = -EFAULT;
				break;
			}

		} else {
			info_32.shmmax = info64.shmmax;
			info_32.shmmin = info64.shmmin;
			info_32.shmmni = info64.shmmni;
			info_32.shmseg = info64.shmseg;
			info_32.shmall = info64.shmall;
	
			if (g_safe_memcpy (buf, &info_32, sizeof(info_32)) < 0) {
				retval = -EFAULT;
				break;
			}
		}
		break;

	    }
	    case SHM_INFO:
	    {
		struct shm_info   si64;   /* 64-bit kernel data structure */
		struct shm_info32 si32;   /* IA-32 kernel data structure */

		retval = SYSCALL (shmctl, (shmid, newcmd, &si64));
		if (retval < 0) {
			break;
		}
	
		si32.used_ids = si64.used_ids;

		ASSERT_HIGH_ZERO (si64.shm_tot);
		si32.shm_tot = (__u32)si64.shm_tot;

		ASSERT_HIGH_ZERO (si64.shm_rss);
		si32.shm_rss = (__u32)si64.shm_rss;

		ASSERT_HIGH_ZERO (si64.shm_swp);
		si32.shm_swp = (__u32)si64.shm_swp;

		ASSERT_HIGH_ZERO (si64.swap_attempts);
		si32.swap_attempts = (__u32)si64.swap_attempts;

		ASSERT_HIGH_ZERO (si64.swap_successes);
		si32.swap_successes = (__u32)si64.swap_successes;
		if (g_safe_memcpy (buf, &si32, sizeof(si32)) < 0) {
			retval = -EFAULT;
		}
	
		break;
	    }
	    case IPC_RMID:
	    case SHM_LOCK:
	    case SHM_UNLOCK:
		/* for above command, the third parameter is not used */
		retval = SYSCALL (shmctl, (shmid, newcmd, &id64_ds));
		break;

	    case IPC_SET:
		/* for IPC_SET only uid, gid and mode in shm_perm are used by 
		   kernel, so we do only necessary conversion */
		if (ipc64) {
			if (g_safe_memcpy (&id64_ds32, buf, sizeof(id64_ds32)) < 0) {
				retval = -EFAULT;
				break;
			}
		} else {
			if (g_safe_memcpy (&id_ds32, buf, sizeof(id_ds32)) < 0) {
				retval = -EFAULT;
				break;
			}
		}
		id64_ds.shm_perm.uid  = ipc64 ? 
				 	id64_ds32.shm_perm.uid : id_ds32.shm_perm.uid;
		id64_ds.shm_perm.gid  = ipc64 ? 
					id64_ds32.shm_perm.gid : id_ds32.shm_perm.gid;
		id64_ds.shm_perm.mode = ipc64 ? 
				   	id64_ds32.shm_perm.mode : id_ds32.shm_perm.mode;
		retval = SYSCALL (shmctl, (shmid, newcmd, &id64_ds));
		break;

	    case IPC_STAT:
	    case SHM_STAT:
		retval = SYSCALL (shmctl, (shmid, newcmd, &id64_ds));
		if (retval < 0) {
			break;
		}
		
		ASSERT_HIGH_ZERO(id64_ds.shm_segsz);
		ASSERT_HIGH_ZERO(id64_ds.shm_atime);
		ASSERT_HIGH_ZERO(id64_ds.shm_dtime);
		ASSERT_HIGH_ZERO(id64_ds.shm_ctime);

		if (ipc64) {

			PERM_CONV(shm);

			id64_ds32.shm_segsz = id64_ds.shm_segsz;
			id64_ds32.shm_atime = id64_ds.shm_atime;
			id64_ds32.shm_dtime = id64_ds.shm_dtime;
			id64_ds32.shm_ctime = id64_ds.shm_ctime;
			id64_ds32.shm_cpid = id64_ds.shm_cpid;
			id64_ds32.shm_lpid = id64_ds.shm_lpid;

			ASSERT_HIGH_ZERO(id64_ds.shm_nattch);
			id64_ds32.shm_nattch = (int)id64_ds.shm_nattch;

			if (g_safe_memcpy (buf, &id64_ds32, sizeof(id64_ds32)) < 0) {
				retval = -EFAULT;
			}
		} else {
			OLD_PERM_CONV(shm);

			id_ds32.shm_atime = id64_ds.shm_atime;
			id_ds32.shm_dtime = id64_ds.shm_dtime;
			id_ds32.shm_ctime = id64_ds.shm_ctime;
			id_ds32.shm_segsz = id64_ds.shm_segsz;
			id_ds32.shm_cpid = id64_ds.shm_cpid;
			id_ds32.shm_lpid = id64_ds.shm_lpid;

			assert ((id64_ds.shm_nattch >> 48) == 0);
			id_ds32.shm_nattch = id64_ds.shm_nattch;

			if (g_safe_memcpy (buf, &id_ds32, sizeof (id_ds32)) < 0) {
				retval = -EFAULT;
			}
		}
		break;

	    default:
		retval = -EINVAL;
		break;

	}
	return retval;
}


#define IPCOP_semop	 1
#define IPCOP_semget	 2
#define IPCOP_semctl	 3
#define IPCOP_semtimedop 4
#define IPCOP_msgsnd	11
#define IPCOP_msgrcv	12
#define IPCOP_msgget	13
#define IPCOP_msgctl	14
#define IPCOP_shmat	21
#define IPCOP_shmdt	22
#define IPCOP_shmget	23
#define IPCOP_shmctl	24

/* sys_ipc is a common system call entry for system V IPC including semaphore, 
   message and shared memory. So the meaning of the parameter varies with "call" */
long HIDDEN
sys32_ipc (__u32 call, int first, int second, int third, __u32 ptr, __u32 fifth)
{
	int   old_version;
	__u32 newcall;
	int   retval;

        /* low part of call specifies the operation, high part is used backward 
           compatibility - if it's not 0, special treatment needed for 
	   shmat and msgrsv */
	old_version = ((call & 0xffff0000) == 0? 0 : 1); 
	newcall = call & 0xffff; 

	switch (newcall) {
	    case IPCOP_semop:
		/* first: semaphore id, ptr: semaphore buffer, 
		   second: number of operations */
		retval = LONG_SYSCALL3 (semop, first, 
				        (struct sembuf *)UL(ptr), 
					second);
		break;
	    case IPCOP_semget:
		/* first: key, 
		   second: number of semaphores in the semaphore set, 
		   third: semflag */
		retval = SYSCALL (semget, (first, second, third));
		break;
	    case IPCOP_semctl:
		/* first: semaphore id, second: semaphores number, third: cmd
		   ptr: pointer to union semun */
		if (ptr == 0) {
			retval = -EINVAL;
			break;
		}
		retval = semctl32 (first, second, third, (void *)UL(ptr));
		break;
	    case IPCOP_semtimedop:
	    {
		/* first: semaphore id, ptr: semaphore buffer, 
		   second: number of operations,
		   fifth: timeout */
		struct timespec32 ts32;
		struct timespec   ts64;

		if (IS_VALID_PTR(I2P(fifth))) {
			if (g_safe_memcpy (&ts32, I2P(fifth), sizeof(ts32)) < 0) {
				retval = -EFAULT;
				break;
			} 
			ts64.tv_sec = ts32.tv_sec;
			ts64.tv_nsec = ts32.tv_nsec;
		}

		retval = LONG_SYSCALL4 (semtimedop, first, 
				        (struct sembuf *)UL(ptr), 
					second, 
					VALID_PTR(I2P(fifth), &ts64));
		break;
	
	    }
	    case IPCOP_msgsnd:
		/* 
   		   first: message id 
   		   ptr: pointer of original 32-bit message buffer
   		   second: message size
		   third: message flag
 		 */
		retval = do_msgsnd32 (first, 
				      (struct msgbuf32 *)UL(ptr),
				      second, third);
		break;
	    case IPCOP_msgrcv:
	    {
		/* 
		   for new version: 
   		   first: message id 
   		   second: message size
		   third: message flag
   		   ptr: pointer to ipc_kludge32, used due to the parameter number
			limitation 

		   for old version:
		   ptr:	pointer to original 32-bit message buffer
		   fifth: message type
 		 */
		int msgtyp;
		struct msgbuf32 *msg32;

		if (!old_version) {
			struct ipc_kludge32 tmp;

			if (ptr == 0) {
				retval = -EINVAL;
				break;
			}
			retval = g_safe_memcpy (&tmp, I2P(ptr), 
						sizeof(struct ipc_kludge32));
			if (retval != 0) {
				retval = -EFAULT;
				break;
			}
			msgtyp = tmp.msgtyp;
			msg32 = (struct msgbuf32 *)I2P(tmp.msgp);
		} else {
			msgtyp = fifth;
			msg32 = (struct msgbuf32 *)I2P(ptr);
		}

		retval = do_msgrcv32 (first, msg32, second, msgtyp, third);
		break;
	    }
	    case IPCOP_msgget:
	    	/* first: key, second: message flag */
		retval = SYSCALL (msgget, ((key_t) first, second));
		break;
	    case IPCOP_msgctl:
		/* first: message queue id, second: cmd, ptr: pointer to 
  		   32-bit msqid_ds */
		retval = msgctl32 (first, second, (struct msqid64_ds32 *)UL(ptr));
		break;
	    case IPCOP_shmat:
		/* first: shared memory id, second: shared memory flag, 
		   third: returned address, ptr: prefered address */ 
		if (old_version) {
			retval = -EINVAL;
			break;
		}
		retval = shmat32 (first, second, third, (void *)UL(ptr));
		break;
	    case IPCOP_shmdt:
		retval = SYSCALL (shmdt, ((char *)UL(ptr)));
		if (!IS_SYS_ERROR (retval)) {
			retval = remove_shm_region (UL(ptr));
		}
		break;
	    case IPCOP_shmget:
		retval = SYSCALL (shmget, (first, second, third));
		break;
	    case IPCOP_shmctl:
		/* first: shared memory id, second: cmd, ptr: pointer to 
		   shmid32_ds */
		retval = shmctl32 (first, second, 
				   (struct shmid64_ds32 *)UL(ptr));
		break;
	    default:
		retval = -EINVAL;
		break;
	}

	return retval;
}

long HIDDEN
sys32_sendfile (int out_fd, int in_fd, __u32 poff32, unsigned int count)
{
	extern long sendfile (int, int, off_t *, size_t);
	int    ret;
	off_t  off64 = 0;
	int    off32;

	if (IS_VALID_PTR(I2P(poff32))) {
		if (g_safe_memcpy (&off32, I2P(poff32), sizeof(int)) < 0) {
			return -EFAULT;
		}
		off64 = (off_t)off32;
	}

	ret = SYSCALL (sendfile, (out_fd, in_fd, 
		       VALID_PTR(I2P(poff32), &off64), count));

	ASSERT_HIGH_ZERO(off64);
	off32 = (int)off64;
	if ((ret == 0) && (poff32 != 0)) {
		if (g_safe_memcpy (I2P(poff32), &off32, sizeof(int)) < 0) {
			return -EFAULT;
		}
	}

	return ret;
}

