/*
 * ia32_machine.c
 *
 * wrapper functions for syscalls that deal with machine. 
 * common to both internal and external wrappers.
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Kevin Tian <kevin.tian@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>

#include "ia32ss_common.h"
#include "user32.h"

int __sysctl(struct __sysctl_args *args);

long HIDDEN
sys32_sysinfo (__u32 info)
{
	struct sysinfo info64;
	struct sysinfo32 info32;
	int retval;

	if ((retval = SYSCALL(sysinfo, (&info64))) < 0)
		return retval;

	/* Check to see if any memory value is too large, or scale down if needed */
	if ((info64.totalram >> 32) || (info64.totalswap >> 32)) {
		int bitcount = 0;
		__u32 page_size = SYSCALL (getpagesize, ());

		while (info64.mem_unit < page_size) {
			info64.mem_unit <<= 1;
			bitcount++;
		}

		info64.totalram >>= bitcount;
		info64.freeram >>= bitcount;
		info64.sharedram >>= bitcount;
		info64.bufferram >>= bitcount;
		info64.totalswap >>= bitcount;
		info64.freeswap >>= bitcount;
		info64.totalhigh >>= bitcount;
		info64.freehigh >>= bitcount;
	}

	info32.uptime = (__s32)info64.uptime;
	info32.loads[0] = (__u32)info64.loads[0];
	info32.loads[1] = (__u32)info64.loads[1];
	info32.loads[2] = (__u32)info64.loads[2];
	info32.totalram = (__u32)info64.totalram;
	info32.freeram = (__u32)info64.freeram;
	info32.sharedram = (__u32)info64.sharedram;
	info32.bufferram = (__u32)info64.bufferram;
	info32.totalswap = (__u32)info64.totalswap;
	info32.freeswap = (__u32)info64.freeswap;
	info32.procs = info64.procs;
	info32.totalhigh = (__u32)info64.totalhigh;
	info32.freehigh = (__u32)info64.freehigh;
	info32.mem_unit = info64.mem_unit;

	if (g_safe_memcpy(I2P(info), &info32, sizeof(info32)) < 0)
		retval = -EFAULT;

	return retval;
}

struct sysctl_args32 {
	__u32 name;
	__s32 nlen;
	__u32 oldval;
	__u32 oldlenp;
	__u32 newval;
	__u32 newlen;
	__u32 __unused[4];
};

long HIDDEN
sys32_sysctl (__u32 args)
{
	struct sysctl_args32 arg32;
	struct __sysctl_args arg64;
	__u64 olen64 = 0;
	int v, retval;

	if (g_safe_memcpy((void*)&arg32, I2P(args), sizeof(arg32)) < 0)
		return -EFAULT;

	arg64.oldval = I2P(arg32.oldval);
	v = IS_VALID_PTR(arg64.oldval);
	if (v) {
		__u32 olen32;
		if (g_safe_memcpy(&olen32, I2P(arg32.oldlenp), sizeof(__u32)) < 0)
			return -EFAULT;
		olen64 = (__u64)olen32;
		arg64.oldlenp = &olen64;
	} else
		arg64.oldlenp = NULL;
	
	arg64.name = I2P(arg32.name);
	arg64.nlen = arg32.nlen;
	arg64.newval = I2P(arg32.newval);
	arg64.newlen = (__u64)arg32.newlen;
	memset(&arg64.__unused, 0, sizeof(arg64.__unused));

	retval = SYSCALL (_sysctl, (&arg64));

	if (!retval && v) {
		*((__u32*)I2P(arg32.oldlenp)) = (__u32)olen64;
	}
	return retval;
}

long HIDDEN 
sys32_newuname (__u32 name)
{
	struct utsname *namep;
	int retval;

	namep = (struct utsname*) I2P(name);
	retval = SYSCALL (newuname, (namep));

	if (!retval) {
		if (g_safe_memcpy(namep->machine, "i686\0\0\0", 8) < 0)
			return -EFAULT;
	}
	return retval;	
}
