/*
 *  ia32_signal.c
 *
 * wrapper functions for syscalls related with signal.
 * common for both internal and external wrappers.
 *  
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Kevin Tian <kevin.tian@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

/*
#define __USE_POSIX
#define __USE_POSIX199309
*/

#include <ia32x_utils.h>
#include "user32.h"



long HIDDEN
sys32_rt_sigtimedwait (const __u32 set, __u32 info, const __u32 timeout, __u32 ss_size)
{
	struct timespec32 ts32;
	struct timespec ts64;
	siginfo_t si64;
	long   retval;
        
	if (IS_VALID_PTR(I2P(timeout))) {
		if (g_safe_memcpy (&ts32, I2P(timeout), sizeof(ts32)) < 0) {
                        return -EFAULT;
		}
		ts64.tv_sec = ts32.tv_sec;
		ts64.tv_nsec = ts32.tv_nsec;
	}
        
	/* treated as a long syscall */
	retval = LONG_SYSCALL4 (rt_sigtimedwait, 
			        set, 
			        VALID_PTR(I2P(info), &si64), 
			        VALID_PTR(I2P(timeout), &ts64), 
			        ss_size);

	if ((retval >= 0) && IS_VALID_PTR(I2P(info))) {
		siginfo_t32 si32;               

		si32.si_signo = si64.si_signo;
		si32.si_errno = si64.si_errno;
		si32.si_code = si64.si_code;

                if (si32.si_code < 0) {
                        memcpy (&si32._sifields._pad, &si64._sifields._pad, SI_PAD_SIZE);
                } else {
                        switch (si32.si_signo) {
			    case SIGPOLL:
				si32._sifields._sigpoll._band = (__u32)si64.si_band;
                                si32._sifields._sigpoll._fd = si64.si_fd;
				break;
			    case SIGCHLD:
				si32._sifields._sigchld._status = si64.si_status;
				si32._sifields._sigchld._utime = (__u32)si64.si_utime;
				si32._sifields._sigchld._stime = (__u32)si64.si_stime;
				/* fall through */
			    default:
				si32._sifields._sigchld._pid = (int)si64.si_pid;
				si32._sifields._sigchld._uid = (int)si64.si_pid;
				break;
			}
		}

		if (g_safe_memcpy (I2P(info), &si32, sizeof(si32)) < 0)
			return -EFAULT;
	}
	return retval;
}


long HIDDEN 
sys32_rt_sigqueueinfo (int pid, int sig, __u32 info)
{
	siginfo_t si64;
	siginfo_t32 si32;

	if (g_safe_memcpy(&si32, I2P(info), sizeof(si32)) < 0)
		return -EFAULT;

	if (si32.si_code >= 0)
		return -EPERM;

	si64.si_signo = si32.si_signo;
	si64.si_errno = si32.si_errno;
	si64.si_code = si32.si_code;

	memcpy (&si64._sifields._pad, &si32._sifields._pad, SI_PAD_SIZE);

	return (int)SYSCALL (rt_sigqueueinfo, (pid, sig, &si64));
}

