/* 
 * ia32ss_common.h
 *
 * contains declarations needed for both external and internal wrappers
 *
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/



#ifndef _IA32SS_H
#define _IA32SS_H

#include <ia32x_utils.h>

#define HIDDEN __attribute__ ((visibility ("hidden")))

extern long HIDDEN (*g_safe_memcpy)(void *, void *, unsigned long);

struct legacy_mmap_arg_struct {
	unsigned int addr;
	unsigned int len;
	unsigned int prot;
	unsigned int flags;
	unsigned int fd;
	unsigned int offset;
};

extern __thread long thread_personality  __attribute__ ((tls_model ("initial-exec")));
static inline unsigned int
convert_prot64_to_prot32 (unsigned int prot)
{
	unsigned int retval = PROT_NONE;

	assert (PROT_NONE == 0);

	if (prot & PROT_READ) {
		retval |= PROT_READ;
	}
	
	if (prot & PROT_WRITE) {
		retval |= PROT_WRITE | PROT_READ;
	}
	
	if (prot & PROT_EXEC) {
		retval |= PROT_EXEC | PROT_READ;
	} else if ((retval & PROT_READ) && RX_PERSON(thread_personality)) {
		retval |= PROT_EXEC;
	}
	
	return retval;
}

#define LP64_TIMEVAL(_lp64_t,_ilp32_t)                              \
  ({                                                                \
    int retval = 0;                                                 \
    struct timeval32 tv32;                                          \
	if (g_safe_memcpy(&tv32, &_ilp32_t, sizeof(tv32)) < 0) {    \
		return -EFAULT;                                     \
	}                                                           \
    (_lp64_t).tv_sec = (long long)((tv32).tv_sec);                  \
    (_lp64_t).tv_usec = (long long)((tv32).tv_usec);                \
	retval;                                                     \
  })

#define ILP32_TIMEVAL(_ilp32_t,_lp64_t)                             \
  ({                                                                \
    int retval = 0;                                                 \
    struct timeval32 tv32;                                          \
	(tv32).tv_sec = (long)((_lp64_t).tv_sec);                   \
    (tv32).tv_usec = (long)((_lp64_t).tv_usec);                     \
	if (g_safe_memcpy(&_ilp32_t, &tv32, sizeof(tv32)) < 0) {    \
		retval = -EFAULT;                                   \
	}                                                           \
	retval;                                                     \
  })

extern long HIDDEN mprotect32_none4k (unsigned int start, unsigned int len, int prot);
extern int HIDDEN shmat32 (int shmid, int shmflag, __u32 ret_ptr, void *shmaddr);

#endif // _IA32SS_H

