/*
 * ia32ss_list.h
 *
 * generic interface for a list
 * the list elements contain the fields - 
 * 	long key - id of the element (must be unique in the list)
 * 	unsigned long data - arbitrary data
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#ifndef _IA32SS_LIST_H
#define _IA32SS_LIST_H

#define HIDDEN __attribute__ ((visibility ("hidden")))

struct list_t;

typedef void * list_handle;

/* initialize a list. must be called before any use of 
 * the list.
 * input:	list - pointer to the list's anchor
 * output:	0 on success, negative error code on error
 */
long HIDDEN list_init (struct list_t ** list);


/* free all resources used by the given list. should be
 * called when the list is no longer used.
 * input:	list - pointer to the list's anchor
 * output:	0 on success, negative error code on error
 */
long HIDDEN list_destroy (struct list_t ** list);


/* insert a new element to the list
 * input:	list - pointer to the list's anchor
 * 			key - the value by which list elements are retrieved
 * 			data - data associated with the key
 * output:	handle to the new element or negative error code on error
 */
list_handle HIDDEN list_insert (struct list_t ** list, long key, unsigned long data);


/* remove an element from the list
 * input:	list - pointer to the list's anchor
 * 			handle - handle of the element to be removed
 * output:	0 on success, negative error code on error
 */
long HIDDEN list_remove (struct list_t ** list, list_handle handle);


/* retrieve a list-element by the key
 * input:	list - pointer to the list's anchor
 * 			key - key of the element
 * output:	returns handle to the requested element on success, or 0 if handle 
 * 			does not exist
 */
list_handle HIDDEN list_retrieve (struct list_t ** list, long key);


/* get the first element in the given list - sorted by key ascending
 * input:	list - pointer to the list's anchor
 * output:	handle of the first element or 0 if the list is empty
 */
list_handle HIDDEN list_get_first (struct list_t ** list);


/* get the next element in the list - sorted by key ascending
 * input:	handle - handle of the element
 * output:	handle of the next element or 0 on end-of-list
 */
list_handle HIDDEN get_element_next (list_handle handle);


/* get the prev element in the list - sorted by key ascending
 * input:	handle - handle of the element
 * output:	handle of the prev element or 0 on first-in-list
 */
list_handle HIDDEN get_element_prev (list_handle handle);


/* return the data of the given list element
 * input:	handle - handle of the element
 * outpu:	the elemen't data
 * remarks: the validity of the handle is not checked
 */
unsigned long HIDDEN get_element_data (list_handle handle);


/* return the key of the given list element
 * input:	handle - handle of the element
 * outpu:	the elemen't key
 * remarks: the validity of the handle is not checked
 */
long HIDDEN get_element_key (list_handle handle);

#endif // _IA32SS_LIST_H
