/*  
 * make_ia32_ioctl_h.c
 * 
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include "ia32_ioctl_prototype.h"
#include <ia32x_utils.h>

#define MAX_HANDLER_NAME_LENGTH		256
#define MAX_IOCTLCMD_NAME_LENGTH	MAX_HANDLER_NAME_LENGTH
#define MAX_BUFF_SIZE	4096

static inline void insert_func_name (char * func_name);
static int insert_ht (unsigned int cmd,
		char * cmd_name,
		unsigned int new_cmd,
		char * new_cmd_name,
		char * handler_name);
static int setup_ioctl_handler_table (void);
struct ioctl_entry * get_ioctl_entry_by_index (long idx);

struct func_name{
	char name[MAX_HANDLER_NAME_LENGTH];
	struct func_name * next;
};
struct func_name * p_func_name_list = NULL;

inline void insert_func_name (char * func_name)
{
	struct func_name * p = NULL, * q = NULL;
	if (p_func_name_list == NULL) {
		p_func_name_list = (struct func_name*)malloc(sizeof(struct func_name));
		strcpy(p_func_name_list->name, "NULL");	/* no need to declare NULL*/
		p_func_name_list->next = NULL;
	}
	
	p = p_func_name_list;
	while (p) {
		if (strcmp(p->name, func_name) == 0)
			return;
		q = p;
		p = p->next;
	}
	p = (struct func_name*)malloc(sizeof(struct func_name));
	strcpy(p->name, func_name);
	p->next = NULL;
	q->next = p;
	return;
}

struct ioctl_entry {
	unsigned int	cmd;
	unsigned int	new_cmd;
	unsigned long	hash;
	long		index;
	long		next_index;
	char cmd_name[MAX_IOCTLCMD_NAME_LENGTH];
	char new_cmd_name[MAX_IOCTLCMD_NAME_LENGTH];
	char handler_name[MAX_HANDLER_NAME_LENGTH];
	struct ioctl_entry *next;
};

unsigned char ioctl32_unused_hash_value_table[IOCTL_ARRAY_DEFAULT_SIZE];
struct ioctl_entry * ioctl32_hash_table_list = NULL;
long ioctl_entry_array_length = IOCTL_ARRAY_DEFAULT_SIZE;
long unused_ioctl_entry_number = IOCTL_ARRAY_DEFAULT_SIZE;

#define IOCTL_NORMAL(cmd, handler)	(cmd), #cmd, (cmd)/*new_cmd is the same as cmd*/, "", #handler
#define IOCTL_NEWCMD(cmd, newcmd, handler)	(cmd), #cmd, (newcmd), #newcmd, #handler
#define IOCTL_NEWCMD_ONLY(cmd, newcmd)	\
				(cmd), #cmd, (newcmd), #newcmd, "NULL"

/*
 * Insert ioctl handler item into hash table
 */
int insert_ht(unsigned int cmd, char * cmd_name, unsigned int new_cmd,
		char * new_cmd_name, char * handler_name)
{
	struct ioctl_entry * p = (struct ioctl_entry *) malloc (sizeof(struct ioctl_entry));
	struct ioctl_entry * q = NULL, * k = NULL;

	if (p == NULL) {
		printf("NO MEMORY!\n");
		exit(1);
	}
	
	p->cmd = cmd;
	strcpy(p->cmd_name, cmd_name);
	p->new_cmd = new_cmd;
	strcpy(p->new_cmd_name, new_cmd_name);
	p->hash = ioctl_hash_value(cmd);
	strcpy(p->handler_name, handler_name);
	p->next = NULL;
	p->index = -1;
	p->next_index = -1;

	if (!ioctl32_hash_table_list) {
		ioctl32_hash_table_list = p;
	}
	else if (ioctl32_hash_table_list->hash >= p->hash) {
		p->next = ioctl32_hash_table_list;
		ioctl32_hash_table_list = p;
	} else {
		q = ioctl32_hash_table_list;
		while(q && (q->hash < p->hash)) {
			k = q;
			q = q->next;
		}
		q = k->next;
		k->next = p;
		p->next = q;
	}

	if (ioctl32_unused_hash_value_table[p->hash] == 0) {
		unused_ioctl_entry_number--;
		ioctl32_unused_hash_value_table[p->hash] = 1;
		p->index = p->hash;
	}
	/* the handler func declaration */
	if (p->handler_name) {
		insert_func_name(p->handler_name);
	}
	return 0;
}

int setup_ioctl_handler_table(void)
{
	struct ioctl_entry * p = ioctl32_hash_table_list;
	long i;
	int unfound;
	ioctl_entry_array_length = IOCTL_ARRAY_DEFAULT_SIZE;
	

	if (p == NULL) {
		printf("There is no ioctl handlers!\n");
		return 1;
	}

	/*set up the array index*/
	while (p) {
		if (p->index != -1) {
			p = p->next;
			continue;
		}

		if (unused_ioctl_entry_number <= 0) {
			/*expand the array is needed*/
			p->index = ioctl_entry_array_length;
			ioctl_entry_array_length++;
		} else {
			unfound = 1;
			for (i = 0 ; (i < IOCTL_ARRAY_DEFAULT_SIZE) && unfound ; i++) {
				if (ioctl32_unused_hash_value_table[i] == 0) {
					p->index = i;
					ioctl32_unused_hash_value_table[i] = 1;
					unused_ioctl_entry_number--;
					unfound = 0;
				}
			}
			if (unfound == 1) {
				printf("The array is crushed!\n");
				return 1;
			}
		}
		p = p->next;
	}
	/*set up the array link*/
	p = ioctl32_hash_table_list;
	while (p) {
		struct ioctl_entry * head, * tail;
		assert (p->index != -1);
		if (p->hash != (unsigned long) p->index) {
			p = p->next;
			continue;
		}
		tail = p;
		head = ioctl32_hash_table_list;

		while(head) {
			if ((head != tail) && (head != p) && (head->hash == tail->hash)) {
				tail->next_index = head->index;
				tail = head;
			}
			head = head->next;
		}
                p = p->next;
        }

	return 0;
}

struct ioctl_entry * get_ioctl_entry_by_index(long idx)
{
	struct ioctl_entry * p = ioctl32_hash_table_list;
	
	while (p) {
		if (p->index == idx) {
			return p;
		}
		p = p->next;
	}
	
	return NULL;
}


int main(int argc, char ** argv, char ** envp)
{
	long i;
	FILE * fpout;
	struct func_name * p, * q;
	struct ioctl_entry * k;
	char buff[MAX_BUFF_SIZE];
	char end_of_export_part[] = "#define	end_of_exportable_part\n";

	memset((void*)ioctl32_unused_hash_value_table, 0, sizeof(unsigned char) * IOCTL_ARRAY_DEFAULT_SIZE);
	
	insert_ht(IOCTL_NORMAL(SIOCGIFCONF, ifconf_ioctl32));
	insert_ht(IOCTL_NEWCMD(VFAT_IOCTL_READDIR_BOTH32, VFAT_IOCTL_READDIR_BOTH, vfat_ioctl32));
	insert_ht(IOCTL_NEWCMD(VFAT_IOCTL_READDIR_SHORT32, VFAT_IOCTL_READDIR_SHORT, vfat_ioctl32));
	insert_ht(IOCTL_NORMAL(RAW_SETBIND, rawsetbind32));
	insert_ht(IOCTL_NORMAL(RAW_GETBIND, rawgetbind32));
	insert_ht(IOCTL_NORMAL(HDIO_GETGEO, hdio_getgeo32));
	insert_ht(IOCTL_NORMAL(BLKGETSIZE, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(BLKRAGET, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(BLKFRAGET, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(BLKSECTGET, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NEWCMD_ONLY(REISERFS_IOC_UNPACK32, REISERFS_IOC_UNPACK));
	insert_ht(IOCTL_NEWCMD_ONLY(EXT2_IOC_GETFLAGS32, EXT2_IOC_GETFLAGS));
	insert_ht(IOCTL_NEWCMD_ONLY(EXT2_IOC_SETFLAGS32, EXT2_IOC_SETFLAGS));
	insert_ht(IOCTL_NEWCMD_ONLY(EXT2_IOC_GETVERSION32, EXT2_IOC_GETVERSION));
	insert_ht(IOCTL_NEWCMD_ONLY(EXT2_IOC_SETVERSION32, EXT2_IOC_SETVERSION));
	insert_ht(IOCTL_NORMAL(SIOCGIFFLAGS, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFFLAGS, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFADDR, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFADDR, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFDSTADDR, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFDSTADDR, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFBRDADDR, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFBRDADDR, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFNETMASK, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFNETMASK, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFMETRIC, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFMETRIC, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFMEM, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFMEM, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFMTU, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFMTU, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFHWADDR, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFHWADDR, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCADDMULTI, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDELMULTI, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFINDEX, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFPFLAGS, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFPFLAGS, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFTXQLEN, socket_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFTXQLEN, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGIFMAP, siocgifmap_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCSIFMAP, siocsifmap_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 1, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 2, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 3, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 4, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 5, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 6, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 7, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 8, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 9, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 10, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 11, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 12, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 13, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 14, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDEVPRIVATE + 15, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCBONDENSLAVE, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCBONDRELEASE, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCBONDSETHWADDR, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCBONDCHANGEACTIVE, socket_read_arg32_only_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCBONDSLAVEINFOQUERY, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCBONDINFOQUERY, socket_putdata_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCETHTOOL, socket_putdata_ioctl32));
	insert_ht(IOCTL_NEWCMD_ONLY(BLKBSZGET32, BLKBSZGET));
	insert_ht(IOCTL_NEWCMD_ONLY(BLKBSZSET32, BLKBSZSET));
	insert_ht(IOCTL_NEWCMD_ONLY(BLKGETSIZE64_32, BLKGETSIZE64));
	insert_ht(IOCTL_NORMAL(BLKPG, blkpg_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCADDRT, socket_rtentry_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCDELRT, socket_rtentry_ioctl32));
	insert_ht(IOCTL_NORMAL(SIOCGSTAMP, siocgstamp_ioctl32));
	insert_ht(IOCTL_NEWCMD(AUTOFS_IOC_SETTIMEOUT32, AUTOFS_IOC_SETTIMEOUT, arg32_to_arg64_ioctl32));
	insert_ht(IOCTL_NORMAL(LOOP_SET_STATUS, loopset_ioctl32));
	insert_ht(IOCTL_NORMAL(LOOP_GET_STATUS, loopget_ioctl32));
	insert_ht(IOCTL_NEWCMD(MEMWRITEOOB32, MEMWRITEOOB, mtdoob_ioctl32));
	insert_ht(IOCTL_NEWCMD(MEMREADOOB32, MEMREADOOB, mtdoob_ioctl32));
	insert_ht(IOCTL_NORMAL(HDIO_GET_UNMASKINTR, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(HDIO_GET_MULTCOUNT, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(HDIO_GET_KEEPSETTINGS, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(HDIO_GET_32BIT, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(HDIO_GET_NOWERR, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(HDIO_GET_DMA, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(HDIO_GET_NICE, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NORMAL(TIOCGSERIAL, tiocgserial_ioctl32));
	insert_ht(IOCTL_NORMAL(TIOCSSERIAL, tiocsserial_ioctl32));
	insert_ht(IOCTL_NEWCMD(PPPIOCSACTIVE32, PPPIOCSACTIVE, pppfilter_ioctl32));
	insert_ht(IOCTL_NEWCMD(PPPIOCSPASS32, PPPIOCSPASS, pppfilter_ioctl32));
	insert_ht(IOCTL_NEWCMD(PPPIOCGIDLE32, PPPIOCGIDLE, pppiocgidle_ioctl32));
	insert_ht(IOCTL_NEWCMD(PPPIOCSCOMPRESS32, PPPIOCSCOMPRESS, pppiocscompress_ioctl32));
	insert_ht(IOCTL_NORMAL(FBIOGET_FSCREENINFO, fbgfscreeninfo_ioctl32));
	insert_ht(IOCTL_NORMAL(FBIOGETCMAP, fbiogetcmap_ioctl32));
	insert_ht(IOCTL_NORMAL(FBIOPUTCMAP, fbioputcmap_ioctl32));
	insert_ht(IOCTL_NORMAL(GIO_UNIMAP, giounimap_ioctl32));
	insert_ht(IOCTL_NORMAL(PIO_UNIMAP, piounimap_ioctl32));
	insert_ht(IOCTL_NORMAL(GIO_FONTX, giofontx_ioctl32));
	insert_ht(IOCTL_NORMAL(PIO_FONTX, piofontx_ioctl32));
	insert_ht(IOCTL_NORMAL(KDFONTOP, kdfonttop_ioctl32));
	insert_ht(IOCTL_NEWCMD(MTIOCGET32, MTIOCGET, mtiocget_ioctl32));
	insert_ht(IOCTL_NEWCMD(MTIOCPOS32, MTIOCPOS, mtiocpos_ioctl32));
	insert_ht(IOCTL_NEWCMD(MTIOCGETCONFIG32, MTIOCGETCONFIG, mtiocgetconf_ioctl32));
	insert_ht(IOCTL_NEWCMD(MTIOCSETCONFIG32, MTIOCSETCONFIG, mtiocsetconf_ioctl32));
	insert_ht(IOCTL_NORMAL(CDROMREADAUDIO, cdromreadaudio_ioctl32));
	insert_ht(IOCTL_NORMAL(CDROM_SEND_PACKET, cdromsendpacket_ioctl32));
	insert_ht(IOCTL_NORMAL(SG_IO, sgio_ioctl32));
	insert_ht(IOCTL_NEWCMD(USBDEVFS_CONTROL32, USBDEVFS_CONTROL, usbdevfs_control_ioctl32));
	insert_ht(IOCTL_NEWCMD(USBDEVFS_BULK32, USBDEVFS_BULK, usbdevfs_bulk_ioctl32));
	insert_ht(IOCTL_NEWCMD(USBDEVFS_REAPURB32, USBDEVFS_REAPURB, arg32_to_arg64_ioctl32));
	insert_ht(IOCTL_NEWCMD(USBDEVFS_REAPURBNDELAY32, USBDEVFS_REAPURBNDELAY, arg32_to_arg64_ioctl32));
	insert_ht(IOCTL_NEWCMD(USBDEVFS_DISCSIGNAL32, USBDEVFS_DISCSIGNAL, usbdevfs_discsignal_ioctl32));
#ifndef UNUSED_IOCTL
	insert_ht(IOCTL_NEWCMD(VIDIOCGTUNER32, VIDIOCGTUNER, vidiocgtuner_ioctl32));
	insert_ht(IOCTL_NEWCMD(VIDIOCSTUNER32, VIDIOCSTUNER, vidiocstuner_ioctl32));
	insert_ht(IOCTL_NEWCMD(VIDIOCGWIN32, VIDIOCGWIN, vidiocgwin_ioctl32));
	insert_ht(IOCTL_NEWCMD(VIDIOCSWIN32, VIDIOCSWIN, vidiocswin_ioctl32));
	insert_ht(IOCTL_NEWCMD(VIDIOCGFBUF32, VIDIOCGFBUF, vidiocgfbuf_ioctl32));
	insert_ht(IOCTL_NEWCMD(VIDIOCSFBUF32, VIDIOCSFBUF, vidiocsfbuf_ioctl32));
	insert_ht(IOCTL_NEWCMD(VIDIOCGFREQ32, VIDIOCGFREQ, arg32_to_arg64_writeback_only));
	insert_ht(IOCTL_NEWCMD(VIDIOCSFREQ32, VIDIOCSFREQ, arg32_to_arg64_read_only));
	insert_ht(IOCTL_NEWCMD(ATM_GETLINKRATE32, ATM_GETLINKRATE, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETNAMES32, ATM_GETNAMES, atm_iobuf_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETTYPE32, ATM_GETTYPE, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETESI32, ATM_GETESI, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETADDR32, ATM_GETADDR, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_RSTADDR32, ATM_RSTADDR, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_ADDADDR32, ATM_ADDADDR, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_DELADDR32, ATM_DELADDR, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETCIRANGE32, ATM_GETCIRANGE, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_SETCIRANGE32, ATM_SETCIRANGE, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_SETESI32, ATM_SETESI, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_SETESIF32, ATM_SETESIF, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETSTAT32, ATM_GETSTAT, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETSTATZ32, ATM_GETSTATZ, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_GETLOOP32, ATM_GETLOOP, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_SETLOOP32, ATM_SETLOOP, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NEWCMD(ATM_QUERYLOOP32, ATM_QUERYLOOP, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_GETSTAT, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_GETSTATZ, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_SETDIAG, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_CLRDIAG, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_GETDIAG, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_SETFRAMING, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_GETFRAMING, atmif_sioc_ioctl32));
	insert_ht(IOCTL_NORMAL(SONET_GETFRSENSE, atmif_sioc_ioctl32));
#endif
	
	if (setup_ioctl_handler_table()) {
		printf("setup ioctl table error!\n");
		return 1;
	}

	fpout = fopen("./ia32_ioctl.h", "w");
	
	if (!fpout) {
		printf("Open ia32_ioctl.h ERROR!\n");
		return 1;
	}

	/* the include file */
	fputs("#include \"ia32_ioctl_prototype.h\"\n\n", fpout);
	
	p = p_func_name_list;
	if (p) {
		q = p;
		p = p->next;	/* ignore ioctl */
		free(q);
	}
	while (p) {
		q = p;
		fputs ("static long ", fpout);
		fputs (p->name, fpout);
		fputs ("(int fd, unsigned int cmd, unsigned long arg);\n", fpout);
		p = p->next;
		free(q);
	}

	/* the ioctl_table structure */
	fputs ("\n\nstruct ioctl_entry {\n", fpout);
	fputs ("\tunsigned int cmd;\n", fpout);
	fputs ("\tunsigned int new_cmd;\n", fpout);
	fputs ("\tlong (*handler)(int, unsigned int, unsigned long);\n", fpout);
	fputs ("\tlong next;\n", fpout);
	fputs ("};\n", fpout);

	/* the ioctl handler table */
	fputs ("\n\nstruct ioctl_entry ioctl_handler_table[] = {\n", fpout);
	for (i = 0 ; i < ioctl_entry_array_length ; i++) {
		if ((i < IOCTL_ARRAY_DEFAULT_SIZE) && (ioctl32_unused_hash_value_table[i] == 0)) {
			/*empty entry*/
			fputs ("\t{/*empty entry*/0, 0, NULL, -1}", fpout);
		}
		else {
			k = get_ioctl_entry_by_index(i);
			if (k) {
				sprintf (buff, "\t{/*%ld*//*%s*/%ul, /*%s*/%ul, %s, %ld}\0", 
						i,
						k->cmd_name,
						k->cmd,
						k->new_cmd_name,
						k->new_cmd,
						strcmp(k->handler_name,"ioctl")?k->handler_name:"NULL/*ioctl*/",
						k->next_index);
				fputs (buff, fpout);
			}
			else {
				printf("Not found!\n");
				return 1;
			}
		}
		if (i != ioctl_entry_array_length - 1) {
			fputs (",\n", fpout);
		}
		else {
			fputs ("\n", fpout);
		}
        }
	fputs ("};\n\n", fpout);

	fclose(fpout);
	
	return 0;
}

