/* 
 * user32.h
 *
 * contains declarations needed for external wrapper files
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#ifndef _USER32_H
#define _USER32_H


#include <ia32x_utils.h>
#include "ia32ss_common.h"
#include "user32_ia32_syscall_prototypes.h"

#define TRUE					1
#define FALSE					0

#define IS_HIGH_ZERO(l)  (((l) >> 32) == 0)
#define ASSERT_HIGH_ZERO(l)  assert ((l >> 32) == 0)

#define UNUSED __attribute__((unused))

extern long HIDDEN init_mem_module (int page_size, 
                                    long (*p_safe_memcpy)(void *, void *, unsigned long));

#ifdef DEBUG
# define DBCODE(sw,statement)  do { if(sw) { statement; } } while(0)
#else
# define DBCODE(sw,statement)
#endif // DEBUG

/* convert int to pointer */
static inline void * I2P (unsigned addr)
{
    void * p;
    if ((unsigned long)(addr) > IA32_MAX_USER_ADDR) {
        __asm__ __volatile__ ("sxt4 %0=%1;;\n" : "=r"(p) : "r"(addr));        
    } else {
        __asm__ __volatile__ ("zxt4 %0=%1;;\n" : "=r"(p) : "r"(addr));        
    }
    return p;
}

#define P2U(addr) (unsigned)(unsigned long)(addr)

extern long __long_syscall_wrapper (unsigned long p1, unsigned long p2,
                                    unsigned long p3, unsigned long p4, unsigned long p5,
                                    unsigned long p6, long syscall_id);
#define LONG_SYSCALL0(s) __long_syscall_wrapper (0ul,0ul,0ul,0ul,0ul,0ul,SYS_##s)
#define LONG_SYSCALL1(s,p1) __long_syscall_wrapper ((unsigned long)(p1),0ul,0ul,0ul,0ul,0ul,SYS_##s)
#define LONG_SYSCALL2(s,p1,p2) __long_syscall_wrapper ((unsigned long)(p1),(unsigned long)(p2),0ul,0ul,0ul,0ul,SYS_##s)
#define LONG_SYSCALL3(s,p1,p2,p3) __long_syscall_wrapper ((unsigned long)(p1),(unsigned long)(p2),(unsigned long)(p3),0ul,0ul,0ul,SYS_##s)
#define LONG_SYSCALL4(s,p1,p2,p3,p4) __long_syscall_wrapper ((unsigned long)(p1),(unsigned long)(p2),(unsigned long)(p3),(unsigned long)(p4),0ul,0ul,SYS_##s)
#define LONG_SYSCALL5(s,p1,p2,p3,p4,p5) __long_syscall_wrapper ((unsigned long)(p1),(unsigned long)(p2),(unsigned long)(p3),(unsigned long)(p4),(unsigned long)(p5),0ul,SYS_##s)
#define LONG_SYSCALL6(s,p1,p2,p3,p4,p5,p6) __long_syscall_wrapper ((unsigned long)(p1),(unsigned long)(p2),(unsigned long)(p3),(unsigned long)(p4),(unsigned long)(p5),(unsigned long)(p6),SYS_##s)

typedef void(*PLABEL_PTR_TYPE)();
#define PLABEL_FUNC(p)		((p)[0])
#define PLABEL_GP(p)		((p)[1])

/* Copy from ilp32, and removed check on (void*)-1 from IS_VALID_PTR */
#define IS_VALID_PTR(ptr)       ((void*)(ptr) != (void*)NULL)
#define NOT_VALID_PTR(ptr)      (!IS_VALID_PTR(ptr))
#define VALID_PTR(ptr32, ptr64) ((IS_VALID_PTR(ptr32)) ? (void*)ptr64 : (void*)(ptr32))
                                                                                                                             

extern int HIDDEN get_page_prot_e (unsigned long addr, int * is_exist);
#define get_page_prot(addr) get_page_prot_e(addr, 0)

extern mutex_handle HIDDEN g_memory_sync;
extern mutex_handle HIDDEN g_ctxmap_sync;

long iti_after_fork (long is_parent UNUSED);
long iti_before_fork (void);

#endif // _USER32_H
