/* 
 * user32_asm.S
 *
 * assembly functions for the external wrapper
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/



#include <ia32x_utils.h>
#include "ia32toia64.h"
#include "user32_const_for_asm.h"


/***************************************************************************/
//
// long __ilp32_wrapper 
//		r32 = 		(unsigned long) id of syscall32
//		r33-r38 =	(unsigned long) param1-param6 
//								
/***************************************************************************/

.psr abi64;
.psr lsb;
.lsb;
.text;
.global __ilp32_wrapper
.proc __ilp32_wrapper
__ilp32_wrapper:

.prologue
.fframe 0
{ .mii
.save ar.pfs, r39
	alloc loc0 = ar.pfs, 7, 6, 7, 0
.save rp, r40
	mov loc1 = b0
	mov out4 = in5
}


.body
// get the address of the conversion table
.global ia32_2_ia64
{ .mmi
	addl loc2=@ltoff(ia32_2_ia64), r1;;
	ld8 loc2=[loc2]
	shl loc3=in0, 4	;;
}
// get the index into the table
// get the table entry for the requested syscall
// the entry is a struct
// {
//		long flag; // whether the data is syscall id or function address
//		long data; 
// }
{ .mii
	mov out0 = in1 // out0==r45
	add loc2=loc2, loc3 ;;
	mov out1 = in2
}

{ .mmi
// get the flag
	ld8 loc4=[loc2], 8 ;;
	mov out2 = in3
	mov out3 = in4
}

// p6,p7,p12 are used as follows -
// p6==1 -> CT_DIRECT
// p7==1 -> CT_WRAPPER
// p12==1-> CT_LONG

{ .mmi
// get the data (func-addr | syscall id)
	ld8 r15=[loc2]
	mov out5 = in6
// can we directly call the system ?
	cmp.eq p6,p9=CT_DIRECT, loc4 ;;
}

{ .mii
// is this a long syscall ?
(p9)	cmp.eq.unc p12,p7=CT_LONG, loc4
// prepare the output param for long-syscall-wrapper
	mov out6=r15 ;;
// call the system
(p6)	break.i __BREAK_SYSCALL ;;
}

{ .mii
// load the address of the wrapper function
(p7)	ld8 loc5=[r15]
// check error code of the syscall
(p6)	cmp.eq.unc p8,p0=-1, r10 ;;
// invoke the syscall
(p7)	mov b6=loc5
}


{ .mbb
// negate the syscall return value in case of error
(p8)	sub r8=r0,r8
// call the wrapper function
(p7)	br.call.sptk.many b0=b6
// if this function maps to a long syscall then call it now
(p12)	br.call.sptk.few b0=__long_syscall_wrapper ;;
}



L1:
{ .mii
	nop.m 0
	mov.i ar.pfs = loc0
	mov b0 = loc1
}


{ .bbb
	br.ret.sptk.many b0
	nop.b 0
	nop.b 0
}

.endp __ilp32_wrapper

/***************************************************************************/
//
// long __long_syscall_wrapper 
//		r32-r37 =	(unsigned long) param1-param6 
//		r38 = 		(unsigned long) id of syscall32
//								
/***************************************************************************/

.psr abi64;
.psr lsb;
.lsb;
.text;
.global __long_syscall_wrapper
.proc __long_syscall_wrapper
__long_syscall_wrapper:

.prologue
.fframe 0

{ .mii
.save ar.pfs, r39
	alloc loc0 = ar.pfs, 7, 7, 6, 0
.save rp, r40
	mov loc1 = b0
.save pr, r41
	mov loc2=pr
}




.body

{ .mii
	addl loc4=@ltoff(@tprel(long_syscall_flag)),gp
	mov out0=in0
	mov out1=in1;;
}

{ .mmi
	ld8 loc5=[loc4] ;;
// put the address of long_syscall_flag in loc4
	add loc4=loc5,r13
	mov out2=in2
}

{ .mmi
	mov out3=in3 ;;
// set p1
	cmp.eq p1,p0=r0,r0
// put the new value of the flag in loc3
	mov loc3=BEFORE_BREAK 
}

{ .mii
	mov out4=in4
	mov out5=in5 ;;
	mov r15=in6
}


{ .mmi
// read the old value of the flag into loc5 
// and write its new value atomically
	xchg8 loc5=[loc4], loc3 ;;
	mov loc6=SYSCALL_COMPLETED
// check the long-syscall-flag. if it's anything other
// than AFTER_CHECK_QUE then skip the syscall
	cmp.eq p7,p9=AFTER_CHECK_QUE, loc5
}

{ .mfb
	mov loc5=AFTER_SYSCALL
	nop.f 0
(p9)	br.cond.sptk.few skip_syscall
}

// invoke the system iff no signal arrived
.global L_bbi
L_bbi:
{ .mmi
// p1 will be cleared by the signal handler in case a signal
// arrived after the long-syscall-flag was assigned with
// BEFORE_BREAK
(p1)	break __BREAK_SYSCALL ;; // must be first in bundle
	st8 [loc4]=loc5 // AFTER_SYSCALL
	cmp.eq	p8,p0=-1, r10	;;
}

{ .mmi
(p1)	st8 [loc4]=loc6 // SYSCALL_COMPLETED
	nop.m 0
(p8)	sub r8=r0,r8
}

// restore preserved regs
skip_syscall:
{ .mii
	nop.m 0
	mov.i ar.pfs=loc0
	mov b0=loc1
}

{ .mib
	nop.m 0
	mov pr=loc2, -1
	br.ret.sptk.many b0
}

.endp __long_syscall_wrapper

