/*
 * user32_convers_tab.c
 *
 * conversion table from IA-32 syscall id's into 64 bit syscalls/wrapper funcs
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include "user32_ia32_syscall_prototypes.h"
#include "user32_const_for_asm.h"
#include "user32_pathwalk.h"

#define SYS_DIRECT(name) {CT_DIRECT,SYS_##name}
#define SYS_LONG(name) {CT_LONG,SYS_##name}
#define SYS_WRAPPER(name) {CT_WRAPPER,(unsigned long)name}
#if NOKERNELALTROOT
#define SYS_PWS(name) {CT_WRAPPER,(unsigned long)pws_##name}
#else
#define SYS_PWS(name) {CT_DIRECT,SYS_##name}
#endif

 
struct  {
	long is_ia64_syscall;
	unsigned long data;
} ia32_2_ia64[] = {

/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*exit*/			SYS_DIRECT(exit),
/*fork*/			SYS_WRAPPER(sys32_ni_syscall),
/*read*/			SYS_LONG(read),
/*write*/			SYS_LONG(write),
/*open*/			SYS_PWS(open),
/*close*/			SYS_DIRECT(close),
/*waitpid*/			SYS_WRAPPER(sys32_waitpid),
/*creat*/			SYS_PWS(creat),
/*link*/			SYS_PWS(link),
/*unlink*/			SYS_PWS(unlink),
/*xecve*/			SYS_WRAPPER(sys32_execve),
/*chdir*/			SYS_PWS(chdir),
/*time*/			SYS_WRAPPER(sys32_time),
/*mknod*/			SYS_PWS(mknod),
/*chmod*/			SYS_PWS(chmod),
/*lchown*/			SYS_PWS(lchown),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*lseek*/			SYS_WRAPPER(sys32_lseek),
/*getpid*/			SYS_DIRECT(getpid),
/*mount*/			SYS_PWS(mount),
/*oldumount*/			SYS_WRAPPER(sys32_oldumount),
/*setuid*/			SYS_DIRECT(setuid),
/*getuid*/			SYS_DIRECT(getuid),
/*stime*/			SYS_WRAPPER(sys32_stime),
/*ptrace*/			SYS_WRAPPER(sys32_ptrace),
/*alarm*/			SYS_WRAPPER(sys32_alarm),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*pause*/			SYS_WRAPPER(sys32_pause),
/*utime*/			SYS_WRAPPER(sys32_utime),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*access*/			SYS_PWS(access),
/*nice*/			SYS_WRAPPER(sys32_nice),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*sync*/			SYS_DIRECT(sync),
/*kill*/			SYS_LONG(kill),
/*rename*/			SYS_PWS(rename),
/*mkdir*/			SYS_PWS(mkdir),
/*rmdir*/			SYS_PWS(rmdir),
/*dup*/				SYS_DIRECT(dup),
/*pipe*/			SYS_WRAPPER(sys32_pipe),
/*times*/			SYS_WRAPPER(sys32_times),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*brk*/				SYS_WRAPPER(sys32_brk),
/*setgid*/			SYS_DIRECT(setgid),
/*getgid*/			SYS_DIRECT(getgid),
/*signal*/			SYS_WRAPPER(sys32_signal),
/*geteuid*/			SYS_DIRECT(geteuid),
/*getegid*/			SYS_DIRECT(getegid),
/*acct*/			SYS_PWS(acct),
/*umount*/			SYS_PWS(umount),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ioctl*/			SYS_WRAPPER(sys32_ioctl),
/*fcntl*/			SYS_WRAPPER(sys32_fcntl),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*setpgid*/			SYS_DIRECT(setpgid),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*umask*/			SYS_DIRECT(umask),
/*chroot*/			SYS_PWS(chroot),
/*ustat*/			SYS_DIRECT(ustat),
/*dup2*/			SYS_DIRECT(dup2),
/*getppid*/			SYS_DIRECT(getppid),
/*getpgrp*/			SYS_WRAPPER(sys32_getpgrp),
/*setsid*/			SYS_DIRECT(setsid),
/*sigaction*/			SYS_WRAPPER(sys32_sigaction),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*setreuid*/			SYS_DIRECT(setreuid),
/*setregid*/			SYS_DIRECT(setregid),
/*sigsuspend*/			SYS_WRAPPER(sys32_sigsuspend),
/*sigpending*/			SYS_WRAPPER(sys32_sigpending),
/*sethostname*/			SYS_DIRECT(sethostname),
/*setrlimit*/			SYS_WRAPPER(sys32_setrlimit),
/*old_getrlimit*/		SYS_WRAPPER(sys32_old_getrlimit),
/*getrusage*/			SYS_WRAPPER(sys32_getrusage),
/*gettimeofday*/		SYS_WRAPPER(sys32_gettimeofday),
/*settimeofday*/		SYS_WRAPPER(sys32_settimeofday),
/*getgroups16*/			SYS_WRAPPER(sys32_getgroups16),
/*setgroups16*/			SYS_WRAPPER(sys32_setgroups16),
/*old_select*/			SYS_WRAPPER(sys32_old_select),
/*symlink*/			SYS_PWS(symlink),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*readlink*/			SYS_PWS(readlink),
/*uselib*/			SYS_DIRECT(uselib),
/*swapon*/			SYS_PWS(swapon),
/*reboot*/			SYS_DIRECT(reboot),
/*readdir*/			SYS_WRAPPER(sys32_readdir),
/*mmap*/			SYS_WRAPPER(sys32_mmap),
/*munmap*/			SYS_WRAPPER(sys32_munmap),
/*truncate*/			SYS_PWS(truncate),
/*ftruncate*/			SYS_DIRECT(ftruncate),
/*fchmod*/			SYS_DIRECT(fchmod),
/*fchown*/			SYS_DIRECT(fchown),
/*getpriority*/			SYS_WRAPPER(sys32_getpriority),
/*setpriority*/			SYS_DIRECT(setpriority),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*statfs*/			SYS_WRAPPER(sys32_statfs),
/*fstatfs*/			SYS_WRAPPER(sys32_fstatfs),
/*ioperm*/			SYS_WRAPPER(sys32_ni_syscall),
/*socketcall*/			SYS_WRAPPER(sys32_socketcall),
/*syslog*/			SYS_DIRECT(syslog),
/*setitimer*/			SYS_WRAPPER(sys32_setitimer),
/*getitimer*/			SYS_WRAPPER(sys32_getitimer),
/*newstat*/			SYS_WRAPPER(sys32_newstat),
/*newlstat*/			SYS_WRAPPER(sys32_newlstat),
/*newfstat*/			SYS_WRAPPER(sys32_newfstat),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*iopl*/			SYS_WRAPPER(sys32_ni_syscall),
/*vhangup*/			SYS_DIRECT(vhangup),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*wait4*/			SYS_WRAPPER(sys32_wait4),
/*swapoff*/			SYS_PWS(swapoff),
/*sysinfo*/			SYS_WRAPPER(sys32_sysinfo),
/*ipc*/				SYS_WRAPPER(sys32_ipc),
/*fsync*/			SYS_DIRECT(fsync),
/*sigreturn*/			SYS_WRAPPER(sys32_sigreturn),
/*clone*/			SYS_WRAPPER(sys32_clone),
/*setdomainname*/		SYS_DIRECT(setdomainname),
/*newuname*/			SYS_WRAPPER(sys32_newuname),
/*modify_ldt*/			SYS_WRAPPER(sys32_ni_syscall),
/*adjtimex*/			SYS_WRAPPER(sys32_adjtimex),
/*mprotect*/			SYS_WRAPPER(sys32_mprotect),
/*sigprocmask*/			SYS_WRAPPER(sys32_sigprocmask),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*quotactl*/			SYS_DIRECT(quotactl),
/*getpgid*/			SYS_DIRECT(getpgid),
/*fchdir*/			SYS_DIRECT(fchdir),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*sysfs*/			SYS_DIRECT(sysfs),
/*personality*/			SYS_WRAPPER(sys32_personality),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*setfsuid*/			SYS_DIRECT(setfsuid),
/*setfsgid*/			SYS_DIRECT(setfsgid),
/*llseek*/			SYS_WRAPPER(sys32_llseek),
/*getdents*/			SYS_WRAPPER(sys32_getdents),
/*select*/			SYS_WRAPPER(sys32_select),
/*flock*/			SYS_DIRECT(flock),
/*msync*/			SYS_WRAPPER(sys32_msync),
/*readv*/			SYS_WRAPPER(sys32_readv),
/*writev*/			SYS_WRAPPER(sys32_writev),
/*getsid*/			SYS_DIRECT(getsid),
/*fdatasync*/			SYS_DIRECT(fdatasync),
/*sysctl*/			SYS_WRAPPER(sys32_sysctl),
/*mlock*/			SYS_WRAPPER(sys32_mlock),
/*munlock*/			SYS_WRAPPER(sys32_munlock),
/*mlockall*/			SYS_DIRECT(mlockall),
/*munlockall*/			SYS_DIRECT(munlockall),
/*sched_setparam*/		SYS_DIRECT(sched_setparam),
/*sched_getparam*/		SYS_DIRECT(sched_getparam),
/*sched_setscheduler*/		SYS_DIRECT(sched_setscheduler),
/*sched_getscheduler*/		SYS_DIRECT(sched_getscheduler),
/*sched_yield*/			SYS_DIRECT(sched_yield),
/*sched_get_priority_max*/	SYS_DIRECT(sched_get_priority_max),
/*sched_get_priority_min*/	SYS_DIRECT(sched_get_priority_min),
/*sched_rr_get_interval*/	SYS_WRAPPER(sys32_sched_rr_get_interval),
/*nanosleep*/			SYS_WRAPPER(sys32_nanosleep),
/*mremap*/			SYS_WRAPPER(sys32_mremap),
/*setresuid*/			SYS_DIRECT(setresuid),
/*getresuid16*/			SYS_WRAPPER(sys32_getresuid16),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*poll*/			SYS_LONG(poll),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*setresgid*/			SYS_DIRECT(setresgid),
/*getresgid16*/			SYS_WRAPPER(sys32_getresgid16),
/*prctl*/			SYS_DIRECT(prctl),
/*rt_sigreturn*/		SYS_WRAPPER(sys32_rt_sigreturn),
/*rt_sigaction*/		SYS_WRAPPER(sys32_rt_sigaction),
/*rt_sigprocmask*/		SYS_DIRECT(rt_sigprocmask),
/*rt_sigpending*/		SYS_WRAPPER(sys32_rt_sigpending),
/*rt_sigtimedwait*/		SYS_WRAPPER(sys32_rt_sigtimedwait),
/*rt_sigqueueinfo*/		SYS_WRAPPER(sys32_rt_sigqueueinfo),
/*rt_sigsuspend*/		SYS_LONG(rt_sigsuspend),
/*pread*/			SYS_WRAPPER(sys32_pread),
/*pwrite*/			SYS_WRAPPER(sys32_pwrite),
/*chown*/			SYS_PWS(chown),
/*getcwd*/			SYS_DIRECT(getcwd),
/*capget*/			SYS_DIRECT(capget),
/*capset*/			SYS_DIRECT(capset),
/*sigaltstack*/			SYS_WRAPPER(sys32_sigaltstack),
/*sendfile*/			SYS_WRAPPER(sys32_sendfile),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*ni_syscall*/			SYS_WRAPPER(sys32_ni_syscall),
/*vfork*/			SYS_WRAPPER(sys32_ni_syscall),
/*getrlimit*/			SYS_WRAPPER(sys32_getrlimit),
/*mmap2*/			SYS_WRAPPER(sys32_mmap2),
/*truncate64*/			SYS_WRAPPER(sys32_truncate64),
/*ftruncate64*/			SYS_WRAPPER(sys32_ftruncate64),
/*stat64*/			SYS_WRAPPER(sys32_stat64),
/*lstat64*/			SYS_WRAPPER(sys32_lstat64),
/*fstat64*/			SYS_WRAPPER(sys32_fstat64),
/*lchown*/			SYS_DIRECT(lchown),
/*getuid*/			SYS_DIRECT(getuid),
/*getgid*/			SYS_DIRECT(getgid),
/*geteuid*/			SYS_DIRECT(geteuid),
/*getegid*/			SYS_DIRECT(getegid),
/*setreuid*/			SYS_DIRECT(setreuid),
/*setregid*/			SYS_DIRECT(setregid),
/*getgroups*/			SYS_DIRECT(getgroups),
/*setgroups*/			SYS_DIRECT(setgroups),
/*fchown*/			SYS_DIRECT(fchown),
/*setresuid*/			SYS_DIRECT(setresuid),
/*getresuid*/			SYS_DIRECT(getresuid),
/*setresgid*/			SYS_DIRECT(setresgid),
/*getresgid*/			SYS_DIRECT(getresgid),
/*chown*/			SYS_DIRECT(chown),
/*setuid*/			SYS_DIRECT(setuid),
/*setgid*/			SYS_DIRECT(setgid),
/*setfsuid*/			SYS_DIRECT(setfsuid),
/*setfsgid*/			SYS_DIRECT(setfsgid),
/*pivot_root*/			SYS_PWS(pivot_root),
/*mincore*/			SYS_DIRECT(mincore),
/*madvise*/			SYS_DIRECT(madvise),
/*getdents64*/			SYS_DIRECT(getdents64),
/*fcntl64*/			SYS_WRAPPER(sys32_fcntl64),
/* ------------ new syscalls ------------ */
/*tux*/				SYS_WRAPPER(sys32_ni_syscall),
/*secutiry*/			SYS_WRAPPER(sys32_ni_syscall),
/*gettid*/			SYS_DIRECT(gettid),
/*readahead*/			SYS_DIRECT(readahead),
/*setxattr*/			SYS_PWS(setxattr),
/*lsetxattr*/			SYS_PWS(lsetxattr),
/*fsetxattr*/			SYS_DIRECT(fsetxattr),
/*getxattr*/			SYS_PWS(getxattr),
/*lgetxattr*/			SYS_PWS(lgetxattr),
/*fgetxattr*/			SYS_DIRECT(fgetxattr),
/*listxattr*/			SYS_PWS(listxattr),
/*llistxattr*/			SYS_PWS(llistxattr),
/*flistxattr*/			SYS_DIRECT(flistxattr),
/*removexattr*/			SYS_PWS(removexattr),
/*lremovexattr*/		SYS_PWS(lremovexattr),
/*fremovexattr*/		SYS_DIRECT(fremovexattr),
/*tkill*/			SYS_LONG(tkill),
/*sendfile64*/			SYS_WRAPPER(sys32_sendfile64),
/*futex*/			SYS_WRAPPER(sys32_futex),
/*sched_setaffinity*/		SYS_DIRECT(sched_setaffinity),
/*sched_getaffinity*/		SYS_DIRECT(sched_getaffinity),
/*set_thread_area*/		SYS_WRAPPER(sys32_ni_syscall),
/*get_thread_area*/		SYS_WRAPPER(sys32_ni_syscall),
/*io_setup*/			SYS_WRAPPER(sys32_io_setup),
/*io_destroy*/			SYS_WRAPPER(sys32_io_destroy),
/*io_getevents*/		SYS_WRAPPER(sys32_io_getevents),
/*io_submit*/			SYS_WRAPPER(sys32_io_submit),
/*io_cancel*/			SYS_WRAPPER(sys32_io_cancel),
/*fadvise64*/			SYS_DIRECT(fadvise64),
/*free_hugepages*/		SYS_WRAPPER(sys32_ni_syscall),
/*exit_group*/			SYS_DIRECT(exit_group),
/*lookup_dcookie*/		SYS_DIRECT(lookup_dcookie),
/*epoll_create*/		SYS_DIRECT(epoll_create),
/*epoll_ctl*/			SYS_WRAPPER(sys32_epoll_ctl),
/*epoll_wait*/			SYS_WRAPPER(sys32_epoll_wait),
/*remap_file_pages*/		SYS_WRAPPER(sys32_ni_syscall),
/*set_tid_address*/		SYS_DIRECT(set_tid_address),
/*timer_create*/		SYS_WRAPPER(sys32_timer_create),
/*timer_settime*/		SYS_WRAPPER(sys32_timer_settime),
/*timer_gettime*/		SYS_WRAPPER(sys32_timer_gettime),
/*timer_getoverrun*/		SYS_DIRECT(timer_getoverrun),
/*timer_delete*/		SYS_DIRECT(timer_delete),
/*clock_settime*/		SYS_WRAPPER(sys32_clock_settime),
/*clock_gettime*/		SYS_WRAPPER(sys32_clock_gettime),
/*clock_getres*/		SYS_WRAPPER(sys32_clock_getres),
/*clock_nanosleep*/		SYS_WRAPPER(sys32_clock_nanosleep),
/*statfs64*/			SYS_WRAPPER(sys32_ni_syscall/*sys32_statfs64*/),
/*fstatfs64*/			SYS_WRAPPER(sys32_fstatfs64),
/*tgkill*/			SYS_LONG(tgkill),
/*utimes*/			SYS_WRAPPER(sys32_utimes),
/*fadvise64_64*/		SYS_DIRECT(fadvise64),

/* NEXT LINE MUST BE THE LAST IN THE TABLE */
/* end of table */		{-1, 0}
};

