/*
 * user32_files.c
 *
 * wrapper functions for syscalls that deal with files.
 * used only by external wrapper
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Kevin Tian <kevin.tian@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>

#include "user32.h"

long HIDDEN
sys32_select (int nfds, __u32 readfds, __u32 writefds,
	      __u32 exceptfds, __u32 timeout)
{
	struct timeval32 to32;
	struct timeval to64;
	int v, retval;

	DBPRINT (0, "nfds=%d readfds=%p writefds=%p exceptfds=%p timeout=%p\n", 
		 nfds, I2P(readfds), I2P(writefds), I2P(exceptfds), I2P(timeout));

	v = IS_VALID_PTR(I2P(timeout));
	if (v) {
		if (g_safe_memcpy((void *)&to32, I2P(timeout), sizeof(to32)) < 0)
			return -EFAULT;
		to64.tv_sec = to32.tv_sec;
		to64.tv_usec = to32.tv_usec;
	}

	retval = LONG_SYSCALL5 (select, nfds, readfds, writefds, exceptfds, VALID_PTR(I2P(timeout), &to64));
	
	if ((retval >= 0) && v) {
		struct timeval32 *top32 = (struct timeval32*) I2P(timeout);
		top32->tv_sec = to64.tv_sec;
		top32->tv_usec = to64.tv_usec;
	}

	return retval;
}


long HIDDEN
sys32_llseek(int fd, __u32 offset_high, __u32 offset_low,
	     __u32 result, __u32 origin)
{
	__s64 offset;
	__s64 retval;

	offset = ((__s64)offset_high << 32) | offset_low;
	DBPRINT (0, "fd=%d offset=%lx origin=%d\n", fd, offset, origin);

	retval = SYSCALL (lseek, (fd, offset, origin));
	if (retval < 0) {
		return (int)retval;
	}

	/* It's loff_t(64bit) type */
	if (g_safe_memcpy (I2P(result), &retval, sizeof(retval)))
		return -EFAULT;
	return 0;
}

long HIDDEN
sys32_sendfile64(int out_fd, int in_fd, __u32 offset, __u32 count)
{
	/* offset is allowed to be NULL, which means sending from current offset recorded in fd */
	return (int)SYSCALL (sendfile, (out_fd, in_fd, (__off64_t*)I2P(offset), count));
}
