/*
 * user32_mem_regions.h
 *
 * prototypes of function to handle the list of memory regions that were 
 * mapped by the application
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#ifndef _USER32_MEM_REGIONS_H
#define _USER32_MEM_REGIONS_H

#include "ia32ss_common.h"
#include "ia32ss_list.h"


/* attributes of a memory region */
typedef struct _region_attrib_t {
	unsigned long len;
	unsigned prot;
	unsigned flags;
} region_attrib;

#define NEXT_REGION(p_region)	((void*) get_element_next ((list_handle) p_region))
#define PREV_REGION(p_region)	((void*) get_element_prev ((list_handle) p_region))
#define REGION_STARTADDR(p_region)	((unsigned long)get_element_key((list_handle) p_region))
#define REGION_ENDADDR(p_region)					\
	({								\
	 	region_attrib * ra = (void*)get_element_data (region);	\
		unsigned long region_end;				\
		region_end = REGION_STARTADDR (p_region) + ra->len;	\
		region_end;						\
	})

extern long HIDDEN find_region (unsigned long addr, void **p_region);
extern long HIDDEN remove_region (unsigned long start, unsigned long len);
extern long HIDDEN insert_region (unsigned long start, unsigned long len, unsigned prot, unsigned flags);
extern long HIDDEN is_inside_region (unsigned long start, unsigned long len);
extern long HIDDEN get_region_prot (unsigned long addr, unsigned *prot, unsigned long *start, 
                                    unsigned long *len, unsigned *flags);
extern long HIDDEN set_region_prot (unsigned long start, unsigned long len, unsigned prot);
extern unsigned long HIDDEN get_ia32_free_memory (unsigned long start, unsigned long len, 
                                                  unsigned flags, int is_ia32_user_space);
#ifdef DEBUG
extern void HIDDEN print_regions(void);
#endif
#endif //  _USER32_MEM_REGIONS_H
