/* 
 * btlib.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef BTLIB_H
#define BTLIB_H

#include <ia32x_utils.h>
#include "btrans.inc/ia32exec_os_api.h"

#include "btlib_thread_data.h"

#undef TRUE
#define TRUE	1
#undef FALSE
#define FALSE	0
#define OFFSETOF(field,struct_type) (unsigned long)&(((struct_type*)0)->field)
#define FULL_WORD_MASK 0xffffffffu
#define LO_WORD_MASK   0x00000000fffffffful

#define P2U(addr) (unsigned)(uintptr_t)(addr)
#define P2I(addr) (int)(uintptr_t)(addr)

#define BLOCK_ALL_SIGNALS1()														\
{																					\
	long __ret = SYSCALL (rt_sigprocmask, (SIG_SETMASK, &set, &oldset, sizeof(k_sigset_t)));\
	sys_err_chk (__ret, "sigprocmask");												\
}

#define BLOCK_ALL_SIGNALS()			\
{						\
	k_sigset_t set, oldset;			\
	sigfillset (&set);			\
	BLOCK_ALL_SIGNALS1();

#define RESTORE_SIGNALS1()															\
{																					\
	long __ret = SYSCALL (rt_sigprocmask, (SIG_SETMASK,&oldset,NULL,sizeof(k_sigset_t)));	\
	sys_err_chk (__ret, "sigprocmask");												\
}

#define RESTORE_SIGNALS()															\
	RESTORE_SIGNALS1();																\
}

/* try tkill() at first, retry kill() when tkill() is not supported */
#define KILL_THREAD_SIGNAL(__tid, __signo)				\
({									\
	long __ret = SYSCALL (tkill, ((__tid), (__signo)));		\
	if (__ret == -ENOSYS) {						\
		__ret = SYSCALL (kill, ((__tid), (__signo)));		\
	}								\
	__ret;								\
})

#define	EFLAGS_INIT_VAL		0x202
#define	FPCW_INIT_VAL		0x37f
#define	FPTW_INIT_VAL		0xffff
#define	MXCSR_INIT_VAL		0x1f80
#define	FPSW_EXCEPTIONS_MASK	0x80FFu

/* longjmp_status values */
enum { CONTINUE, SIGNAL, RUN, DELIVER, IA32_SYSCALL, TRAP };

/* 
 * a function pointer in 64 bit is an array of two unsigned longs
 * the first entry is the function pointer and the second is the value of the 
 * global data pointer in the module where the function resides
 */
#define PLABEL_FUNC(p)		((p)[0])
#define PLABEL_GP(p)		((p)[1])

/* Base addresses for code and memory allocation (above 4GB) and BTGeneric load address */

#define BTGENERIC_LOAD_ADDRESS		0x2000000100000000
#define INITIAL_CODE_ADDRESS		0x2000000110000000
#define INITIAL_DATA_ADDRESS		0x2000000120000000
#define INITIAL_DEFAULT_ADDRESS		0x2000000130000000
#define INITIAL_BTLIB_ADDRESS		0x2000000150000000
#define BTGENERIC_MEMORY_MASK		0xfffffffff0000000ul

/* page alignement macro */
#define ROUNDUP(v,b) (((v) + (b) - 1) & ~((b) - 1))
#define ROUNDDOWN(v,b)	((v) & ~((b) - 1))

#define PAGESTART(vadd,psize) (unsigned long)ROUNDDOWN(vadd, psize)
#define PAGEALIGN(vadd,psize) (unsigned long)ROUNDUP(vadd,psize)

/* System call parameter extraction macros based on Assembly-HOWTO */
#define IA32_SYSCALL_ID(ctx)		(ctx)->eax
#define IA32_SYSCALL_PARAM1(ctx)	(ctx)->ebx
#define IA32_SYSCALL_PARAM2(ctx)	(ctx)->ecx
#define IA32_SYSCALL_PARAM3(ctx)	(ctx)->edx
#define IA32_SYSCALL_PARAM4(ctx)	(ctx)->esi
#define IA32_SYSCALL_PARAM5(ctx)	(ctx)->edi
#define IA32_SYSCALL_PARAM6(ctx)	(ctx)->ebp
#define IA32_SYSCALL_RETURN(ctx)	(ctx)->eax

#define IA32_SYSCALL_NONE		-1

/* constants for ptrace emulation */
#define SEMREADY			0
#define SEMRESUME			1
#define FINISH_WAIT			1
#define DISCARD_WAIT			2
#define PTEMU_SIG0			SIG_RT_MAX
#define SI_BTRANS			0xc0000000
#define PTEMU_CANCEL			0x00080000000ul

/* 
 * this constants are used in the ptrace field of the tls
 * they all should be at the higher dword of the field
 * the lower dword is kept for cont_signo
 */
#define PTEMU_PTRACED			0x00100000000ul
#define PTEMU_SYSCALL			0x00200000000ul
#define PTEMU_CONT			0x00400000000ul
#define PTEMU_NEW			0x00800000000ul
#define PTEMU_DISCARD			0x01000000000ul
#define PTEMU_SELF_CANONIZE		0x02000000000ul
#define PTEMU_SIG_PROCESSED		0x04000000000ul
#define PTEMU_INSIDE_SYSCALL		0x08000000000ul
#define PTEMU_NEWLY_ATTACHED		0x08000000000ul
#define PTEMU_DETACH			0x10000000000ul
#define PTEMU_DETACH_LAST_THREAD	0x20000000000ul
#define PTEMU_IN_CONT_PROCESS		0x40000000000ul
#define PTEMU_SUSPENSION_REQ		0x80000000000ul

/* The argument structure for the legacy IA-32 mmap() function. */
typedef struct legacy_mmap_args_t {
	unsigned addr;
	unsigned len;
	unsigned prot;
	unsigned flags;
	unsigned fd;
	unsigned offset;
} legacy_mmap_args_t;

/*****************************************************************************/
/* SYSCALLS */
/*****************************************************************************/
#define IA32_SYSCALL_INT                0x80

#define IA32_SYSCALL_EXIT		1ul
#define IA32_SYSCALL_FORK		2ul
#define IA32_SYSCALL_READ		3ul
#define IA32_SYSCALL_WRITE		4ul
#define IA32_SYSCALL_OPEN		5ul
#define IA32_SYSCALL_CLOSE		6ul
#define IA32_SYSCALL_WAITPID		7ul
#define IA32_SYSCALL_EXECVE		11ul
#define IA32_SYSCALL_PTRACE		26ul
#define IA32_SYSCALL_PAUSE		29ul
#define IA32_SYSCALL_ACCESS		33ul
#define IA32_SYSCALL_KILL		37ul
#define IA32_SYSCALL_DUP		41ul
#define IA32_SYSCALL_TIMES		43ul
#define IA32_SYSCALL_BRK		45ul
#define IA32_SYSCALL_SIGNAL		48ul
#define IA32_SYSCALL_IOCTL		54ul
#define IA32_SYSCALL_FCNTL		55ul
#define IA32_SYSCALL_SIGACTION		67ul
#define IA32_SYSCALL_SIGSUSPEND		72ul
#define IA32_SYSCALL_SIGPENDING		73ul
#define IA32_SYSCALL_SELECT		82ul
#define IA32_SYSCALL_READLINK		85ul
#define IA32_SYSCALL_USELIB		86ul
#define IA32_SYSCALL_MMAP		90ul
#define IA32_SYSCALL_MUNMAP		91ul
#define IA32_SYSCALL_SOCKETCALL		102ul
#define IA32_SYSCALL_WAIT4		114ul
#define IA32_SYSCALL_IPC		117ul
#define IA32_SYSCALL_SIGRETURN		119ul
#define IA32_SYSCALL_CLONE		120ul
#define IA32_SYSCALL_NEWUNAME		122ul
#define IA32_SYSCALL_MODIFY_LDT		123ul
#define IA32_SYSCALL_MPROTECT		125ul
#define IA32_SYSCALL_SIGPROCMASK	126ul
#define IA32_SYSCALL_GETPGID		132ul
#define IA32_SYSCALL_PERSONALITY	136ul
#define IA32_SYSCALL_LLSEEK		140ul
#define IA32_SYSCALL_NEWSELECT		142ul
#define IA32_SYSCALL_READV		145ul
#define IA32_SYSCALL_WRITEV		146ul
#define IA32_SYSCALL_GETSID		147ul
#define IA32_SYSCALL_NANOSLEEP		162ul
#define IA32_SYSCALL_MREMAP		163ul
#define IA32_SYSCALL_POLL		168ul
#define IA32_SYSCALL_RT_SIGRETURN	173ul
#define IA32_SYSCALL_RT_SIGACTION	174ul
#define IA32_SYSCALL_RT_SIGPROCMASK	175ul
#define IA32_SYSCALL_RT_SIGPENDING	176ul
#define IA32_SYSCALL_RT_SIGTIMEDWAIT	177ul
#define IA32_SYSCALL_RT_SIGSUSPEND	179ul
#define IA32_SYSCALL_GETCWD		183ul
#define IA32_SYSCALL_SIGALTSTACK	186ul
#define IA32_SYSCALL_VFORK		190ul
#define IA32_SYSCALL_MMAP2		192ul
#define IA32_SYSCALL_STAT64		195ul
#define IA32_SYSCALL_LSTAT64		196ul
#define IA32_SYSCALL_GETDENTS64		220ul
#define IA32_SYSCALL_FCNTL64		221ul
#define IA32_SYSCALL_TKILL		238ul
#define IA32_SYSCALL_FUTEX		240ul
#define IA32_SYSCALL_SCHED_SETAFFINITY  241ul
#define IA32_SYSCALL_SET_THREAD_AREA 	243ul
#define IA32_SYSCALL_GET_THREAD_AREA 	244ul
#define IA32_SYSCALL_IO_SETUP		245ul
#define IA32_SYSCALL_IO_DESTROY		246ul
#define IA32_SYSCALL_IO_GETEVENTS	247ul
#define IA32_SYSCALL_IO_SUBMIT		248ul
#define IA32_SYSCALL_IO_CANCEL		249ul
#define IA32_SYSCALL_EXIT_GROUP		252ul
#define IA32_SYSCALL_EPOLL_CREATE	254ul
#define IA32_SYSCALL_EPOLL_CTL		255ul
#define IA32_SYSCALL_EPOLL_WAIT		256ul
#define IA32_SYSCALL_SET_TID_ADDRESS	258ul
#define IA32_SYSCALL_TGKILL		270ul

/*****************************************************************************/
/* STACK */
/*****************************************************************************/
#define IA32_STACK_EXEC_DEFAULT	0
#define IA32_STACK_EXEC_ENABLE	1
#define IA32_STACK_EXEC_DISABLE	2

/*****************************************************************************/
/* SIGNALS */
/*****************************************************************************/
#define SIG_NUM_TOTAL         64
#define SIG_RT_MIN	      32
#define SIG_RT_MAX	      64

#define SIG_NUM_PER_WORD      32
#define WORDS_IN_MASK         (SIG_NUM_TOTAL / SIG_NUM_PER_WORD)

#define MINSIGSTKSZ32         2048

#define SI_FROMUSER(siptr)    ((siptr)->si_code <= 0)
#define SI_FROMKERNEL(siptr)  ((siptr)->si_code > 0)

#define SA_RESTORER           0x04000000
#define IA32_EFLAG            0x200
#define IA32_FSR_DEFAULT      0x55550000u	/* set all tag bits */
#define IA32_FCR_DEFAULT      0x17800000037fu	/* extended precision, all masks */

/* order of registers in IA-32 user struct (used for ptrace) */
#define EBX 0
#define ECX 1
#define EDX 2
#define ESI 3
#define EDI 4
#define EBP 5
#define EAX 6
#define DS  7
#define ES  8
#define FS  9
#define GS  10
#define ORIG_EAX 11
#define EIP 12
#define CS  13
#define EFL 14
#define UESP 15
#define SS   16

#define IA64_SC_FLAG_ONSTACK_BIT        0       /* is handler running on signal stack? */
#define IA64_SC_FLAG_IN_SYSCALL_BIT     1       /* did signal interrupt a syscall? */
#define IA64_SC_FLAG_FPH_VALID_BIT      2       /* is state in f[32]-f[127] valid? */

#define IA64_SC_FLAG_ONSTACK            (1 << IA64_SC_FLAG_ONSTACK_BIT)
#define IA64_SC_FLAG_IN_SYSCALL         (1 << IA64_SC_FLAG_IN_SYSCALL_BIT)
#define IA64_SC_FLAG_FPH_VALID          (1 << IA64_SC_FLAG_FPH_VALID_BIT)

#define NUM_SECS_IN_THREAD_SCALABILITY_INTERVAL 10

/* debug definitions */
#if DEBUG

#define DBCODE(cond, expr)	do{if(cond){expr;}}while(0)

#if USE_ROLLING_TRACE
#define fprintf(fd,format,args...)				\
do {								\
	char fdprintf_buffer[4096];				\
	sprintf (fdprintf_buffer, format, ## args);		\
	btlib_insert_rolling_trace_str (fdprintf_buffer);	\
} while (0)
#endif				// USE_ROLLING_TRACE

#define RELEASE_UNUSED

#else				/* no DEBUG */
#define DBCODE(cond, expr)	/* nothing */
#define RELEASE_UNUSED __attribute__ ((unused))
#endif				/* DEBUG */

#if STAT
#define STATCODE(cond, expr) do { if (cond) expr; } while (0)
#else
#define STATCODE(cond, expr)		/* nothing */
#endif				/* STAT */

#define UNUSED __attribute__ ((unused))

#define ILP32_WRAPPER0(id)				\
	ilp32_wrapper((unsigned long)id, 0ul, 0ul, 0ul, 0ul, 0ul, 0ul)
#define ILP32_WRAPPER1(id, a2)				\
	ilp32_wrapper((unsigned long)id, (unsigned long)a2, 0ul, 0ul, 0ul, 0ul, 0ul)
#define ILP32_WRAPPER2(id, a2, a3)			\
	ilp32_wrapper((unsigned long)id, (unsigned long)a2, (unsigned long)a3, 0ul, 0ul, 0ul, 0ul)
#define ILP32_WRAPPER3(id, a2, a3, a4)			\
	ilp32_wrapper((unsigned long)id, (unsigned long)a2, (unsigned long)a3, (unsigned long)a4, 0ul, 0ul, 0ul)
#define ILP32_WRAPPER4(id, a2, a3, a4, a5)		\
	ilp32_wrapper((unsigned long)id, (unsigned long)a2, (unsigned long)a3, (unsigned long)a4, (unsigned long)a5, 0ul, 0ul)
#define ILP32_WRAPPER5(id, a2, a3, a4, a5, a6)		\
	ilp32_wrapper((unsigned long)id, (unsigned long)a2, (unsigned long)a3, (unsigned long)a4, (unsigned long)a5, (unsigned long)a6, 0ul)
#define ILP32_WRAPPER6(id, a2, a3, a4, a5, a6, a7)	\
	ilp32_wrapper((unsigned long)id, (unsigned long)a2, (unsigned long)a3, (unsigned long)a4, (unsigned long)a5, (unsigned long)a6, (unsigned long)a7)

/* globals */
typedef unsigned long cs_handle;
struct signal_handler_table;
struct _debugee_t;
struct signal_pending_que_t;

extern API_TABLE_ELEMENT_TYPE btl_api_table_elements[];
extern unsigned btl_glst_size;	/* btgeneric's glst size */
extern unsigned btl_bc32;	/* the offset of cpu status inside glst */
extern unsigned long btl_page_size;	/* system page size */
extern void *btl_btg_load_address;
extern unsigned long btl_btg_size;
extern unsigned long btl_is_ptraced;
extern struct signal_pending_que_t *btl_shared_que;
extern int btl_btg_initialized;
#if ! RELEASE
extern char btl_no_affinity;
extern char btl_always_rie;
#endif
extern unsigned long btl_ls_before_break_inst;
extern char btl_ia32_bin_full_path[PATH_MAX];
extern char * btl_ia32_bin_name;
#if SHMEM_PROC_AFFINITY || SHMEM_ORDER_SOLUTION || ALL_THREAD_AFFINITY
extern int btl_narrowed_affinity;
extern int btl_did_ordered_code;
#endif

#if DEBUG
extern char btl_prt_sig;
extern char btl_prt_mem;
extern char btl_prt_cs;
extern char btl_prt_api;
extern char btl_prt_thread;
extern char btl_prt_syscall;
extern char btl_prt_pt;
#ifdef ALL_THREAD_AFFINITY
extern unsigned long btl_start_time;
extern unsigned long btl_cycles_in_second;
#endif
#endif
#ifdef ALL_THREAD_AFFINITY
extern unsigned long btl_cycles_in_second;
extern unsigned long btl_cyc_in_thrd_scl_intvl;
extern unsigned long btl_num_threads;
#endif

/* IA-32 Execution Layer initial binary map struct */
typedef struct {
	unsigned start_brk;
	unsigned start_code;
	unsigned end_code;
	unsigned start_data;
	unsigned end_data;
	unsigned start_stack;
	unsigned brk;
	unsigned entry_point;	/* the entry point of the interpreter */
	unsigned arg_start, arg_end, env_start, env_end;
	unsigned dl_base_addr;
	unsigned dl_size;
} btrans_binary_map_t;

/* btlib_uaccess export functions */
extern long safe_memcpy (void *dest, void *src, unsigned long len);
extern long safe_stringcopy (char * dest, char * src, unsigned long len);

/* btlib_ptemu export functions */
extern void btlib_emulate_trap (void);
extern void btlib_prepare_cont_signo (int cont_signo);
extern long btlib_ptemu_wait (int pid, int *status);
extern long btlib_ptrace (BTGENERIC_IA32_CONTEXT *);
extern void btlib_ptemu0_handler (int signo, siginfo_t * siginfo, struct sigcontext *ctx);
extern long btlib_peekdata (int pid, void *addr, void *buffer, unsigned long size);
extern long btlib_pokedata (int pid, void *addr, const char *buffer, unsigned long size);
extern long ia64_peekdata (int pid, unsigned long addr, unsigned long *dest);
extern long ia64_peekuser (int pid, unsigned long reg, unsigned long *dest);
extern long btlib_stop_debugee (struct _debugee_t *, unsigned, int, int *);
extern void btlib_init_ptemu_module (void);
/* btlib_thread export functionss */
extern long btlib_main_thread_init_tsd (void);
extern long btlib_main_thread_init (const btrans_binary_map_t *app_info, long personality);
extern long btlib_clone (BTGENERIC_IA32_CONTEXT *context);
extern long btlib_fork (BTGENERIC_IA32_CONTEXT *context);
extern void btlib_thread_exit (BTGENERIC_IA32_CONTEXT *context);
extern long btlib_suspend_thread (int pid, unsigned max_count);
extern long btlib_resume_thread (int pid);
extern void btlib_cleanup_thread (int pid);
extern void btlib_init_thread_module (void);
extern void btlib_group_exit (const BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_set_tid_address (const BTGENERIC_IA32_CONTEXT *bc32);
extern int btlib_init_tls_module (void);
extern int btlib_allocate_stack (void **stack, size_t *stacksize, void **tls);
extern int btlib_deallocate_stack (void *stackblock, int *clear_tid, int free_mem);
extern long suspend_all_threads (void);
extern long resume_all_threads (void);

#if ALL_THREAD_AFFINITY
extern void btlib_record_explicit_affinity_request (int target_tid);
extern void btlib_record_thread_enter_simulation (void);
extern void btlib_record_thread_exit_simulation (void);

#define SHOULD_TRACK_THREAD_SCLBLTY 	\
	((btl_num_threads > 1) && btl_narrowed_affinity && !btl_did_ordered_code)
#endif	/* ALL_THREAD_AFFINITY */
#if SHMEM_PROC_AFFINITY
extern void apply_shared_memory_affinity_solution (void);
#elif SHMEM_ORDER_SOLUTION
void apply_shared_memory_ordering_solution (void);
#endif  /* SHMEM_PROC_AFFINITY */

/* btlib_pending_que exports */
extern long btlib_que_is_empty (struct signal_pending_que_t *pque);
extern long btlib_que_is_real_empty (struct signal_pending_que_t *pque);
extern void btlib_que_enque (struct signal_pending_que_t *pque, siginfo_t *siginfo, 
                             unsigned long time_stamp);
extern int btlib_que_deque (siginfo_t *siginfo);
extern int btlib_que_lastsig (struct signal_pending_que_t *pque);
extern void btlib_que_remove_signo (struct signal_pending_que_t *pque, int signo);
extern void btlib_que_free (struct signal_pending_que_t **ppque);
extern void btlib_que_init (struct signal_pending_que_t **ppque);
extern void btlib_que_sigpending (struct signal_pending_que_t *pque, int size, sigset32_t *set);

/* btlib_signal export functions */
extern unsigned allocate_sighand_frame (sigcontext32_t *sc32, int flags, int is_info);
extern long set_sigmask (sigset32_t *mask);
extern void except2signal (BTGENERIC_IA32_CONTEXT *bc32, unsigned ic, siginfo_t *siginfo, 
                           int *trapno);
extern void btlib_init_signal_module (void);
extern int has_permanent_handler (int signo);
extern int has_half_permanent_handler (int signo);
extern void btlib_deliver_signals (void);
extern void btlib_signal_handler (int sig, siginfo_t *siginfo, struct sigcontext *linux_sigcontext);
extern struct signal_handler_table *btlib_create_sighand_table (void);
extern long btlib_sigaction (BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_rt_sigaction (BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_sigaltstack (BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_sigprocmask (BTGENERIC_IA32_CONTEXT *bc32, int is_rt);
extern long btlib_sigpending (BTGENERIC_IA32_CONTEXT *bc32, int is_rt);
extern void btlib_sigreturn (BTGENERIC_IA32_CONTEXT *cur_bc32, int is_rt);
extern void btlib_raise_signal (void);
extern void restart_btg (BTGENERIC_IA32_CONTEXT *new_bc32, sigset32_t *handler_mask);
extern void btlib_run_ia32_handler (siginfo_t *siginfo, sigset32_t *oldmask, unsigned int trapno);
extern void btlib_try_free_sighand_table (btlib_thread_data_t *);
extern void *get_interrupted_bsp (const unsigned long, const unsigned long);
extern void do_longjmp (void);
extern void fsave2btg (BTGENERIC_IA32_CONTEXT *bc32, struct i387_fsave_struct *fsave);
extern void btg2fsave (const BTGENERIC_IA32_CONTEXT *bc32, struct i387_fsave_struct *fsave);
extern void fxsave2btg (BTGENERIC_IA32_CONTEXT *bc32, struct i387_fxsave_struct *fxsave);
extern void btg2fxsave (const BTGENERIC_IA32_CONTEXT *bc32, struct i387_fxsave_struct *fxsave);
extern long btlib_signal (BTGENERIC_IA32_CONTEXT *bc32);

#if DEBUG
extern void print_btgcontext32 (BTGENERIC_IA32_CONTEXT *bc32);
extern void print_sigcontext64 (struct sigcontext *sc);
extern void print_btgcontext64 (BTGENERIC_IA64_CONTEXT *bc64);
extern void print_sigmask (sigset32_t *set, int size, char *header);
extern void print_current_sigmask (char *header);
extern unsigned long who_holds_critical_section (cs_handle ch);
extern void print_maps (void);
extern void unwind_ia32stack (void);
#endif	// DEBUG

/* btlib_memory export funcs */
extern void *btlib_mem_alloc (unsigned long start, size_t length, unsigned prot);
extern long btlib_mem_free (void *start, size_t length);
extern void btlib_init_memory_module (void);
extern long btlib_memory_query_permissions (void *address, void **region_start,
					    unsigned long *region_size);
extern long btlib_ia32_mremap (const BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_ia32_mmap (const BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_ia32_mmap2 (const BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_ia32_munmap (const BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_ia32_mprotect (BTGENERIC_IA32_CONTEXT *bc32);
extern long btlib_memory_change_permissions (unsigned long start,
					     unsigned long size, unsigned prot);

/* btlib_misc export functions */
extern void print_thread_data (btlib_thread_data_t *);
extern void btlib_abort_all (void);
extern void btlib_print_rolling_trace_array (void);
extern void btlib_abort (void);
extern void btlib_insert_rolling_trace_str (char *UNUSED str);
#if ALL_THREAD_AFFINITY
extern unsigned long btlib_get_cpu_mhz(void);
#endif


/* btlib_syscall export functions */
extern void btlib_system_call (void);

/* btlib_critical_section export functions */
extern void btlib_init_cs_module (void);
extern long btlib_init_critical_section (cs_handle ch);
extern long btlib_enter_critical_section (cs_handle ch, int block);
extern void btlib_leave_critical_section (cs_handle ch);
extern cs_handle btlib_create_critical_section (void);
extern void btlib_free_critical_section (cs_handle ch);
extern long is_holding_critical_section (btlib_thread_data_t *);

/* elf loader prototype */
long load_elf32_binary (int argc, char **argv, char **envp,
		btrans_binary_map_t *app_info, long *personality);



/* btlib_main exprot functions */
extern void btrans_jump (void);

/* access halves of qword */
static inline void
SET_LO_DWORD (unsigned long *_ldata, unsigned _sdata)
{
	union {
		unsigned long ldata;
		struct {
			unsigned lo;
			unsigned hi;
		} sdata;
	} _udata;
	_udata.ldata = *_ldata;
	_udata.sdata.lo = _sdata;
	*_ldata = _udata.ldata;
}

static inline unsigned
GET_LO_DWORD (unsigned long _ldata)
{
	union {
		unsigned long ldata;
		struct {
			unsigned lo;
			unsigned hi;
		} sdata;
	} _udata;
	_udata.ldata = _ldata;
	return _udata.sdata.lo;
}
static inline void
SET_HI_DWORD (unsigned long *_ldata, unsigned _sdata)
{
	union {
		unsigned long ldata;
		struct {
			unsigned lo;
			unsigned hi;
		} sdata;
	} _udata;
	_udata.ldata = *_ldata;
	_udata.sdata.hi = _sdata;
	*_ldata = _udata.ldata;
}
static inline unsigned
GET_HI_DWORD (unsigned long _ldata)
{
	union {
		unsigned long ldata;
		struct {
			unsigned lo;
			unsigned hi;
		} sdata;
	} _udata;
	_udata.ldata = _ldata;
	return _udata.sdata.hi;
}

/* convert int to pointer */
static inline void *
I2P (int addr)
{
	void *p;
	if ((unsigned long) (addr) > IA32_MAX_USER_ADDR) {
		__asm__ __volatile__ ("sxt4 %0=%1;;\n":"=r" (p):"r" (addr));
	} else {
		__asm__ __volatile__ ("zxt4 %0=%1;;\n":"=r" (p):"r" (addr));
	}
	return p;
}
static inline void *
U2P (unsigned addr)
{
	void *p;
	if ((unsigned long) (addr) > IA32_MAX_USER_ADDR) {
		__asm__ __volatile__ ("sxt4 %0=%1;;\n":"=r" (p):"r" (addr));
	} else {
		__asm__ __volatile__ ("zxt4 %0=%1;;\n":"=r" (p):"r" (addr));
	}
	return p;
}

/* return non-zero iff signal is not shared by all thread-group */
static inline long
IS_PRIVATE_SIG (siginfo_t *si)
{
	long ret;
	ret = (si->si_code & SI_TKILL) ? 1 : 0;
	return ret;
}
/* yield execution */
static inline void YIELD_THREAD_EXECUTION (void)
{
	SYSCALL (sched_yield, ());
}

/*****************************************************************************/
/* finish the cont process of the debuggee. clear the cont-process flag and
 * check whether there is a suspension request pending. if there is one then
 * stop self thread by sending ptemu-signal to self 
 * note that thare might be a gap between the point when the in-cont-process
 * flags is cleared and the actual point where btgenric is entered (which is
 * the point where the cont-process is really finished). this gap should not
 * be a problem here - the reason for the cont-process is to prevent btlib from
 * stopping while inside a critical code area. once this function is called
 * it is safe to restart execution of debuggee. a signal that will stop the
 * execution of the debuggee at a later stage can be considered as part of next
 * simulation session */
/* it is written as a macro and not static-inline function because we want the
 * to have the name of the function that called it in the debug printing */
/*****************************************************************************/
#if PTEMU_ENABLED
#define FINISH_CONT_PROCESS()							\
{										\
	int susp_req;								\
	susp_req = (cur_thread_data->ptrace & PTEMU_SUSPENSION_REQ) ? 1 : 0;	\
	cur_thread_data->ptrace &= ~ (PTEMU_IN_CONT_PROCESS | PTEMU_SUSPENSION_REQ); \
	if (susp_req) {								\
		DBPRINT (btl_prt_pt, "PTEMU_SUSPENSION_REQ\n");			\
		SYSCALL (tkill, (cur_thread_data->tid, PTEMU_SIG0));		\
	}		\
}
#else
#define FINISH_CONT_PROCESS()
#endif

#endif	/* BTLIB_H */

