/*  
 * btlib_core_file.h
 *  
 * definitions for IA-32 core file support
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _BTLIB_CORE_FILE_H
#define _BTLIB_CORE_FILE_H

#include <ia32x_utils.h>

extern void core_file(siginfo_t *siginfo);
extern int should_core_dump(int signo);

#define PROC_CORE_PATTERN "/proc/sys/kernel/core_pattern"
#define PROC_CORE_USES_PID "/proc/sys/kernel/core_uses_pid"

#define ROUND(a,b)  ((((a)+((b)-1))/(b))*(b))

#define MAX_MEMORY_REGIONS 10000
#define NT_PRXFPREG       0x46e62b7f /* copied from gdb/include/elf/common.h */		
#define MIN_COREDUMP 4096
#define IA32EL_DUMP_64_CORE_FILE "IA32EL_DUMP_64_CORE_FILE="

#define COREFILE_SYSCALL(syscall_name, syscall_params)                         \
({                                                                             \
	int _retval;                                                           \
	_retval = SYSCALL (syscall_name, syscall_params);                      \
	if (IS_SYS_ERROR (_retval)) {                                          \
		longjmp(cur_thread_data->btgeneric_run_context, m_fildes);     \
	}                                                                      \
	_retval;                                                               \
})

#define COREFILE_WRITE(size, syscall_params)                                       \
({                                                                                 \
	int _ret;                                                                  \
	if ((m_core_file_size + size) <= m_core_file_limit) {                      \
		_ret = SYSCALL (write, syscall_params);                            \
		if (IS_SYS_ERROR (_ret)) {                                         \
			longjmp(cur_thread_data->btgeneric_run_context, m_fildes); \
		}                                                                  \
		m_core_file_size += size;                                          \
	} else {                                                                   \
		COREFILE_SYSCALL (close, (m_fildes));                              \
		m_fildes = -1;                                                     \
		longjmp(cur_thread_data->btgeneric_run_context, m_fildes);         \
	}                                                                          \
	_ret;                                                                      \
})

enum core_policy_ind {
	SET_CORE_BIT=0,
	DONT_SET_CORE_BIT
};

/* from linux/elfcore.h */
typedef struct {
	Elf32_Nhdr nhdr;
	char *name;
	void *desc;
}note;

/* from /usr/include/sys/procfs.h */
typedef struct elf_prstatus prstatus_t;
typedef struct elf_prpsinfo prpsinfo_t;
typedef unsigned int elf_greg_t;
typedef elf_greg_t elf_gregset_t[17];

struct elf_siginfo {
	int si_signo;			/* Signal number.  */
	int si_code;			/* Extra code.  */
	int si_errno;			/* Errno.  */
};

struct elf_prstatus {
	struct elf_siginfo pr_info;		/* Info associated with signal.  */
	short int pr_cursig;		/* Current signal.  */
	unsigned int pr_sigpend;    	/* Set of pending signals.  */
	unsigned int pr_sighold;    	/* Set of held signals.  */
	__pid_t32 pr_pid;
	__pid_t32 pr_ppid;
	__pid_t32 pr_pgrp;
	__pid_t32 pr_sid;
	struct timeval32 pr_utime;		/* User time.  */
	struct timeval32 pr_stime;		/* System time.  */
	struct timeval32 pr_cutime;		/* Cumulative user time.  */
	struct timeval32 pr_cstime;		/* Cumulative system time.  */
	elf_gregset_t pr_reg;		/* GP registers.  */
	int pr_fpvalid;			/* True if math copro being used.  */
};

#define ELF_PRARGSZ     (80)    /* Number of chars for args */

struct elf_prpsinfo {
	char pr_state;			/* Numeric process state.  */
	char pr_sname;			/* Char for pr_state.  */
	char pr_zomb;			/* Zombie.  */
	char pr_nice;			/* Nice val.  */
	unsigned int pr_flag;		/* Flags.  */
	unsigned short pr_uid;
	unsigned short pr_gid;
	int pr_pid, pr_ppid, pr_pgrp, pr_sid;
	/* Lots missing */
	char pr_fname[16];			/* Filename of executable.  */
	char pr_psargs[ELF_PRARGSZ];	/* Initial part of arg list.  */
};

struct i387_fsave_core_struct {
	unsigned cwd;
	unsigned swd;
	unsigned twd;
	unsigned fip;
	unsigned fcs;
	unsigned foo;
	unsigned fos;
	unsigned st_space[20];	/* 8*10 bytes for each FP-reg = 80 bytes */
};

typedef struct thread_core_info_t {
	int tid;
	struct thread_core_info_t *next;
	prstatus_t eprstat;
	struct i387_fsave_core_struct fpu;
	struct i387_fxsave_struct xfpu;
	note prstatus_note;
	note fpu_note;
	note xfpu_note;
} thread_core_info_t;


#endif				// _BTLIB_CORE_FILE_H
