/* 
 * btlib_main.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>


#include "btlib.h"
#include "btlib_sync.h"
#include "btload.src/os_dep_linux.h"
#include "btload.src/new_pe.h"
#include "btload.src/btload_pub.h"
#include "btlib_stats.h"
#include "btlib_ptemu.h"
#include "btlib_syscall.h"

#include <ia32toia64.h>

#if DEBUG
/* Config file and log file handles */
char btl_prt_cs = 0;
char btl_prt_api = 0;
char btl_prt_sig = 0;
char btl_prt_pt = 0;
char btl_prt_mem = 0;
char btl_prt_thread = 0;
char btl_prt_syscall = 0;
#endif				/* DEBUG */
#if ! RELEASE
char btl_no_affinity = 0;
/* specify if always set READ_IMPLIES_EXEC */
char btl_always_rie = 0;
#endif

#define MAX_SWITCHES            16
#define MAX_SWITCH_LEN          1024
#define MAX_INI_BUF_SIZE        MAX_SWITCHES * MAX_SWITCH_LEN

/* globals */
unsigned btl_glst_size = 0;	        /* btgeneric's glst size */
unsigned btl_bc32 = 0;                  /* the offset of cpu status inside glst */
unsigned long btl_page_size = 0ul;      /* system page size */
void *btl_btg_load_address = (void *) BTGENERIC_LOAD_ADDRESS;
unsigned long btl_btg_size = 0ul;
char btl_ia32_bin_full_path[PATH_MAX] = { 0 };       /* full path to IA-32 executable binary */
char * btl_ia32_bin_name = NULL;       /* IA-32 executable binary name */
int btl_btg_initialized = 0;
btrans_binary_map_t g_application_info;         /* IA-32 application information */
char **g_envp;

#if ALL_THREAD_AFFINITY
#if DEBUG
unsigned long btl_cycles_in_second;
#endif
unsigned long btl_cyc_in_thrd_scl_intvl;
unsigned long btl_num_threads = 1;
#endif



/* information for locating BTGeneric */
#define HOME		"HOME="
#define BTGENERIC_EFI	"ia32exec.bin"
char *BTGeneric_search_path[] = { "/usr/local/bin",
	"/usr/bin",
	"/bin",
	"/usr/lib/ia32el",
	""
};
static char BTGeneric_filename[PATH_MAX] = { 0 };
#if !RELEASE
static char m_ini_file[PATH_MAX] = { 0 };
#endif
#define INI_FILE_NAME "Btrans.ini"
#define LOG_FILE_NAME "Btrans.log"

#define RLIM_NOFILE_EXT 2

/* function prototypes */
static void print_ver (char *btg_ver);
static int extend_rlimit_nofile (struct rlimit *);
static long btlib_init (int, char **, char **, btrans_binary_map_t *, long *);
static long set_btgeneric_path (char *dest, unsigned long dest_size, char **envp);
#if ! RELEASE
static long read_config_file (char *argv0, char *btrans_ini_switches[]);
static void open_log_file (char * line, char * argv0, char * logdir);
#endif
static long load_and_init_btgeneric (char **argv RELEASE_UNUSED, char **envp,
				     long is_ver, char btg_ver[PATH_MAX]);
static void report_initial_code_areas (btrans_binary_map_t * app_info);
static void extract_unicode_str (char *ver_str_p, char *ver_str,
				 unsigned long ver_str_size);
static void get_version_str (void *rsrc, unsigned long size, char *ver_str,
			     unsigned long ver_str_size);
static char *find_unicode_str (char *base_addr, unsigned long limit, char *str);
static int shift_argv (int argc, char *argv[]);

#if ALL_THREAD_AFFINITY
#if DEBUG
#include "btlib_asm.h"
#endif
#endif

/*****************************************************************************/
/*****************************************************************************/
int
main (int argc, char *argv[], char *envp[])
{
	long retval;
	struct rlimit orig_rlim;
	long is_ver = 0;
	long rlimit_changed = 0;
	char btg_ver[PATH_MAX];
	long personality;

	/* 
	 * argv[0] is the translator's name
	 * argv[1] is full path to the translated process
	 * argv[2] is the original argv[0]
	 */
	if (argc < 2) {
		retval = 0;
		goto out;
	}

	g_log_file = -1;


	/* check validity of arguments */
	if ((argv[1][0] == '-') && (argv[1][1] == 'v')) {
		if (argv[1][2] != '\0') {
			retval = -EINVAL;
			goto out;
		}
		is_ver = 1;
	}

	/* prepare the tsd of the main thread - don't emit any debug
	 * message before this point */
	btl_page_size = SYSCALL (getpagesize, ());
	retval = btlib_main_thread_init_tsd ();
	if (retval < 0) {
		goto out;
	}
#if (DEBUG && 0)
	{
		int k;
		for (k = 0; k < argc; k++) {
			fprintf (g_log_file, "\targv[%d]='%s'\n", k, argv[k]);
		}
	}
#endif

	if (!is_ver) {
		/* btlib needs 2 more open files than the orig binary */
		rlimit_changed = extend_rlimit_nofile (&orig_rlim);
		if (rlimit_changed < 0) {
			DBPRINT (0, "btl_ia32_bin_name='%s'\n",
			       btl_ia32_bin_name);
			goto out;
		}
		/* initialize btlib */
		retval = btlib_init (argc, argv, envp,
				&g_application_info,
				&personality);
		if (retval < 0) {
			DBPRINT (TRUE, "btlib_init failed\n");
			goto out;
		}
	}

	/* load and initialize btgeneric */
	retval = load_and_init_btgeneric (argv, envp, is_ver, btg_ver);
	if (is_ver) {
		print_ver ((retval < 0) ? NULL : btg_ver);
		retval = 0;
		goto out;
	}
	if (retval < 0) {
		DBPRINT (TRUE, "load_and_init_btgeneric failed\n");
		retval = -ENOEXEC;
		notify_error(NO_BTGENERIC);
		goto out;
	}

	btl_btg_initialized = 1;

	/* set back the old RLIMIT_NOFILE */
	if (rlimit_changed) {
#if DEBUG
		/* need one more for the log file */
		orig_rlim.rlim_cur += 1;
#endif
		retval = SYSCALL (setrlimit, (RLIMIT_NOFILE, &orig_rlim));
		if (IS_SYS_ERROR (retval)) {
			DBPRINT (TRUE,
			         "Unable to set RLIMIT_NOFILE back with rlim_max:"
			         "%d, rlim_cur:%d\n", 
				 (int) orig_rlim.rlim_max,
			         (int) orig_rlim.rlim_cur);
			return retval;
		}
	}

	/* remove first two arguments from argv so 'ps -ef' will show the
	 * proess name and not the translator's name */
	shift_argv (argc, argv);

#if (DEBUG && 0)	/* print the IA-32 stack */
	{
		unsigned char *p;
		long k = 0;
		for (p = (unsigned char *) (unsigned long) g_application_info.start_stack;
		     p < (unsigned char *) IA32_MAX_USER_ADDR; p++) {
			if (k == 0) {
				fprintf (g_log_file, "%p:", p);
			}
			fprintf (g_log_file, "\t%x", *p);
			if (++k == 16) {
				fprintf (g_log_file, "\n");
				k = 0;
			}
		}
	}
#endif	/* DEBUG */

	/* prepare the current thread for running the IA-32 application */
	retval = btlib_main_thread_init (&g_application_info, personality);
	if (retval < 0) {
		goto out;
	}

	/* report to btgeneric about the initial code areas 
	 * (binary and dynamic loader) */
	report_initial_code_areas (&g_application_info);

	/* silent the misaligned-access messages */
	retval = SYSCALL (prctl, (PR_SET_UNALIGN, (unsigned long)PR_UNALIGN_NOPRINT));

#if DEBUG
	if (IS_SYS_ERROR (retval)) {
		DBPRINT (TRUE, "prctl failed:%ld\n", retval);
	}
#endif

#if PTEMU_ENABLED

	DBPRINT (btl_prt_pt, "btl_is_ptraced=%ld r13=%p\n", btl_is_ptraced, __current);
	/* if this process is traced then suspend yourself, simulate the behavior when 
	   a ptraced process call execve */
	if (btl_is_ptraced != IS_PTRACED_NO) {
		assert (cur_thread_data->tid == SYSCALL (gettid, ()));
		/* set execution policy of debuggee */
		{
			EMULATION_POLICY ep = {
				/* memory order should not be changed */
				memory_order_policy: USE_ORDERED_LD_ST,
				/* don't optimize away changes in eflag */			
				strict_state_policy: STATE_POLICY_PRESERVE_EFLAGS,
			};
			BTGENERIC_SET_EMULATION_POLICY(&ep);
			assert (ep.strict_state_policy == STATE_POLICY_PRESERVE_EFLAGS);
			/* we don't need flush instruction cache here because there 
			   are no translations yet */
		}


		/* suspend */
		cur_thread_data->tia_signal.siginfo.si_signo = SIGTRAP;
		retval = SYSCALL (tkill, (cur_thread_data->tid, PTEMU_SIG0));
		sys_err_chk (retval, "kill");
	}
#endif

#if ALL_THREAD_AFFINITY
#if DEBUG
	btl_cycles_in_second = btlib_get_cpu_mhz() * 1000000L;
	btl_start_time = __btlib_read_timestamp_counter();
#endif
	btl_cyc_in_thrd_scl_intvl 
		= btlib_get_cpu_mhz() * 1000000L * NUM_SECS_IN_THREAD_SCALABILITY_INTERVAL;
#endif
	/* enter the IA-32 process */
	btrans_jump ();

	DBPRINT (TRUE, "SHOULD NOT GET HERE !!!\n");

      out:
	return retval;
}

/*****************************************************************************/
/*
 * static int * btlib_init(char ** envp)
 *
 * Purpose: initialize btlib 
 *
 * Input:   pointer to stacked arguments (as passed to main)
 * Output:  ia32stack pointer
 *
 */
/*****************************************************************************/
static long
btlib_init (int argc, char **argv, char **envp,
		btrans_binary_map_t *app_info,
		long *personality)
{
	long retval;

	/* init the sync module */
	btlib_init_sync_module ();

	/* init the memory module */
	btlib_init_memory_module ();

	/* init the signal module */
	btlib_init_signal_module ();

	/* init thread module */
	btlib_init_thread_module ();

#if PTEMU_ENABLED
	/* init ptrace-emulation module */
	btlib_init_ptemu_module ();
#endif

	/* 1. Initialize globals . */
	memset (app_info, 0, sizeof (*app_info));

	/* initialize the IA-32 syscall wrapper library - it is needed 
	 * in load_elf32_binary */
	retval = iti_init ((int) btl_page_size, safe_memcpy);
	if (retval < 0) {
		DBPRINT (TRUE, "ia32toia64_init failed\n");
		return retval;
	}

	/* 2. load the IA-32 binary */
	assert (argc > 1);
	retval = load_elf32_binary (argc, &argv[0], envp,
			app_info, personality);
	if (retval < 0) {
		return retval;
	}
#if (DEBUG & 0)
	fprintf (g_log_file, "app_info:\n");
	fprintf (g_log_file, "\tstart_brk=%x\n", app_info->start_brk);
	fprintf (g_log_file, "\tstart_code=%x\n", app_info->start_code);
	fprintf (g_log_file, "\tend_code=%x\n", app_info->end_code);
	fprintf (g_log_file, "\tstart_data=%x\n", app_info->start_data);
	fprintf (g_log_file, "\tend_data=%x\n", app_info->end_data);
	fprintf (g_log_file, "\tstart_stack=%x\n", app_info->start_stack);
	fprintf (g_log_file, "\tbrk=%x\n", app_info->brk);
	fprintf (g_log_file, "\tentry_point=%x\n", app_info->entry_point);
	fprintf (g_log_file, "\targ_start=%x\n", app_info->arg_start);
	fprintf (g_log_file, "\targ_end=%x\n", app_info->arg_end);
	fprintf (g_log_file, "\tenv_start=%x\n", app_info->env_start);
	fprintf (g_log_file, "\tenv_end=%x\n", app_info->env_end);
#endif				/* DEBUG */

	/* set the brk values in the IA-32 syscall wrapper library */
	retval = iti_set_brk (app_info->end_code, app_info->brk);
	if (retval < 0) {
		DBPRINT (TRUE, "set_brk_values failed\n");
		return -1;
	}

	/* below func uses IA-32 syscall wrapper, so put it here */
	retval = btlib_init_syscall_module ();
	if (retval < 0) {
		DBPRINT (TRUE, "btlib_init_syscall_module failed\n");
		return -1;
	}

	return 0;
}

/*****************************************************************************/
/*
 * void btrans_jump
 *
 * Purpose: Jump into BTGeneric_Run
 *
 * Input: thread->longjmp_status indicates why we are here:
 *          RUN = just jump to BTGeneric
 *          SIGNAL = call the signal raising function
 *          DELIVER = call the signal delivery function
 *
 * Output:
 *
 */
/*****************************************************************************/
void
btrans_jump ()
{
	int status;
	long retval;

	assert (cur_thread_data->tid == SYSCALL (gettid, ()));

	status = sigsetjmp (cur_thread_data->btgeneric_run_context, 0);

	/* get the thread's specific data (in case we got here from __siglongjmp) */
	assert (cur_thread_data->tid == SYSCALL (gettid, ()));

	DBPRINT (0, "status=%d ptrace=%lx\n", status, cur_thread_data->ptrace);

	STATCODE (btl_stats, btlib_update_stat (BTL_STAT_BTGJUMP, COUNT, 0ul));
	
	if (status == 0) {
		cur_thread_data->is_run_context_ready = cur_thread_data->tid;
	}
	/* 
	 * this is the debugee's part of the algorithm for sync_self_canonization.
	 * look at the header of this function in btlib_ptemu.c
	 */
	if (cur_thread_data->ptrace & PTEMU_PTRACED) {
		int cont_signo;

		if (cur_thread_data->ptrace & PTEMU_SELF_CANONIZE) {
			int semid;
			struct sembuf semops[2];

			cur_thread_data->ptrace &= ~PTEMU_SELF_CANONIZE;

			DBPRINT (btl_prt_pt, "self canonization\n");

			/* we should get here only thru BtlReenter */
			assert (status == RUN);

			/* Open the semaphore (allocated by the debugger) */
			semid = SYSCALL (semget, (cur_thread_data->self_canon_semkey, 2, 0600));
			sys_err_chk ((long) semid, "semget");

			/* Wake the debugger */
			semops[0].sem_num = SEMREADY;

			/* TODO: 1 should be enough, but it's not */
			semops[0].sem_op = 2;
			semops[0].sem_flg = 0;
			retval = SYSCALL (semop, (semid, semops, 1));

			/* wait for the debugger */
			DBPRINT (btl_prt_pt, "before wait\n");
			semops[0].sem_num = SEMRESUME;
			semops[0].sem_op = -1;
			semops[0].sem_flg = SEM_UNDO;
			/* don't let signals interrupt us here */
			do {
				retval = SYSCALL (semop, (semid, semops, 1));
			} while (retval == -EINTR);
			sys_err_chk (retval, "semop");
			DBPRINT (btl_prt_pt, "after wait\n");

			/* Remove the semaphore set */
			retval = SYSCALL (semctl, (semid, 0, IPC_RMID, (union semun) 0));
			sys_err_chk (retval, "semctl");
		}
		/* get the signal to continue with */
		cont_signo = GET_LO_DWORD (cur_thread_data->ptrace);
		SET_LO_DWORD (&cur_thread_data->ptrace, 0);
		DBPRINT (btl_prt_pt, "cont_signo=%d\n", cont_signo);
		if (cont_signo != 0) {
			if (cont_signo == PTEMU_SIG0) {
				retval = SYSCALL (tkill, (cur_thread_data->tid, PTEMU_SIG0));
				sys_err_chk (retval, "suspend self");
			} else {
				btlib_prepare_cont_signo (cont_signo);
				status = SIGNAL;
			}
		}
	}

	switch (status) {
	case 0:
	case RUN:
		DBPRINT (0, "eip=%x esp=%x eax=%x ebx=%x ecx=%x edx=%x\n",
				cur_thread_data->bc32->eip,
				cur_thread_data->bc32->esp,
				cur_thread_data->bc32->eax,
				cur_thread_data->bc32->ebx,
				cur_thread_data->bc32->ecx,
				cur_thread_data->bc32->edx);
#ifdef ALL_THREAD_AFFINITY
		if (SHOULD_TRACK_THREAD_SCLBLTY) {
			btlib_record_thread_enter_simulation ();
		}
#endif
		assert (!btlib_is_inside_lock (cur_thread_data));
		FINISH_CONT_PROCESS ();
		BTGENERIC_RUN ();
		break;
	case DELIVER:
		btlib_deliver_signals ();
		break;
	case SIGNAL:
		btlib_raise_signal ();
		break;
	case IA32_SYSCALL:
		btlib_system_call ();
		break;
	case TRAP:
		btlib_emulate_trap ();
		break;
	default:
		assert (!"unknown status");
	}
	assert (!"should never get here");
}

/*****************************************************************************/
/*
 * set_btgeneric_path:
 * search ia32exec.bin in order: predefined path, HOME, current dir
 */
/*****************************************************************************/
static long
set_btgeneric_path (char *dest, unsigned long dest_size, char **envp UNUSED)
{
	long   retval = -1;
	struct stat stat_buf[3];  /* 3 due to different struct defs in the old headers */
	char   buff[PATH_MAX];
	char   **search_path;

	assert (sizeof (buff) >= dest_size);

	/* assign here to avoid unused warning when SYSBTG=0 */
	search_path = BTGeneric_search_path;

	if (envp != (char **) NULL) {
		g_envp = envp;
	}

#if SYSBTG
	while (strnlen (*search_path, (unsigned long)PATH_MAX) != 0) {
		strncpy (buff, *search_path, sizeof(buff) - 1);
		strncat (buff, "/"BTGENERIC_EFI, 
			 MIN(sizeof(buff) - strlen(buff) - 1, strlen(BTGENERIC_EFI) + 1));
		retval = SYSCALL (stat, (buff, stat_buf));

		if (retval == 0) {
			strncpy (dest, buff, dest_size - 1);
#if !RELEASE
			sprintf (m_ini_file, "%s/" INI_FILE_NAME, *search_path);
#endif
			break;
		} else {
			retval = -1;
			search_path++;
		}
	}
#else
 	/* try current directory first */
 	assert (strlen ("./"BTGENERIC_EFI) < sizeof (buff) - 1);
	strncpy (buff, "./"BTGENERIC_EFI, sizeof (buff) - 1);
	retval = SYSCALL (stat, (buff, stat_buf));
	if (retval == 0) {
		strncpy (dest, buff, dest_size - 1);
		goto out;
	} 

	/* try to look for the HOME environment variable */
	size_t home_len = strnlen (HOME, (unsigned long)PATH_MAX);

	if (envp != (char **) NULL) {
		/* Go through the environment strings until we find "HOME=" */
		while (*envp) {
			if (strncmp (*envp, HOME, home_len) == 0) {
				break;
			} else {
				envp++;
			}
		}

		/*
		 * If *envp is not NULL, then we found HOME - 
		 * create btgeneric's path
		 */
		if (*envp) {
 			unsigned long allowed_len = 
				sizeof (buff) - 
				strnlen ("/"BTGENERIC_EFI, (unsigned long)NAME_MAX) - 1;
 			unsigned long actual_len = 
				strnlen (*envp + home_len, allowed_len + 1);

 			assert ((long)allowed_len > 0);
 			if (actual_len < allowed_len) {
 				strncpy (buff, *envp + home_len, allowed_len);
 				assert (buff[actual_len] == '\0');

				strncat (buff, "/"BTGENERIC_EFI, 
			 		 MIN(sizeof(buff) - strlen(buff) - 1, 
					     strlen(BTGENERIC_EFI) + 1));
				retval = SYSCALL (stat, (buff, stat_buf));
				if (retval == 0) {
					strncpy (dest, buff, dest_size - 1);
				}
			}
		}
	}

out:
	/* ini file is in current directory */
#if !RELEASE
	if (retval == 0) {
		sprintf (m_ini_file, "./"INI_FILE_NAME);
	}
#endif

#endif
	DBPRINT (0, "BTGeneric_filename='%s'\n", BTGeneric_filename);
	return retval;
}

#if ! RELEASE
/*****************************************************************************/
/* read the configuration file (Btrans.ini) */
/*****************************************************************************/
static long
read_config_file (char *argv0 UNUSED, char *btrans_ini_switches[])
{
	long  ret;
	char  IniBuffer[MAX_INI_BUF_SIZE] = "";
	long  btrans_ini_switches_num = 0;
	int   bt_IniFile = -1;
	static char btrans_ini_switches_array[MAX_SWITCHES][MAX_SWITCH_LEN];
	char  logdir[PATH_MAX] = { 0 };
	char  *p = IniBuffer, *q;

	DBPRINT (0, "m_ini_file='%s'\n", m_ini_file);

	bt_IniFile = SYSCALL (open, (m_ini_file, O_RDONLY, 0777));

	if (bt_IniFile <= 2) {
		DBPRINT (FALSE, "Can't open Btrans.ini file\n");
		return 0;
	}

	ret = SYSCALL (read, (bt_IniFile, (void *) IniBuffer,
		       (size_t) MAX_INI_BUF_SIZE - 1));

	if (IS_SYS_ERROR (ret)) {
		DBPRINT (TRUE, "Can't read Btrans.ini file\n");
		return 0;
	}

	*(p + ret) = '\0';
	while (*p != '\0') {
		/* skip all the new line chars */
		q = strchr (p + 1, '\n');
		if (q != NULL) {
			/* handle ini files created on NT */
			if (*(q - 1) == '\r') {
				q--;
			}
			while ((*q == '\n') || (*q == '\r')) {
				*q++ = '\0';
			}
		}

		/* 
		 * the log option gets special treatment because it's btlib who
		 *  needs to open the log file
		 */
		if (strncmp (p, "logdir", 6ul) == 0) {
			p += 6;
			if ((*p == '=') && (*(p++) != '\0')) {
				strncpy (logdir, p, sizeof (logdir) - 1);
			}
		} else if (strncmp (p, "log", 3ul) == 0) {
			open_log_file (p, argv0, logdir);
#ifdef STAT
		} else if (strncmp (p, "btl_stats", 9ul) == 0) {
			btl_stats = 1;
#endif
		} else if (strncmp (p, "btl_print_cs", 12ul) == 0) {
			DBCODE (TRUE, btl_prt_cs = 1);
		} else if (strncmp (p, "affinity=no", 11ul) == 0) {
			btl_no_affinity = 1;
		} else if (strncmp (p, "always_rie", 10ul) == 0) {
			btl_always_rie = 1;
		} else if (strncmp (p, "btl_print_api", 13ul) == 0) {
			DBCODE (TRUE, btl_prt_api = 1);
		} else if (strncmp (p, "btl_print_mem", 13ul) == 0) {
			DBCODE (TRUE, btl_prt_mem = 1);
		} else if (strncmp (p, "btl_print_ptrace", 16ul) == 0) {
			DBCODE (TRUE, btl_prt_pt = 1);
		} else if (strncmp (p, "btl_print_sig", 13ul) == 0) {
			DBCODE (TRUE, btl_prt_sig = 1);
		} else if (strncmp (p, "btl_print_thread", 16ul) == 0) {
			DBCODE (TRUE, btl_prt_thread = 1);
		} else if (strncmp (p, "btl_print_syscall", 17ul) == 0) {
			DBCODE (TRUE, btl_prt_syscall = 1);
		} else if (*p != '\0' && *p != ';') {
			/* 
			 * Add next control directive to the 
			 * pseudo-btrans_ini_switches array 
			 */
			assert (strlen (p) < MAX_SWITCH_LEN - 1);
			*btrans_ini_switches_array[1 + btrans_ini_switches_num] = '-';
			strncpy (btrans_ini_switches_array[1 + btrans_ini_switches_num] + 1, 
				 p, (size_t)MAX_SWITCH_LEN - 1);
			btrans_ini_switches_num++;
			assert (btrans_ini_switches_num < MAX_SWITCHES);
		}

		/* Next directive */
		if (q) {
			p = q;
		} else {
			break;
		}
	}
	SYSCALL (close, (bt_IniFile));

	if (btrans_ini_switches_num > 0) {
		int t;
		for (t = 1; t <= btrans_ini_switches_num; t++) {
			DBPRINT (0, "%d is %s\n", t,
			         btrans_ini_switches_array[t]);
			btrans_ini_switches[t] =
				(char *) &(btrans_ini_switches_array[t]);
		}
	}
	return btrans_ini_switches_num + 1;
}
/*****************************************************************************/
/* parse the "log" line in the Btrans.ini file
 * input - the beginning of the line (points at "log")
 * output - pointer to the beginning of the next line
 */
/*****************************************************************************/
static void open_log_file (char *line, char *argv0, char *logdir)
{
	char *p = line;
	int  tmp_log_file;
	char log_file_name[PATH_MAX] = { 0 };
	char *lfnp = log_file_name;

	/* add the logdir */
	if (*logdir != '\0') {
		strncpy (lfnp, logdir, (size_t)PATH_MAX - 1);
		lfnp += strlen (lfnp);
		if (*lfnp != '/') {
			*lfnp = '/';
			lfnp ++;
		}
	}


	p += 3;
	/* log required - prepare log file name */
	if (*p == '=' && *(p + 1) != '\0') {
		/* Name present as well */
		p++;
		assert (strlen (p) < MAX_SWITCH_LEN - 1);
		if (strncmp (p, "<pid>", 5ul) == 0) {
			sprintf (lfnp, "log.%d", SYSCALL (gettid, ()));
		} else if (strncmp (p, "<name>", 6ul) == 0) {
			char *pa = argv0;
			char *pl = pa;
			while (pa > (char *) 1) {
				pl = pa;
				pa = strchr (pa, '/') + 1;
			}
			sprintf (lfnp, "log.%s", pl);
		} else if (strncmp (p, "<name-pid>", 10ul) == 0) {
			char *pa = argv0;
			char *pl = pa;
			while (pa > (char *) 1) {
				pl = pa;
				pa = strchr (pa, '/') + 1;
			}
			sprintf (lfnp, "log.%s-%d", pl, SYSCALL (gettid, ()));
		} else {
			strncpy (lfnp, p, (size_t)PATH_MAX - 1);
		}
	} else {	
		/* else the name remains the default */

		strncat (lfnp, LOG_FILE_NAME, 
			 MIN(sizeof(log_file_name) - strlen(lfnp) - 1, 
			     strlen(LOG_FILE_NAME)));
	}

	DBPRINT(0, "log_file_name='%s'\n", log_file_name);

	/* open the file */
	tmp_log_file = SYSCALL (open, (log_file_name, O_CREAT|O_TRUNC|O_WRONLY,
				0644));
	if (IS_SYS_ERROR (tmp_log_file)) {
		DBPRINT (TRUE, "Can't create LOG file %s:%d\n", 
			 log_file_name, tmp_log_file);
	} else {
		g_log_file = tmp_log_file;
	}
}
#endif

/*****************************************************************************/
/* load and init btgeneric.
 * includes reading the ini file and initialization of the debug globals.
 *
 * if is_ver != 0 then only extract the version string from btgeneric and
 * return
 */
/*****************************************************************************/
static long
load_and_init_btgeneric (char **argv RELEASE_UNUSED, char **envp, long is_ver,
                         char btg_ver[PATH_MAX])
{
	char *btrans_ini_switches[MAX_SWITCHES] = { 0 };
	API_TABLE_TYPE *btrans_api_table_start;
	BT_STATUS_CODE bt_retval;
	long btrans_ini_switches_num = 0;
	long retval;
	unsigned i;
	unsigned long rsrc[2] = { 0ul, 0ul };	/* size and location of rsrc section */
	API_VERSION cur_api_version;
	__s32 index_of_accepted_api_version;
	void *p_btg_get_api_info;
	API_TABLE_TYPE api_table;

	/* load BTGeneric and get its API table start address. */
	retval = set_btgeneric_path (BTGeneric_filename, sizeof (BTGeneric_filename), envp);
	if (retval != 0) {
		DBPRINT (TRUE, "set_btgeneric_path failed\n");
		return -1;
	}
	retval = load_pe_binary (BTGeneric_filename,
				 &btl_btg_load_address,
				 &p_btg_get_api_info, &btl_btg_size, rsrc);

	if (retval != 0) {
		DBPRINT (TRUE, "load_pe_binary returned (%lx).\n", retval);
		return -1;
	}

	/* look for the version string in btgeneric */
	get_version_str ((void *) rsrc[0], rsrc[1], btg_ver,
			 (unsigned long) PATH_MAX);
	if (is_ver) {
		return retval;
	}

	cur_api_version.major_version = MAJOR_VERSION_NUMBER;
	cur_api_version.minor_version = MINOR_VERSION_NUMBER;

	bt_retval = (*(PTR_GET_API_INFO_FUNC) p_btg_get_api_info) (
		(API_VERSION *) (&cur_api_version),
	   	1,
		(__s32 *)(&index_of_accepted_api_version),
	   	(API_TABLE_TYPE **)(&btrans_api_table_start));

	if (bt_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE,
		         "\nCpuProcessInit: BTGenericGetAPIInfo(...) returned "
		         "bt_retval != BT_STATUS_SUCCESS\n");
		return -1;
	}

	/* perform the API table initialization */
	for (i = 0; i < NO_OF_BTGENERIC_APIS; i++) {
		btl_place_holder_table[i] =
			GET_PLABEL_PTR_FROM_API_TABLE ((*btrans_api_table_start), i);
	}

	/* initialize BTGeneric placeholder table and get glst size from BTGeneric */
	api_table = &btl_api_table_elements[0];
	bt_retval = BTGENERIC_START (&cur_api_version,
				     &api_table,
				     btl_btg_load_address,
				     (void *) (btl_btg_size + btl_btg_load_address),
				     -sizeof (btlib_thread_desc),
				     (unsigned *) &btl_glst_size,
				     (unsigned *) &btl_bc32);
	if (bt_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE, "BTGENERIC_START failed\n");
		return -1;
	}

	DBPRINT (0,
	         "load_and_init_btgeneric: btl_glst_size=%u<%x> btl_bc32=%u<%x>\n",
	         btl_glst_size, btl_glst_size, btl_bc32, btl_bc32);

	/*
	 * If Btrans.ini configuration file is present, parse it and set
	 * ia32exec.bin debug parameters (BTGeneric_Debug_Settings)
	 */
	memset ((void *) btrans_ini_switches, 0,
		MAX_SWITCHES * sizeof (char *));
#if ! RELEASE
	btrans_ini_switches_num =
		read_config_file (argv[1], btrans_ini_switches);
#endif
	bt_retval =
		BTGENERIC_DEBUG_SETTINGS (btrans_ini_switches_num,
					  btrans_ini_switches);
	if (bt_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE, "BTGENERIC_DEBUG_SETTINGS failed:%d\n", bt_retval);
		return -1;
	}

	return 0;
}

/*****************************************************************************/
/*
 * Report to BTGeneric the location and dimensions of the loaded 
 * modules
 */
/*****************************************************************************/
static void
report_initial_code_areas (btrans_binary_map_t * app_info)
{
	/* the dynamic loader */
	if (app_info->dl_base_addr != 0) {
		DBPRINT (0, "dl_base_addr=%x dl_size=%x\n",
			 app_info->dl_base_addr, app_info->dl_size);
		BTGENERIC_FLUSH_IA32_INSTRUCTION_CACHE (
			U2P (app_info->dl_base_addr),
			app_info->dl_size,
			BT_FLUSH_ALLOC);
	}
	/* the binary */
	BTGENERIC_FLUSH_IA32_INSTRUCTION_CACHE (
		U2P (app_info->start_code),
		app_info->end_code - app_info->start_code + 1,
		BT_FLUSH_ALLOC);
}

/*****************************************************************************/
/*****************************************************************************/
static char *
find_unicode_str (char *base_addr, unsigned long limit, char *str)
{
	long status = 0;
	char *base_p = base_addr;
	char *start_unicode_str = NULL;
	char *str_p = str;

	while (1) {
		if ((unsigned long) (base_p - base_addr) > limit) {
			/* limit exceeded */
			return NULL;
		}

		if (status == 0) {
			if ((*base_p == *str_p) && (*(base_p + 1) == 0)) {
				status = 1;
				start_unicode_str = base_p;
				str_p++;
				base_p += 2;
			} else {
				base_p++;
			}
		} else if (status == 1) {
			if ((*base_p != *str_p) || (*(base_p + 1) != 0)) {
				status = 0;
				start_unicode_str = NULL;
				str_p = str;
				base_p++;
			} else if (*str_p == '\0') {
				/* bingo */
				break;
			} else {
				str_p++;
				base_p += 2;
			}
		}
	}
	return start_unicode_str;
}

/*****************************************************************************/
/*****************************************************************************/
static void
extract_unicode_str (char *ver_str_p, char *ver_str,
                     unsigned long ver_str_size RELEASE_UNUSED)
{
	unsigned long size = 0;
	while (*ver_str_p != 0) {
		assert (*(ver_str_p + 1) == 0);
		size++;
		assert (size < ver_str_size);
		*ver_str = *ver_str_p;
		ver_str_p += 2;
		ver_str += 1;
	}
	*(ver_str + size) = '\0';
}

/*****************************************************************************/
/*****************************************************************************/
static void
get_version_str (void *rsrc, unsigned long size, char *ver_str,
                 unsigned long ver_str_size)
{
	char str[] = "FileVersion";
	char *ver_str_p = NULL;

	ver_str_p = find_unicode_str (rsrc, size, str);
	assert (ver_str != NULL);

	ver_str_p += 2 * strlen (str);
	while (*ver_str_p == '\0') {
		ver_str_p++;
	}

	extract_unicode_str (ver_str_p, ver_str, ver_str_size);
}

/*****************************************************************************/
/* remove first two arguments from argv. */
/*****************************************************************************/
static int
shift_argv (int argc, char *argv[])
{
	char *end_of_args = argv[argc - 1] + strlen (argv[argc - 1]) + 1;
	char *new_end_of_args = (argc > 2 ? end_of_args - (argv[2] - argv[0])
				 : argv[0]);

	/* shift the args 2 positions to the left, so argv[2] becomes argv[0] */
	if (argc > 2)
		memmove (argv[0], argv[2], (size_t) (end_of_args - argv[2]));
	/* reset the rest of the strings */
	memset (new_end_of_args, 0, (size_t) (end_of_args - new_end_of_args));
	return 0;
}

/*****************************************************************************/
/* btlib need 2 more open files than original binary 
 * ( one fd for the binary, one for the dynamic loader )
 * input:  orig_rlim - will hold the current limit on open fileno when the 
 *                     function returns
 * output: 0 if limit did not change, 1 if limit changed, negative value 
 *         on error */
/*****************************************************************************/
static int
extend_rlimit_nofile (struct rlimit *orig_rlim)
{
	int retval = 0;
	struct rlimit rlim;

	/* get process limit on number of open files */
	retval = SYSCALL (getrlimit, (RLIMIT_NOFILE, orig_rlim));
	if (IS_SYS_ERROR (retval)) {
		DBPRINT (TRUE, "Unable to get RLIMIT_NOFILE:%d\n", retval);
		goto out;
	}

	DBPRINT (0, "Old RLIMIT_NOFILE: rlim_max:%d, rlim_cur:%d\n",
		 (int) orig_rlim->rlim_max, (int) orig_rlim->rlim_cur);

	if (orig_rlim->rlim_cur < NR_OPEN) {
		rlim.rlim_max =
			(orig_rlim->rlim_max + RLIM_NOFILE_EXT) > NR_OPEN ? 
			NR_OPEN : (orig_rlim->rlim_max + RLIM_NOFILE_EXT);
		rlim.rlim_cur =
			(orig_rlim->rlim_cur + RLIM_NOFILE_EXT) > NR_OPEN ? 
			NR_OPEN : (orig_rlim->rlim_cur + RLIM_NOFILE_EXT);
		if (rlim.rlim_cur > rlim.rlim_max) {
			rlim.rlim_cur = rlim.rlim_max;
		}
		retval = SYSCALL (setrlimit, (RLIMIT_NOFILE, &rlim));
		if (IS_SYS_ERROR (retval)) {
			DBPRINT (TRUE,
				 "Unable to set RLIMIT_NOFILE with rlim_max:"
				 "%d, rlim_cur:%d, retval:%d\n",
				 (int) rlim.rlim_max, (int) rlim.rlim_cur,
				 retval);
			goto out;
		}
		retval = 1;
	}
      out:
	return retval;
}

/*****************************************************************************/
/* print the versions of ia32el components */
/*****************************************************************************/
static void
print_ver (char *btg_ver)
{
	char verstr[PATH_MAX];

	if (btg_ver) {
		fprintf (stdout, "IA32X Generic   :\tversion=%s\tpath=%s\n",
			 btg_ver, BTGeneric_filename);
	} else {
		fprintf (stdout, "IA32X Generic   :\tnot found\n");
	}
	fprintf (stdout, "IA32X OS-Wrapper:\tversion=%d.%d.%s",
		 MAJOR_VERSION_NUMBER, MINOR_VERSION_NUMBER, BUILD_NO);
	iti_get_ia32toia64_version (verstr);
	fprintf (stdout, ".%s", verstr);
	lcs_get_ia32x_utils_version (verstr);
	fprintf (stdout, ".%s", verstr);
	fprintf (stdout, ".%s\n", MODE);
}
