/*  
 * btlib_modify_ldt.h
 *  
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef  _BTLIB_SEGMENT_H
#define  _BTLIB_SEGMENT_H

#define LDT_READ         0x0
#define LDT_WRITE        0x1
#define LDT_READ_DEFAULT 0x2
#define LDT_WRITE_NEW    0x11

#define LDT_ENTRIES      8192
#define LDT_ENTRY_SIZE   8

/* segment register */
#define __USER_DS   0x2b
#define __USER_CS   0x23

/* TLS GDT layout in IA-32 kernel ( needed for sys_set_thread_area )*/
#define GDT_USER_ENT_NUM   3
#define GDT_USER_ENT_FIRST       6
#define GDT_USER_ENT_LAST       (GDT_USER_ENT_FIRST + GDT_USER_ENT_NUM - 1)

typedef struct _desc_struct {
	unsigned short limit_low;
	unsigned short base_low;
	unsigned base_mid:8;
	unsigned type:5;
	unsigned dpl:2;
	unsigned present:1;
	unsigned limit_hi:4;
	unsigned avl:1;
	unsigned reserved_0:1;
	unsigned default_big:1;
	unsigned granularity:1;
	unsigned base_hi:8;
} desc_struct;
typedef struct _desc_struct ldt_entry;

typedef struct _user_desc {
	unsigned int entry_number;
	unsigned int base_addr;
	unsigned int limit;
	unsigned int seg_32bit:1;
	unsigned int contents:2;
	unsigned int read_exec_only:1;
	unsigned int limit_in_pages:1;
	unsigned int seg_not_present:1;
	unsigned int useable:1;
} user_desc;

typedef struct _user_desc modify_ldt_s;

#define DT_INDEX(selector)      (((selector) & 0xFFFF) >> 3)
#define IS_LDT_ENTRY(selector)  ((selector & 0x4) != 0)

#define IS_GDT_TLS_ENTRY(selector)                              \
         ((((selector) & 0x4) == 0) &&                          \
         (DT_INDEX(selector) >= GDT_USER_ENT_FIRST) &&           \
         (DT_INDEX(selector) <= GDT_USER_ENT_LAST))

#define SEG_BASEADDR(disc)      (((disc).base_hi << 24)  |      \
                                 ((disc).base_mid << 16) |      \
                                 ((disc).base_low))
#define SEG_LIMIT(disc)         (((disc).limit_hi << 16) | ((disc).limit_low))

#define GDT_BASE                ((__u32)0)
#define GDT_LIMIT               ((__u16)(-1))
#define IDT_BASE                ((__u32)0)
#define IDT_LIMIT               ((__u16)(-1))
#define LDT_SELECTOR            ((__u16)0)

#define SET_DESC_TABLE_REGISTERS(__bc)                                  \
        do {                                                            \
                CONTEXT_EXTFIELD((__bc), gdt_base) = GDT_BASE;          \
                CONTEXT_EXTFIELD((__bc), gdt_limit) = GDT_LIMIT;        \
                CONTEXT_EXTFIELD((__bc), idt_base) = IDT_BASE;          \
                CONTEXT_EXTFIELD((__bc), idt_limit) = IDT_LIMIT;        \
                CONTEXT_EXTFIELD((__bc), ldt_selector) = LDT_SELECTOR;  \
        } while (0)

#define CHECK_DESC_TABLE_REGISTERS(__bc)                                        \
        assert ((CONTEXT_EXTFIELD((__bc), gdt_base) == GDT_BASE) &&             \
                (CONTEXT_EXTFIELD((__bc), gdt_limit) == GDT_LIMIT) &&           \
                (CONTEXT_EXTFIELD((__bc), idt_base) == IDT_BASE) &&             \
                (CONTEXT_EXTFIELD((__bc), idt_limit) == IDT_LIMIT) &&           \
                (CONTEXT_EXTFIELD((__bc), ldt_selector) == LDT_SELECTOR))

long btlib_modify_ldt (BTGENERIC_IA32_CONTEXT * bc32);
long btlib_set_thread_area (BTGENERIC_IA32_CONTEXT * bc32);
long btlib_get_thread_area (BTGENERIC_IA32_CONTEXT * bc32);

long btlib_check_segment (BTGENERIC_IA32_CONTEXT * bc32);
void set_desc (desc_struct * desc, user_desc * u);
void get_desc (user_desc * u, desc_struct * desc);

inline long is_udesc_empty (const user_desc * u_info);

#endif
