/*  
 * btlib_sig_utils.h
 *  
 *  
 *
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef BTLIB_SIG_UTILS_H
#define BTLIB_SIG_UTILS_H


#include <ia32x_utils.h>

extern int m_permanent_handlers[];
extern int m_half_permanent_handlers[];

/* function prototypes */

inline void sigemptyset32 (sigset32_t *set);
inline void sigaddset32 (sigset32_t *set, int sig);
inline long sigismember32 (const sigset32_t *set, int sig);
inline int isemptyset32 (sigset32_t *set, unsigned size);
inline void sigdelset32 (sigset32_t *set, int sig);

inline void sig_and_mask (sigset32_t *dest, const sigset32_t *src, int size);
inline void sig_andnot_mask (sigset32_t *dest, const sigset32_t *src, int size);
inline void sig_or_mask (sigset32_t *dest, const sigset32_t *src, int size);
inline long is_async_signal (siginfo_t *siginfo);
inline void unreg_btlib_handler (int signo);
inline unsigned signal2except (siginfo_t *siginfo);
inline void btgctx2sigctx_32 (const BTGENERIC_IA32_CONTEXT *bc32,
                                     sigcontext32_t *sc,
                                     struct _fpstate_ia32 *fpstate);
inline void sigctx2btgctx_32 (sigcontext32_t *sc, BTGENERIC_IA32_CONTEXT *bc32);
inline void sigctx2btgctx_64 (struct sigcontext *linux_context,
                                     BTGENERIC_IA64_CONTEXT *ia64_sigcontext);
inline void sigaddset32 (sigset32_t *set, int _sig);
inline void sigdelset32 (sigset32_t *set, int _sig);
inline long sigismember32 (const sigset32_t *set, int _sig);
inline void sigset32_2_sigset64 (const sigset32_t *set32, k_sigset_t *set64);
inline void sigset64_2_sigset32 (const k_sigset_t *set64, sigset32_t *set32);


inline void st_conversion_fsave_to_fxsave (unsigned *fxsave_st, void *fsave_st);
inline void st_conversion_fxsave_to_fsave (void *fsave_st, unsigned *fxsave_st);
inline unsigned short twd_convertion_word_to_byte (__u32 twd);
inline __u32 twd_convertion_byte_to_word (struct i387_fxsave_struct *fxsave);


#if DEBUG
void print_stacked_registers (struct sigcontext *sc);
void print_sigaction32 (char *header, struct sigaction32 *sa);
unsigned long load_stacked_reg (struct sigcontext *sc, long num, long *nat);
void sprint_flags (unsigned flags, char *flags_str, long str_len);
#endif

/* Backing store related macros */
#define SOF_MASK 0x7FUL


#define FPREG_NUMBER 8
#define BTGENERIC_FP_BYTES_PER_ENTRY 10
#define FXSAVE_FP_BYTES_PER_ENTRY    16

#define SHIFT_LEFT(x,len) x<<len
#define SHIFT_RIGHT(x,len) x>>len

#define FPREG_VALID   0
#define FPREG_ZERO    1
#define FPREG_SPECIAL 2
#define FPREG_EMPTY   3

#endif

