/*  
 * btlib_signal.h
 *  
 * 
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef BTLIB_SIGNAL_H
#define BTLIB_SIGNAL_H

#include <ia32x_utils.h>

struct critical_section_t;

#define ia32_sigmask(sig)  (1u << ((sig)-1))
#define sigfillset(set)    memset((set), 0xff, sizeof(k_sigset_t))
#define sigemptyset(set)   memset((set), 0, sizeof(k_sigset_t))

typedef struct handler_info {
	struct sigaction32 sa32;
	siginfo_t *siginfo;
	sigcontext32_t sc32;
	struct _fpstate_ia32 fpstate;
	sigset32_t mask;
} handler_info_t;

struct signal_handler_desc {
	/* 
	 * need to align the 'handler' field on 8 bytes boundry because
	 * this field is read by btlib of the debugger when a signal
	 * stops the debugee ( see btlib_ptemu.c:get_debugee_handler() ).
	 * the alignment ensures that the field is read atomically, thus
	 * avoiding the need for sync
	 */
	sigaction32_t sa __attribute__ ((aligned (8)));
};
typedef struct signal_handler_table {
	cs_handle cs;
	int ref_count;
	struct signal_handler_desc handlers[_NSIG + 1];
} sighand_table;

typedef struct frame_info {
	sigcontext32_t *ctx;
	sigset32_t mask;
} frame_info_t;

#define CLONE_PID       0x00001000
#define IS_NPTL_FLAGS(__flags)                  \
	(((__flags) & CLONE_THREAD) ||          \
	 ((__flags) & CLONE_SETTLS) ||          \
	 ((__flags) & CLONE_PARENT_SETTID)  ||  \
	 ((__flags) & CLONE_CHILD_CLEARTID) ||  \
	 ((__flags) & CLONE_DETACHED)       ||  \
	 ((__flags) & CLONE_CHILD_SETTID))

/* signal default action is 'ignore' */
#define SIGNAL_DEFAULT_IGNORE(signo) \
	(((signo) == SIGCONT) ||     \
	 ((signo) == SIGCHLD) ||     \
	 ((signo) == SIGWINCH)||     \
	 ((signo) == SIGURG))

/* signal default action is 'stop' */
#define SIGNAL_DEFAULT_STOP(signo) \
	(((signo) == SIGSTOP) ||   \
	 ((signo) == SIGTSTP) ||   \
	 ((signo) == SIGTTOU) ||   \
	 ((signo) == SIGTTIN))

extern long emulate_sigaction (int signo, sigaction32_t *sa, sigaction32_t *osa, int size);
extern void write_sighand_frame (handler_info_t *hi, void *frame, int is_info);
extern void read_sighand_frame (unsigned esp, frame_info_t *frame_info, int is_info);
extern int is_using_alt_stack (unsigned sp);

#endif				/* BTLIB_SIGNAL_H */

