/* 
 * btlib_stats.c
 *
 * collect and print statistics
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#if STAT

#include "btlib.h"
#include "btlib_stats.h"

char btl_stats = 0;

static struct desc_val {
	char desc[64];
	unsigned long val;
} m_counters[BTL_MAX_STAT_IND] = {
	{"max signals in pending que", 0},
	{"signal passed to other threads", 0},
	{"stacks that were deallocated", 0},
	{"stacks that were reused", 0},
	{"max size of free stack list", 0},
	{"signals received", 0},
	{"sync signals received", 0},
	{"async signals received", 0},
	{"syscalls called", 0},
	{"long syscalls called", 0},
	{"btrans_jump called", 0},
	{"entries in proc-self array", 0}
};

void
btlib_update_stat (int ind, int type, unsigned long new_val)
{
	assert (ind < BTL_MAX_STAT_IND);
	switch (type) {
	case COUNT:
		m_counters[ind].val++;
		break;
	case MAX_VAL:
		m_counters[ind].val = MAX (new_val, m_counters[ind].val);
		break;
	default:
		assert (!"unknown type");
	}
}

void
btlib_print_stats ()
{
	long j;

	fprintf (g_log_file, "=============================================="
		 " BTLIB STATS "
		 "==============================================\n");
	for (j = 0; j < BTL_MAX_STAT_IND; j++) {
		fprintf (g_log_file, "%s = %ld\n",
			 m_counters[j].desc, m_counters[j].val);
	}
	fprintf (g_log_file, "=============================================="
		 "============="
		 "==============================================\n");
}

#endif			// STAT
