/*  
 * btlib_syscall.c
 *  
 *  
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include "btlib.h"
#include <ia32x_utils.h>

#include <ia32toia64.h>

#include "btlib_segment.h"
#include "btlib_syscall.h"
#include "btlib_sync.h"
#include "btlib_stats.h"
#include "btlib_ptemu.h"
#include "btlib_sig_utils.h"


/* the IA-32 kernel space address */
long btl_kernel_page = 0;

/* the address of restore code for sigreturn */
long btl_restorer;
/* the address of rt_restore code for sigreturn */
long btl_rt_restorer;

#if DEBUG
const char *syscall_names[] = {
/*0*/ "nyi",
/*1*/ "exit",
/*2*/ "fork",
/*3*/ "read",
/*4*/ "write",
/*5*/ "open",
/*6*/ "close",
/*7*/ "waitpid",
/*8*/ "creat",
/*9*/ "link",
/*10*/ "unlink",
/*11*/ "execve",
/*12*/ "chdir",
/*13*/ "time",
/*14*/ "mknod",
/*15*/ "chmod",
/*16*/ "lchown",
/*17*/ "break",
/*18*/ "oldstat",
/*19*/ "lseek",
/*20*/ "getpid",
/*21*/ "mount",
/*22*/ "umount",
/*23*/ "setuid",
/*24*/ "getuid",
/*25*/ "stime",
/*26*/ "ptrace",
/*27*/ "alarm",
/*28*/ "oldfstat",
/*29*/ "pause",
/*30*/ "utime",
/*31*/ "stty",
/*32*/ "gtty",
/*33*/ "access",
/*34*/ "nice",
/*35*/ "ftime",
/*36*/ "sync",
/*37*/ "kill",
/*38*/ "rename",
/*39*/ "mkdir",
/*40*/ "rmdir",
/*41*/ "dup",
/*42*/ "pipe",
/*43*/ "times",
/*44*/ "prof",
/*45*/ "brk",
/*46*/ "setgid",
/*47*/ "getgid",
/*48*/ "signal",
/*49*/ "geteuid",
/*50*/ "getegid",
/*51*/ "acct",
/*52*/ "umount2",
/*53*/ "lock",
/*54*/ "ioctl",
/*55*/ "fcntl",
/*56*/ "mpx",
/*57*/ "setpgid",
/*58*/ "ulimit",
/*59*/ "oldolduname",
/*60*/ "umask",
/*61*/ "chroot",
/*62*/ "ustat",
/*63*/ "dup2",
/*64*/ "getppid",
/*65*/ "getpgrp",
/*66*/ "setsid",
/*67*/ "sigaction",
/*68*/ "sgetmask",
/*69*/ "ssetmask",
/*70*/ "setreuid",
/*71*/ "setregid",
/*72*/ "sigsuspend",
/*73*/ "sigpending",
/*74*/ "sethostname",
/*75*/ "setrlimit",
/*76*/ "getrlimit",
/*77*/ "getrusage",
/*78*/ "gettimeofday",
/*79*/ "settimeofday",
/*80*/ "getgroups",
/*81*/ "setgroups",
/*82*/ "select",
/*83*/ "symlink",
/*84*/ "oldlstat",
/*85*/ "readlink",
/*86*/ "uselib",
/*87*/ "swapon",
/*88*/ "reboot",
/*89*/ "readdir",
/*90*/ "mmap",
/*91*/ "munmap",
/*92*/ "truncate",
/*93*/ "ftruncate",
/*94*/ "fchmod",
/*95*/ "fchown",
/*96*/ "getpriority",
/*97*/ "setpriority",
/*98*/ "profil",
/*99*/ "statfs",
/*100*/ "fstatfs",
/*101*/ "ioperm",
/*102*/ "socketcall",
/*103*/ "syslog",
/*104*/ "setitimer",
/*105*/ "getitimer",
/*106*/ "stat",
/*107*/ "lstat",
/*108*/ "fstat",
/*109*/ "olduname",
/*110*/ "iopl",
/*111*/ "vhangup",
/*112*/ "idle",
/*113*/ "vm86old",
/*114*/ "wait4",
/*115*/ "swapoff",
/*116*/ "sysinfo",
/*117*/ "ipc",
/*118*/ "fsync",
/*119*/ "sigreturn",
/*120*/ "clone",
/*121*/ "setdomainname",
/*122*/ "uname",
/*123*/ "modify_ldt",
/*124*/ "adjtimex",
/*125*/ "mprotect",
/*126*/ "sigprocmask",
/*127*/ "create_module",
/*128*/ "init_module",
/*129*/ "delete_module",
/*130*/ "get_kernel_syms",
/*131*/ "quotactl",
/*132*/ "getpgid",
/*133*/ "fchdir",
/*134*/ "bdflush",
/*135*/ "sysfs",
/*136*/ "personality",
/*137*/ "afs_syscall",
/*138*/ "setfsuid",
/*139*/ "setfsgid",
/*140*/ "_llseek",
/*141*/ "getdents",
/*142*/ "_newselect",
/*143*/ "flock",
/*144*/ "msync",
/*145*/ "readv",
/*146*/ "writev",
/*147*/ "getsid",
/*148*/ "fdatasync",
/*149*/ "_sysctl",
/*150*/ "mlock",
/*151*/ "munlock",
/*152*/ "mlockall",
/*153*/ "munlockall",
/*154*/ "sched_setparam",
/*155*/ "sched_getparam",
/*156*/ "sched_setscheduler",
/*157*/ "sched_getscheduler",
/*158*/ "sched_yield",
/*159*/ "sched_get_priority_max",
/*160*/ "sched_get_priority_min",
/*161*/ "sched_rr_get_interval",
/*162*/ "nanosleep",
/*163*/ "mremap",
/*164*/ "setresuid",
/*165*/ "getresuid",
/*166*/ "vm86",
/*167*/ "query_module",
/*168*/ "poll",
/*169*/ "nfsservctl",
/*170*/ "setresgid",
/*171*/ "getresgid",
/*172*/ "prctl",
/*173*/ "rt_sigreturn",
/*174*/ "rt_sigaction",
/*175*/ "rt_sigprocmask",
/*176*/ "rt_sigpending",
/*177*/ "rt_sigtimedwait",
/*178*/ "rt_sigqueueinfo",
/*179*/ "rt_sigsuspend",
/*180*/ "pread",
/*181*/ "pwrite",
/*182*/ "chown",
/*183*/ "getcwd",
/*184*/ "capget",
/*185*/ "capset",
/*186*/ "sigaltstack",
/*187*/ "sendfile",
/*188*/ "getpmsg",
/*189*/ "putpmsg",
/*190*/ "vfork",
/*191*/ "ugetrlimit",
/*192*/ "mmap2",
/*193*/ "truncate64",
/*194*/ "ftruncate64",
/*195*/ "stat64",
/*196*/ "lstat64",
/*197*/ "fstat64",
/*198*/ "lchown32",
/*199*/ "getuid32",
/*200*/ "getgid32",
/*201*/ "geteuid32",
/*202*/ "getegid32",
/*203*/ "setreuid32",
/*204*/ "setregid32",
/*205*/ "getgroups32",
/*206*/ "setgroups32",
/*207*/ "fchown32",
/*208*/ "setresuid32",
/*209*/ "getresuid32",
/*210*/ "setresgid32",
/*211*/ "getresgid32",
/*212*/ "chown32",
/*213*/ "setuid32",
/*214*/ "setgid32",
/*215*/ "setfsuid32",
/*216*/ "setfsgid32",
/*217*/ "pivot_root",
/*218*/ "mincore",
/*219*/ "madvise",
/*220*/ "getdents64",
/*221*/ "fcntl64",
/*222*/ "NYI",
/*223*/ "security",
/*224*/ "sys_gettid",
/*225*/ "readahead",
/*226*/ "setxattr",
/*227*/ "lsetxattr",
/*228*/ "fsetxattr",
/*229*/ "getxattr",
/*230*/ "lgetxattr",
/*231*/ "fgetxattr",
/*232*/ "listxattr",
/*233*/ "llistxattr",
/*234*/ "flistxattr",
/*235*/ "removexattr",
/*236*/ "lremovexattr",
/*237*/ "fremovexattr",
/*238*/ "tkill",
/*239*/ "sendfile64",
/*240*/ "futex",
/*241*/ "sched_setaffinity",
/*242*/ "sched_getaffinity",
/*243*/ "set_thread_area",
/*244*/ "get_thread_area",
/*245*/ "io_setup",
/*246*/ "io_destroy",
/*247*/ "io_getevents",
/*248*/ "io_submit",
/*249*/ "io_cancel",
/*250*/ "fadvise64",
/*251*/ "free_hugepages",
/*252*/ "exit_group",
/*253*/ "lookup_dcookie",
/*254*/ "epoll_create",
/*255*/ "epoll_ctl",
/*256*/ "epoll_wait",
/*257*/ "remap_file_pages",
/*258*/ "set_tid_address",
/*259*/ "timer_create",
/*260*/ "timer_settime",
/*261*/ "timer_gettime",
/*262*/ "timer_getoverrun",
/*263*/ "timer_delete",
/*264*/ "clock_settime",
/*265*/ "clock_gettime",
/*266*/ "clock_getres",
/*267*/ "clock_nanosleep",
/*268*/ "NYI",
/*269*/ "fstatfs64",
/*270*/ "tgkill",
/*271*/ "utimes",
/*272*/ "fadvise64_64"
};

#define SYSCALL_NAMES_MAX_ID 272
#endif	// DEBUG

struct _proc_self_tag;
typedef struct _proc_self_tag proc_self_t;
typedef int (* pre_syscall_fnp_t) (proc_self_t * fd_info, BTGENERIC_IA32_CONTEXT * bc32);
typedef int (* post_syscall_fnp_t) (proc_self_t * fd_info, BTGENERIC_IA32_CONTEXT * bc32, long retval);

/* hold post processing function for read operations from /proc/self files */
struct _proc_self_tag {
	int fd;
	int tid;
	unsigned flags;
	pre_syscall_fnp_t pre_read_fnp;
	post_syscall_fnp_t post_read_fnp;
};
#define PROC_SELF_NOT_CUR_SET 1
static proc_self_t * m_proc_self_fds = NULL;
static unsigned long m_proc_self_fds_size = 0;

/* function prototypes */
static int pre_read_proc_stat (proc_self_t * fd_info, BTGENERIC_IA32_CONTEXT * bc32);
static int post_read_proc_stat (proc_self_t * fd_info, BTGENERIC_IA32_CONTEXT * bc32, long retval);
static long is_proc_self_file (char * filename, char * procself_filename);
static void insert_proc_self_fd (int fd, int tid, pre_syscall_fnp_t pre_read_pfn, post_syscall_fnp_t post_read_pfn);
static void remove_proc_self_fd (int fd);
static long ia32_wait (BTGENERIC_IA32_CONTEXT *bc32);
static long ia32_system_call (BTGENERIC_IA32_CONTEXT *bc32, void *ia64p);
static int  need_deliver_pending (int syscall_id); 
static long btlib_readlink (BTGENERIC_IA32_CONTEXT * bc32);
static long btlib_personality (BTGENERIC_IA32_CONTEXT * bc32);
#if DEBUG
static void UNUSED print_syscall_before (int syscall_id);
static void UNUSED print_syscall_after (int syscall_id, long retval);
static void UNUSED print_socketcall_info (BTGENERIC_IA32_CONTEXT *bc32);
static void UNUSED print_getdents (BTGENERIC_IA32_CONTEXT *bc32);
static void UNUSED print_ipc_info (BTGENERIC_IA32_CONTEXT *bc32);
#endif

/*****************************************************************************/
/*****************************************************************************/
static int 
need_deliver_pending (int syscall_id)
{
	/* for sigaction/rt_sigaction/signal cases, we'd better check the 
	   sighandler also, but not checking doesn't do harm */
	if ((syscall_id == IA32_SYSCALL_SIGPROCMASK) || 
	    (syscall_id == IA32_SYSCALL_RT_SIGPROCMASK) || 
	    (syscall_id == IA32_SYSCALL_SIGACTION) || 
	    (syscall_id == IA32_SYSCALL_RT_SIGACTION) || 
	    (syscall_id == IA32_SYSCALL_SIGNAL)) {
		return 1;
	} else {
		return 0;
	}
}

/*****************************************************************************/
/*****************************************************************************/
static inline int
pre_read (BTGENERIC_IA32_CONTEXT *bc32)
{
	long ret = 0;
	int fd = (int)cur_thread_data->bc32->ebx;
	unsigned j;
	for (j = 0; j < m_proc_self_fds_size; j ++) {
		if (m_proc_self_fds[j].fd == fd) {
			break;			
		}
	}
	if (j < m_proc_self_fds_size) {
		if (m_proc_self_fds[j].pre_read_fnp != NULL) {
			ret = m_proc_self_fds[j].pre_read_fnp (&m_proc_self_fds[j], bc32);
		}
	}
	return ret;
}
/*****************************************************************************/
/*****************************************************************************/
static inline int
post_read (BTGENERIC_IA32_CONTEXT *bc32, long retval)
{
	long ret = retval;
	int fd = (int)cur_thread_data->bc32->ebx;
	unsigned j;
	for (j = 0; j < m_proc_self_fds_size; j ++) {
		if (m_proc_self_fds[j].fd == fd) {
			break;			
		}
	}
	if (j < m_proc_self_fds_size) {
		if (m_proc_self_fds[j].post_read_fnp != NULL) {
			ret = m_proc_self_fds[j].post_read_fnp (&m_proc_self_fds[j], bc32, retval);
		}
	}
	return ret;
}
/*****************************************************************************/
/* call the wrapper syscall for IA-32 syscalls
 * input  : bc32 - the IA-32 context of the thread
 *          ia64p - btlib may pass pointers above 4G for some of the syscalls.
 * output : the return value of the syscall
 * remarks: in case of long syscalls, the syscall_id flag
 *          of the thread is set before calling the wrapper
 *          and cleared when returning from it.
 *
 *          ALL LONG SYSCALLS MUST GO THROUGH THIS FUNCTION
 */
/*****************************************************************************/
static long
ia32_system_call (BTGENERIC_IA32_CONTEXT *bc32, void *ia64p)
{
	unsigned long param2;
	int syscall_id;
	long retval;
	sigset32_t    mask, oldmask;

	assert (bc32 != NULL);

	syscall_id = cur_thread_data->syscall_id;

	/* wait syscalls may want to pass a status parameter which is
	   not IA-32 address */
	param2 = (((syscall_id == IA32_SYSCALL_WAITPID) ||
		   (syscall_id == IA32_SYSCALL_WAIT4)) && (ia64p != NULL)) ?
		(unsigned long) ia64p : (unsigned long) IA32_SYSCALL_PARAM2 (bc32);

	/* for sigsuspend, signal handler uses signal mask specified by the parameter */
	if (syscall_id == IA32_SYSCALL_SIGSUSPEND) {
		if (safe_memcpy (&mask.sig[0], U2P(IA32_SYSCALL_PARAM3 (bc32)), 
				 sizeof(mask.sig[0])) < 0) {
			return -EFAULT;
		}

		mask.sig[1] = 0;
	} else if (syscall_id == IA32_SYSCALL_RT_SIGSUSPEND) {
		if (IA32_SYSCALL_PARAM2 (bc32) != sizeof (sigset32_t))  {
			return -EINVAL;
		}

		if (safe_memcpy (&mask, U2P(IA32_SYSCALL_PARAM1 (bc32)),
				 sizeof(mask)) < 0) {
			return -EFAULT;
		}
	}

	/* for sigsuspend, check signal queue first with new signal mask */
	if ((syscall_id == IA32_SYSCALL_SIGSUSPEND) || 
	    (syscall_id == IA32_SYSCALL_RT_SIGSUSPEND)) {

		sigdelset32 (&mask, SIGKILL);
		sigdelset32 (&mask, SIGSTOP);

		oldmask = cur_thread_data->sigmask;
		cur_thread_data->sigmask = mask; 

		if (!btlib_que_is_empty (cur_thread_data->pending_que) || 
		    !btlib_que_is_empty (btl_shared_que)) {
			long  *lsf = NULL;
			
			iti_long_syscall_interface (&lsf, NULL);
			*lsf = NOT_IN_SYSCALL;
			cur_thread_data->bc32->eax = (unsigned) -EINTR;
			restart_btg (NULL, &oldmask);
			assert (!"should not get here");
		}
	}

repeat_call:
	/* call the IA-32 syscall using the 64 bit wrapper */
	retval = ILP32_WRAPPER6 ((unsigned long) syscall_id,
				 (unsigned long) IA32_SYSCALL_PARAM1 (bc32),
				 param2,
				 (unsigned long) IA32_SYSCALL_PARAM3 (bc32),
				 (unsigned long) IA32_SYSCALL_PARAM4 (bc32),
				 (unsigned long) IA32_SYSCALL_PARAM5 (bc32),
				 (unsigned long) IA32_SYSCALL_PARAM6 (bc32));


	if (retval == -EINTR) {
		int signo;

		assert (is_long_syscall (syscall_id));
		assert (cur_thread_data->tid == SYSCALL (gettid, ()));
		DBPRINT (btl_prt_syscall, "interrupted long syscall\n");

		/* send the signal */
		if (!btlib_que_is_real_empty (cur_thread_data->pending_que) ) {
			/* for signal with permanent signal handler and blocked, 
			   the EINTR is ont a real one */
			signo = btlib_que_lastsig (cur_thread_data->pending_que);


			if (has_permanent_handler(signo) && 
			    sigismember32 (&cur_thread_data->sigmask, signo)) {
				goto repeat_call;
			}


			*cur_thread_data->long_syscall_flag = NOT_IN_SYSCALL;
			if ((syscall_id == IA32_SYSCALL_SIGSUSPEND) ||  
			    (syscall_id == IA32_SYSCALL_RT_SIGSUSPEND)) {
				restart_btg (NULL, &oldmask);  
			} else {
				restart_btg (NULL, NULL);  
			}
			assert (!"should not get here");
		}
	}
	return retval;
}

/*****************************************************************************/
/* implement the IA-32 wait4 and waitpid syscalls */
/*****************************************************************************/
static long
ia32_wait (BTGENERIC_IA32_CONTEXT *bc32)
{
	int child_pid = 0;
	int *statusp = NULL;
	int *orig_statusp = NULL;
	int pid, options;
	int status;

	/* get the syscall params */
	pid = (int) IA32_SYSCALL_PARAM1 (bc32);
	orig_statusp = U2P (IA32_SYSCALL_PARAM2 (bc32));
	options = (int) IA32_SYSCALL_PARAM3 (bc32);

	if (orig_statusp == NULL) {
		statusp = &status;
	} else {
		statusp = orig_statusp;
	}
	assert (statusp != NULL);



#if PTEMU_ENABLED
repeat_waiting:
#endif
	DBPRINT (btl_prt_syscall, "pid=%d status=%p options=%x\n",
	         pid, statusp, options);

	child_pid = ia32_system_call ((BTGENERIC_IA32_CONTEXT *) bc32, statusp);

	/* see whether the syscall completed */
	DBPRINT (btl_prt_syscall, "lsf=%ld\n", *cur_thread_data->long_syscall_flag);
	if (*cur_thread_data->long_syscall_flag != SYSCALL_COMPLETED) {
		goto out;
	}

	DBPRINT (btl_prt_pt, "ia32_system_call=%d status=%d\n", 
		 child_pid, *statusp);
#if PTEMU_ENABLED
	if (child_pid > 0) {
		long retval;
		/* 
		 * do the work for the ptrace emulation. btlib_ptemu_wait
		 * returns with either FINISH_WAIT or DISCARD_WAIT. 
		 * in case of FINISH_WAIT there is nothing special to do. 
		 * in case of DISCARD_WAIT we need to pretend as if the wait
		 * syscall did not return any info. so - if the syscall is blocking
		 * we just repeat it. if it is non-blocking then we change the 
		 * return-value to 0.
		 */
		retval = btlib_ptemu_wait (child_pid, statusp);
		DBPRINT (btl_prt_syscall, "btlib_ptemu_wait=%s\n",
		         (retval == DISCARD_WAIT) ? "DISCARD" : "FINISH");
		if (retval == DISCARD_WAIT) {
			DBPRINT (btl_prt_syscall, "WNOHANG=%d\n",
			         (options & WNOHANG) ? 1 : 0);

			if (options & WNOHANG) {
				child_pid = 0;
			} else {
				*cur_thread_data->long_syscall_flag = AFTER_CHECK_QUE;	
				goto repeat_waiting;
			}
		}
	}
#endif

#if DEBUG
	if (btl_prt_thread | btl_prt_syscall) {
		if (child_pid > 0) {
			if (__WIFEXITED (*statusp)) {
				DBPRINT (TRUE, "exit %d\n",
					 __WEXITSTATUS (*statusp));
			} else if (__WIFSIGNALED (*statusp)) {
				DBPRINT (TRUE, "kill signal %d\n",
					 __WTERMSIG (*statusp));
			} else {
				DBPRINT (TRUE, "stop signal %d\n",
					 __WSTOPSIG (*statusp));
			}
		}
	}
#endif
	/* if we are sure that the thread terminated then cleanup its resources */
	if ((child_pid > 0) && !__WIFSTOPPED (*statusp)) {
		btlib_cleanup_thread (child_pid);
	}
out:
	return child_pid;
}

/*****************************************************************************/
/*
 * btlib_system_call
 * Remarks: most of the syscalls are delegated to the system  - 
 *          this function calls ia32_system_call, which calls
 *          ilp32_wrapper. but some of the syscall are handled
 *          internally by btlib. 
 *          as for now, all long syscalls are delegated to the system.
 *          if this will change - the function which handles the long
 *          syscall will have to take care of interrupted syscalls, the
 *          same way it is done in ia32_system_call.
 */
/*****************************************************************************/
void
btlib_system_call ()
{
	long retval = -ENOSYS;
	int syscall_id;
	int is_rt = 1;
#if PTEMU_ENABLED
	int ptrace_syscall = 0;
#endif
	syscall_id = IA32_SYSCALL_ID (cur_thread_data->bc32);
	DBPRINT (btl_prt_syscall, "id-%d(%s) called eip=%x esp=%x\n",
		 syscall_id,
		 (syscall_id <= SYSCALL_NAMES_MAX_ID) ?
		 syscall_names[syscall_id] : "???",
		 cur_thread_data->bc32->eip, cur_thread_data->bc32->esp);
	
	STATCODE (btl_stats, btlib_update_stat (BTL_STAT_SYSCALL, COUNT, 0ul));
	
#if DEBUG && 0
	print_syscall_before (syscall_id);
#endif
	
	/* 
	   make sure the que is empty before invoking
	   1 - long syscalls 
	   2 - system call that may affect the pending que, e.g., 
	       sigaction(SIG_IGN) should remove corresponding pending 
	       signal
	 */


	STATCODE (btl_stats && is_long_syscall (syscall_id), 
		  btlib_update_stat (BTL_STAT_LSYSCALL, COUNT, 0ul));

	if (is_long_syscall (syscall_id) || need_deliver_pending (syscall_id)) {
		DBPRINT (btl_prt_syscall, "long syscall\n");
		assert (*cur_thread_data->long_syscall_flag == NOT_IN_SYSCALL);

		*cur_thread_data->long_syscall_flag = CHECK_QUE;
		/* it's enough to check the private que here */
		if (!btlib_que_is_empty (cur_thread_data->pending_que)) {
			DBPRINT (btl_prt_syscall, "signal pending\n");
			*cur_thread_data->long_syscall_flag = NOT_IN_SYSCALL;
			longjmp (cur_thread_data->btgeneric_run_context, DELIVER);
			assert (!"should not get here");
		}
		*cur_thread_data->long_syscall_flag = AFTER_CHECK_QUE;
	}

	/* advance eip pass the int 0x80 instruction */
	cur_thread_data->syscall_id = cur_thread_data->bc32->eax;
	cur_thread_data->bc32->eax = (unsigned) -ENOSYS;
	cur_thread_data->bc32->eip += 2;

#if PTEMU_ENABLED
	/* 
	 * if there is a ptrace_syscall request then suspend yourself 
	 * in IA-32 systems the os prepares things as follows - 
	 * 1) syscall_id is kept in orig_eax
	 * 2) eax holds -ENOSYS
	 * 3) the debugee is stopped with sigtrap
	 * 4) if the option TRACESYSGOOD is set then the signal number 
	 *    is ored with 0x80
	 *
	 * we try to emulate all this stuff here. the orig_eax is 
	 * kept in the tls. 
	 */
	if ((cur_thread_data->ptrace & PTEMU_PTRACED) &&
	    (cur_thread_data->ptrace & PTEMU_SYSCALL)) {

		ptrace_syscall = 1;

		/* put SIGTRAP in the tia_signal and suspend yourself
		   by sending PTEMU_SIG. this will be caught by 
		   btlib of the debugger */
		memset (&cur_thread_data->tia_signal, 0,
			sizeof (cur_thread_data->tia_signal));
		cur_thread_data->tia_signal.siginfo.si_signo = SIGTRAP;
		cur_thread_data->tia_signal.siginfo.si_signo |= IA32_SYSCALL_INT;

		DBPRINT (btl_prt_pt, "PTEMU_SYSCALL syscall_id=%d si_signo=%d\n",
			 syscall_id,
			 cur_thread_data->tia_signal.siginfo.si_signo);

		retval = SYSCALL (tkill, (cur_thread_data->tid, PTEMU_SIG0));
		sys_err_chk (retval, "kill");
	}
#endif		// PTEMU_ENABLED

	switch (syscall_id) {
	case IA32_SYSCALL_OPEN:
		retval = ia32_system_call (cur_thread_data->bc32, NULL);
		if ( ! IS_SYS_ERROR (retval) ) {
			if (is_proc_self_file (U2P(cur_thread_data->bc32->ebx), "stat")) {
				insert_proc_self_fd ((int)retval,
						cur_thread_data->tid,
						pre_read_proc_stat,
						post_read_proc_stat);
			}
		}
		break;
	case IA32_SYSCALL_CLOSE:
		retval = ia32_system_call (cur_thread_data->bc32, NULL);
		if ( ! IS_SYS_ERROR (retval) ) {
			remove_proc_self_fd ((int)cur_thread_data->bc32->ebx);
		}
		break;
	case IA32_SYSCALL_READ:
		retval = pre_read (cur_thread_data->bc32);
		if (retval == 0) {
			retval = ia32_system_call (cur_thread_data->bc32, NULL);
			if ( ! IS_SYS_ERROR (retval) ) {
				retval = post_read (cur_thread_data->bc32, retval);
			}
		}
		break;
	case IA32_SYSCALL_VFORK:
		/* TODO - add special handling for vfork */
		/* fall through */
	case IA32_SYSCALL_FORK:
		retval = btlib_fork (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_READLINK:
		retval = btlib_readlink (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_CLONE:
		retval = btlib_clone (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_MMAP:
		retval = btlib_ia32_mmap (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_MMAP2:
		retval = btlib_ia32_mmap2 (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_MUNMAP:
		retval = btlib_ia32_munmap (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_MREMAP:
		retval = btlib_ia32_mremap (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_EXIT_GROUP:
		btlib_group_exit (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_EXIT:
		btlib_thread_exit (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_SIGACTION:
		retval = btlib_sigaction (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_RT_SIGACTION:
		retval = btlib_rt_sigaction (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_SIGNAL:
		retval = btlib_signal (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_SIGALTSTACK:
		retval = btlib_sigaltstack (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_SIGRETURN:
		is_rt = 0;
		/* fall through */
	case IA32_SYSCALL_RT_SIGRETURN:
		btlib_sigreturn (cur_thread_data->bc32, is_rt);
		break;
	case IA32_SYSCALL_MPROTECT:
		retval = btlib_ia32_mprotect (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_WAITPID:
	case IA32_SYSCALL_WAIT4:
		retval = ia32_wait (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_PERSONALITY:
		retval = btlib_personality (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_MODIFY_LDT:
		retval = btlib_modify_ldt (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_PTRACE:
#ifdef PTEMU_ENABLED
		retval = btlib_ptrace (cur_thread_data->bc32);
		break;
#endif
	case IA32_SYSCALL_SET_THREAD_AREA:
		retval = btlib_set_thread_area (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_GET_THREAD_AREA:
		retval = btlib_get_thread_area (cur_thread_data->bc32);
		break;
	case IA32_SYSCALL_SET_TID_ADDRESS:
		retval = btlib_set_tid_address (cur_thread_data->bc32);
		break;

	case IA32_SYSCALL_USELIB:
		assert (!"unimplemented syscall");
		retval = -ENOSYS;
		break;
#if ALL_THREAD_AFFINITY
	case IA32_SYSCALL_SCHED_SETAFFINITY:
		/* if IA-32 thread explicitly calls setaffinity()
		   we'll pass the call to the wrapper, and if it succeeds,
		   we record the new explicitly-set mask so it will be inherited
		   correctly in case of fork or clone.
		   The BTL_LOCK_IND_THREAD cs is locked to prevent race condition 
		   between the thread that calls setaffinity and a thread that 
		   does shared-mem affinitiy */
		btlib_acquire_lock (BTL_LOCK_IND_THREAD);
		retval = ia32_system_call (cur_thread_data->bc32, NULL);
		if (!IS_SYS_ERROR (retval)) {
			/* param1 of the call is target_tid */
			btlib_record_explicit_affinity_request (
				(int)IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));
		}
		btlib_release_lock (BTL_LOCK_IND_THREAD);
		break;
#endif	/* ALL_THREAD_AFFINITY */


#if defined(SHMEM_PROC_AFFINITY) || defined(SHMEM_ORDER_SOLUTION)
	case IA32_SYSCALL_IPC:
		/* ipc call of type (first param) "SHMAT", is a request for shared memory.
		   In this case we have to apply the shared-memory-process-affinity solution */
		if ((int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32) == SHMAT) {
			DBPRINT (btl_prt_mem || btl_prt_thread,
			         "ipc called with flag SHMAT, Applying shared memory solution\n");
#if SHMEM_PROC_AFFINITY
			
			apply_shared_memory_affinity_solution ();
#elif SHMEM_ORDER_SOLUTION
			apply_shared_memory_ordering_solution ();
#endif
		}
		/* now call the default handler of sys-calls */
		retval = ia32_system_call (cur_thread_data->bc32, NULL);
		break;
#endif
	case IA32_SYSCALL_SIGPROCMASK:
		is_rt = 0;
		/* fall through */
	case IA32_SYSCALL_RT_SIGPROCMASK:
		retval = btlib_sigprocmask (cur_thread_data->bc32, is_rt);
		break;
	case IA32_SYSCALL_SIGPENDING:
		is_rt = 0;
		/* fall through */
	case IA32_SYSCALL_RT_SIGPENDING:
		retval = btlib_sigpending (cur_thread_data->bc32, is_rt);
		break;
	default:
		retval = ia32_system_call (cur_thread_data->bc32, NULL);
		break;
	}

	DBPRINT (btl_prt_syscall, "id=%d(%s) - returned %lx(%ld)\n",
		 syscall_id,
		 (syscall_id <= SYSCALL_NAMES_MAX_ID) ?
		 syscall_names[syscall_id] : "???", retval, retval);

	/* check whether the long syscall was executed */
	if (is_long_syscall (syscall_id)) {
		long state = *cur_thread_data->long_syscall_flag;
		DBPRINT (btl_prt_sig, "state=%ld\n", state);
		*cur_thread_data->long_syscall_flag = NOT_IN_SYSCALL;
		if ((state != SYSCALL_COMPLETED) && (state != AFTER_CHECK_QUE)) {
			/* the syscall was not executed. repeat it */
			cur_thread_data->bc32->eip -= 2;
			cur_thread_data->bc32->eax = cur_thread_data->syscall_id;
			cur_thread_data->syscall_id = IA32_SYSCALL_NONE;
			goto out;
		}
	}

#if DEBUG && 1 
	print_syscall_after (syscall_id, retval);
#endif	// DEBUG

	/* store the return value in eax */
	cur_thread_data->bc32->eax = retval;

#if PTEMU_ENABLED
	/* 
	 * in case of fork() -
	 * if the child process is ptraced then it should be stopped with SIGSTOP
	 */
	if (((syscall_id == IA32_SYSCALL_FORK) || (syscall_id == IA32_SYSCALL_CLONE)) &&
	    (cur_thread_data->bc32->eax == 0)) {

		if (btl_is_ptraced != IS_PTRACED_NO) {
			DBPRINT (btl_prt_thread | btl_prt_pt, "child ptraced\n");

			cur_thread_data->ptrace = PTEMU_PTRACED;
			cur_thread_data->tia_signal.siginfo.si_signo = SIGSTOP;
			retval = SYSCALL (tkill, (cur_thread_data->tid, PTEMU_SIG0));
			sys_err_chk (retval, "kill");
		}
	} else if (ptrace_syscall) {
		/* if there is a ptrace_syscall request then suspend yourself */
		DBPRINT (btl_prt_pt, "PTEMU_SYSCALL end syscall\n");
		/* 
		 * put SIGTRAP in the tia_signal and suspend yourself
		 * by sending PTEMU_SIG. this will be caught by 
		 * btlib of the debugger
		 */
		memset (&cur_thread_data->tia_signal, 0,
			sizeof (cur_thread_data->tia_signal));
		cur_thread_data->tia_signal.siginfo.si_signo = SIGTRAP;
		cur_thread_data->tia_signal.siginfo.si_signo |= IA32_SYSCALL_INT;
		retval = SYSCALL (tkill, (cur_thread_data->tid, PTEMU_SIG0));
		sys_err_chk (retval, "kill");
	}
#endif
	cur_thread_data->syscall_id = IA32_SYSCALL_NONE;
	*cur_thread_data->long_syscall_flag = NOT_IN_SYSCALL;

out:
	restart_btg (NULL, NULL);

	assert (!"should not get here");
}








/*****************************************************************************/
/* implement the readlink syscall - handle reads of /proc/self/exe and the like */
/*****************************************************************************/
static long btlib_readlink (BTGENERIC_IA32_CONTEXT * bc32)
{
	char * ulinkp = U2P (IA32_SYSCALL_PARAM1 (bc32));
	char * uret = U2P (IA32_SYSCALL_PARAM2 (bc32));
	unsigned long usize = IA32_SYSCALL_PARAM3 (bc32);
	long ret;

	if (is_proc_self_file (ulinkp, "exe")) {
		unsigned long act_size, t1;
		t1 = strnlen (btl_ia32_bin_full_path,
				sizeof (btl_ia32_bin_full_path) - 1);
		act_size = MIN (t1, usize);
		ret = safe_memcpy (uret, btl_ia32_bin_full_path, act_size);
		ret = IS_SYS_ERROR (ret) ? -EFAULT : (int)act_size;
	} else {
		/* otherwise, just delegate to the kernel */
		ret = ILP32_WRAPPER3 (IA32_SYSCALL_READLINK, ulinkp, uret, usize);
	}

	DBPRINT (0, "ret=%ld\n", ret);
	return ret;
}




/*****************************************************************************/
/* implement the personality syscall - don't allow changing the personality  */
/*****************************************************************************/
#define GET_CURRENT_PERSONALITY 0xffffffff

static long btlib_personality (BTGENERIC_IA32_CONTEXT * bc32)
{
	unsigned person;
	unsigned person_bak;
	long ret;

	person = IA32_SYSCALL_PARAM1 (bc32);
	person_bak = person;

	/* do not put READ_IMPLIES_EXEC into kernel */
	if (person != GET_CURRENT_PERSONALITY) {
		person &= ~READ_IMPLIES_EXEC;
	}

	/* we do not permit the change of personality */
	if ((person != PER_LINUX) && (person != GET_CURRENT_PERSONALITY)) {
		DBPRINT (btl_prt_syscall, "cannot change personality\n");
		return -EINVAL;
	}

#if DEBUG
	person = SYSCALL (personality, ((unsigned long)GET_CURRENT_PERSONALITY));
	assert (person == PER_LINUX32);
#endif	
	
	if (cur_thread_data->personality == NULL) {
		ret = iti_personality_interface (&cur_thread_data->personality);
		assert (ret == 0);
	}

	/* build up the return value */
	ret = PER_LINUX;
	if (RX_PERSON(*cur_thread_data->personality)) {
		ret |= READ_IMPLIES_EXEC;
	}

	/* reset personality */	
	if (person_bak != GET_CURRENT_PERSONALITY) {
#if ! RELEASE
		if (btl_always_rie) {
			person_bak |= READ_IMPLIES_EXEC;
		}
#endif
		*cur_thread_data->personality = person_bak;
	}
		
	return ret;
}




/*****************************************************************************/
/*****************************************************************************/
/* TODO - the function assumes that the whole stat string was read
   at once from the file */
static int post_read_proc_stat (proc_self_t * fd_info,
		BTGENERIC_IA32_CONTEXT * bc32, long retval)
{
	assert ( ! IS_SYS_ERROR (retval) );
	char * buf = U2P (bc32->ecx);
	char ia32_stat_fields[2048];
	long ret;

	int state = 1;
	char * start_code;
	char * pending;
	unsigned long size;
	extern btrans_binary_map_t g_application_info;
	long j = 0;

	DBPRINT (1, "retval=%ld fd_info->flags=%x buf=%p<%s>\n",
			retval, fd_info->flags, buf, buf);
	assert ( ! IS_SYS_ERROR (retval) );

	if ((fd_info->flags & PROC_SELF_NOT_CUR_SET) && (retval > 0)) {
		ret = -EBADFD;
		goto out;
	}
	if (buf[0] == '\0') {
		ret = 0;
		goto out;
	}
	/* goto 'start_code' field */
	while (state > 0 && state < 26) {
		while (j < retval) {
			if (buf[j++] == ' ') {
				break;
			}
			if (j >= retval) {
				assert ( ! "reading stat file in chunks" );
				ret = -EBADFD;
				goto out;
			}
		}
		state ++;
	}
	assert (state == 26); 
	start_code = buf + j;

	/* goto pass 'eip' field ('pending' field) */
	while (state > 25 && state < 31) {
		while (j < retval) {
			if (buf[j++] == ' ') {
				break;
			}
			if (j >= retval) {
				assert ( ! "reading stat file in chunks" );
				ret = -EBADFD;
				goto out;
			}
		}
		state ++;
	}
	pending = buf + j;
	size = strlen (pending);

	/* prepare 32 bit fields */
	DBPRINT (0, "%x %x %x %x %x\n",
			g_application_info.start_code,
			g_application_info.end_code,
			g_application_info.start_stack,
			bc32->esp,
			bc32->eip);
	sprintf (ia32_stat_fields, "%u %u %u %u %u ",
			g_application_info.start_code,
			g_application_info.end_code,
			g_application_info.start_stack,
			bc32->esp,
			bc32->eip);

	/* copy to buffer */
	assert (start_code < pending);
	if (strlen (ia32_stat_fields) > (unsigned long)(pending - start_code)) {
		assert ( ! "ia32_stat_fields string is too long" );
		ret = -1;
		goto out;
	}
	strcpy (start_code, ia32_stat_fields);

	char * pp = start_code + strlen (ia32_stat_fields);
	memmove (pp, pending, size);
	*(pp + size) = '\0';
	ret = strlen (buf);
out:
	DBPRINT (1, "ret=%ld\n", ret);
	return ret;
}

/************************************************************************** 
* add an entry to the proc-fd array *
**************************************************************************/
void insert_proc_self_fd(int fd, int tid, pre_syscall_fnp_t pre_read_fnp,
		post_syscall_fnp_t post_read_fnp)
{
	unsigned j = 0;
	while (j < m_proc_self_fds_size) {
		if (m_proc_self_fds[j].fd != -1)
			break;
		j ++;
	}
	if (j == m_proc_self_fds_size) {
		unsigned long newsize = sizeof (m_proc_self_fds[0]) * (m_proc_self_fds_size + 1);
		m_proc_self_fds = realloc (m_proc_self_fds, newsize);
		if (m_proc_self_fds == NULL) {
			DBPRINT (TRUE, "realloc failed\n");
			btlib_abort_all ();
		}
		m_proc_self_fds_size ++;
		STATCODE (btl_stats, btlib_update_stat
				(BTL_STAT_PROC_FDS, MAX_VAL, m_proc_self_fds_size));
	}
	m_proc_self_fds[j].fd = fd;
	m_proc_self_fds[j].tid = tid;
	m_proc_self_fds[j].flags = 0;
	m_proc_self_fds[j].pre_read_fnp = pre_read_fnp;
	m_proc_self_fds[j].post_read_fnp = post_read_fnp;
}

/************************************************************************** 
* clear an entry in the proc-fd array *
**************************************************************************/
void remove_proc_self_fd (int fd)
{
	unsigned j = 0;
	while (j < m_proc_self_fds_size) {
		if (m_proc_self_fds[j].fd == fd)
			break;
		j ++;
	}
	if (j < m_proc_self_fds_size) {
		m_proc_self_fds[j].fd = -1;
	}
}


/************************************************************************** 
* init btlib sigreturn code address *
**************************************************************************/
int btlib_init_syscall_module (void)
{
	/* allocate one IA-32 kernel page */
	btl_kernel_page = iti_alloc_kernel_page (PROT_EXEC | PROT_READ | PROT_WRITE);
	if (btl_kernel_page < 0) {
		return -1;
	}

	btl_restorer = btl_kernel_page;
	btl_rt_restorer = btl_kernel_page + LEGACY_RESTORER_SIZE * 2;

	memcpy ((void *)btl_restorer, RESTORER_CODE, LEGACY_RESTORER_SIZE);
	memcpy ((void *)btl_rt_restorer, RT_RESTORER_CODE, LEGACY_RESTORER_SIZE);

	return 0;
}

/*****************************************************************************/
/* returns 1 if given file is like /proc/self, 0 otherwise  */
/*****************************************************************************/
static long is_proc_self_file (char * filename, char * procself_filename)
{
	char link[PATH_MAX + 2];
	long ret;
	int is_procself_file = 0;
	char proc_self_file[PATH_MAX];


	ret = safe_stringcopy (link, filename, sizeof(link) - 1);
	if (IS_SYS_ERROR (ret) || ret > PATH_MAX) {
		DBPRINT (btl_prt_syscall, "ret=%ld\n", ret);
		goto out;
	}
	link[ret] = '\0';
	DBPRINT (0, "link='%s'\n", link);

	/* we need to copy full-path of ia32 binary to the user given
	 * buffer if the link is something link /proc/self/exe */
	sprintf (proc_self_file, "/proc/self/%s", procself_filename);
	if (strncmp (proc_self_file, link,
				strlen (proc_self_file) + 1) == 0) {
		is_procself_file = 1;
	} else {
		char proc_pid_exe[PATH_MAX];
		sprintf (proc_pid_exe, "/proc/%d/%s", SYSCALL (gettid, ()),
				procself_filename);
		if (strncmp (proc_pid_exe, link,
					strlen (proc_pid_exe) + 1) == 0) {
			is_procself_file = 1;
		}
	}
out:
	DBPRINT (btl_prt_syscall, "retval=%d\n", is_procself_file);
	return is_procself_file;
}

/*****************************************************************************/
/*****************************************************************************/
static int pre_read_proc_stat (proc_self_t * fd_info, BTGENERIC_IA32_CONTEXT * bc32)
{
	int ret = 0;
	int fd = IA32_SYSCALL_PARAM1 (bc32);
	/* get the offset from begining of file */
	long offset = SYSCALL (lseek, (fd, 0l, SEEK_CUR));
	DBPRINT (1, "offset=%ld\n", offset);
	if (IS_SYS_ERROR (offset)) {
		ret = offset;
	} else {
		/* if the offset is not zero - it means that the user
		 * is trying to read /proc/self/stat in chunks, which
		 * btlib does not support - return an error value */
		if (offset != 0) {
			fd_info->flags |= PROC_SELF_NOT_CUR_SET;
			ret = 0;
		}
	}
	return ret;
}

#if DEBUG
/************************************************************************** 
* print info of sockecall syscall *
**************************************************************************/

char *call_names[] = {
	"",
	"SYS_SOCKET",
	"SYS_BIND",
	"SYS_CONNECT",
	"SYS_LISTEN",
	"SYS_ACCEPT",
	"SYS_GETSOCKNAME",
	"SYS_GETPEERNAME",
	"SYS_SOCKETPAIR",
	"SYS_SEND",
	"SYS_RECV",
	"SYS_SENDTO",
	"SYS_RECVFROM",
	"SYS_SHUTDOWN",
	"SYS_SETSOCKOPT",
	"SYS_GETSOCKOPT",
	"SYS_SENDMSG",
	"SYS_RECVMSG"
};
static void
print_socketcall_info (BTGENERIC_IA32_CONTEXT * bc32)
{
	int call;
	unsigned *a;
	char details_str[16 * 1024];
	struct sockaddr32 *sap;
	struct msghdr32 *smh;

	call = IA32_SYSCALL_PARAM1 (bc32);
	a = U2P (IA32_SYSCALL_PARAM2 (bc32));
	assert ((call > 0) && (call < SYS_RECVMSG) && (a != NULL));

	switch (call) {
	case SYS_SOCKET:
		sprintf (details_str, "family=%d type=%d protocal=%d", a[0],
			 a[1], a[2]);
		break;
	case SYS_BIND:
	case SYS_CONNECT:
	case SYS_ACCEPT:
	case SYS_GETSOCKNAME:
	case SYS_GETPEERNAME:
		sap = U2P (a[1]);
		sprintf (details_str, "fd=%d addr.family=%u "
			 "addr.data='%x %x %x %x %x %x %x %x %x %x %x %x %x %x' "
			 "addrlen=%x",
			 a[0],
			 sap->sa_family,
			 sap->sa_data[0], sap->sa_data[1], sap->sa_data[2],
			 sap->sa_data[3], sap->sa_data[4], sap->sa_data[5],
			 sap->sa_data[6], sap->sa_data[7], sap->sa_data[8],
			 sap->sa_data[9], sap->sa_data[10], sap->sa_data[11],
			 sap->sa_data[12], sap->sa_data[13], a[2]);
		break;
	case SYS_LISTEN:
		sprintf (details_str, "fd=%d backlog=%d", a[0], a[1]);
		break;
	case SYS_SOCKETPAIR:
		sprintf (details_str, "family=%d type=%d protocal=%d vec=%x",
			 a[0], a[1], a[2], a[3]);
		break;
	case SYS_SEND:
	case SYS_RECV:
		sprintf (details_str, "fd=%d buf=%x len=%x flags=%x", a[0],
			 a[1], a[2], a[3]);
		break;
	case SYS_SENDTO:
	case SYS_RECVFROM:
		sap = U2P (a[4]);
		sprintf (details_str,
			 "fd=%d buf=%x len=%x flags=%x addr.family=%u "
			 "addr.data='%x %x %x %x %x %x %x %x %x %x %x %x %x %x' "
			 "addrlen=%x", a[0], a[1], a[2], a[3], sap->sa_family,
			 sap->sa_data[0], sap->sa_data[1], sap->sa_data[2],
			 sap->sa_data[3], sap->sa_data[4], sap->sa_data[5],
			 sap->sa_data[6], sap->sa_data[7], sap->sa_data[8],
			 sap->sa_data[9], sap->sa_data[10], sap->sa_data[11],
			 sap->sa_data[12], sap->sa_data[13], a[5]);
		break;
	case SYS_SHUTDOWN:
		sprintf (details_str, "fd=%d how=%d", a[0], a[1]);
		break;
	case SYS_SETSOCKOPT:
	case SYS_GETSOCKOPT:
		sprintf (details_str,
			 "fd=%d level=%d optname=%d optval=%x optlen=%d", a[0],
			 a[1], a[2], a[3], a[4]);
		break;
	case SYS_SENDMSG:
	case SYS_RECVMSG:
		smh = U2P (a[1]);
		sprintf (details_str, "fd=%d "
			 "msg.name=%x msg.namelen=%x msg.iov=%x msg.iovlen=%x "
			 "msg.control=%x msg.controllen=%x msg.flags=%x "
			 "flags=%x",
			 a[0], smh->msg_name, smh->msg_namelen, smh->msg_iov,
			 smh->msg_iovlen, smh->msg_control, smh->msg_controllen,
			 smh->msg_flags, a[2]);
	}

	fprintf (g_log_file, "socketcall: call=%s %s\n", call_names[call],
		 details_str);
}

static void
print_getdents (BTGENERIC_IA32_CONTEXT * bc32)
{
	int fd = IA32_SYSCALL_PARAM1 (bc32);
	struct linux_dirent64 *UNUSED dirent = U2P (IA32_SYSCALL_PARAM2 (bc32));
	unsigned UNUSED count = IA32_SYSCALL_PARAM3 (bc32);

	fprintf (g_log_file, "getdents: fd=%d\n", fd);
}

char *ipc_calls[] = {
	"",
	"SEMOP",		/*1 */
	"SEMGET",		/*2 */
	"SEMCTL",		/*3 */
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"MSGSND",		/*11 */
	"MSGRCV",		/*12 */
	"MSGGET",		/*13 */
	"MSGCTL",		/*14 */
	"",
	"",
	"",
	"",
	"",
	"",
	"SHMAT",		/*21 */
	"SHMDT",		/*22 */
	"SHMGET",		/*23 */
	"SHMCTL"		/*24 */
};

char *sem_ctl_val[] = {
	"IPC_RMID",		/*0 */
	"IPC_SET",		/*1 */
	"IPC_STAT",		/*2 */
	"IPC_INFO",		/*3 */
	"",			/*4 */
	"",			/*5 */
	"",			/*6 */
	"",			/*7 */
	"",			/*8 */
	"",			/*9 */
	"",			/*10 */
	"GETPID",		/*11 */
	"GETVAL",		/*12 */
	"GETALL",		/*13 */
	"GETNCNT",		/*14 */
	"GETZCNT",		/*15 */
	"SETVAL",		/*16 */
	"SETALL",		/*17 */
	"SEM_STAT",		/*18 */
	"SEM_INFO"		/*19 */
};
char *msg_ctl_val[] = {
	"IPC_RMID",		/*0 */
	"IPC_SET",		/*1 */
	"IPC_STAT",		/*2 */
	"IPC_INFO",		/*3 */
	"",			/*4 */
	"",			/*5 */
	"",			/*6 */
	"",			/*7 */
	"",			/*8 */
	"",			/*9 */
	"",			/*10 */
	"MSG_STAT",		/*11 */
	"MSG_INFO"		/*12 */
};

char *shm_ctl_val[] = {
	"IPC_RMID",		/*0 */
	"IPC_SET",		/*1 */
	"IPC_STAT",		/*2 */
	"IPC_INFO",		/*3 */
	"",			/*4 */
	"",			/*5 */
	"",			/*6 */
	"",			/*7 */
	"",			/*8 */
	"",			/*9 */
	"",			/*10 */
	"SHM_LOCK",		/*11 */
	"SHM_UNLOCK",		/*12 */
	"SHM_STAT",		/*13 */
	"SHM_INFO"		/*14 */
};

struct msgbuf32 {
	__s32 mtype;
	char mtext[1];
};
#define IPC_64 0x0100

static void
print_ipc_info (BTGENERIC_IA32_CONTEXT * bc32)
{
	unsigned call, first, second, third, ptr, fifth, cmd, ipc64;
	char buffer[4096];
	char *p;
	unsigned j;

	call = IA32_SYSCALL_PARAM1 (bc32);
	first = IA32_SYSCALL_PARAM2 (bc32);
	second = IA32_SYSCALL_PARAM3 (bc32);
	third = IA32_SYSCALL_PARAM4 (bc32);
	ptr = IA32_SYSCALL_PARAM5 (bc32);
	fifth = IA32_SYSCALL_PARAM6 (bc32);

	sprintf (buffer, "(%d) print_ipc_info: call=%u('%s'):",
		 cur_thread_data->tid, call, 
		 (call > 0 && call < 25) ? ipc_calls[call] : "");

	p = buffer + strlen (buffer);

	switch (call) {
	case SEMOP:
	case SEMTIMEDOP:
		sprintf (p, "semid=%d nops=%d ", first, second);
		p += strlen (p);
		if (call == SEMTIMEDOP) {
			int *t32 = (void *) (uintptr_t) fifth;
			sprintf (p, "timeout=%u,%u", *t32, *(t32 + 1));
			p += strlen (p);
		}
		sprintf (p, "\n");
		p += strlen ("\n");

		for (j = 0; j < second; j++) {
			struct sembuf *sb =
			    (struct sembuf *) (uintptr_t) ptr + j;
			sprintf (p, "\tsem_num=%d sem_op=%d sem_flg=%x\n",
				 sb->sem_num, sb->sem_op, sb->sem_flg);
			p += strlen (p);
		}
		break;
	case SEMGET:
		sprintf (p, "key=%d nsems=%d semflg=%x\n",
			 first, second, third);
		p += strlen (p);
		break;
	case SEMCTL:
		ipc64 = (third & IPC_64) ? 1 : 0;
		cmd = ipc64 ? (third ^ IPC_64) : third;
		sprintf (p, "semid=%d semnum=%d cmd=%d('%s%s') val=%d(%x)\n",
			 first, second, third,
			 ipc64 ? "IPC_64 | " : "",
			 (cmd < 20) ? sem_ctl_val[cmd] : "", ptr, ptr);
		p += strlen (p);
		break;
	case MSGSND:
		sprintf (p, "msqid=%d msgsz=%d msgflg=%x\n",
			 first, second, third);
		p += strlen (p);
		for (j = 0; j < second; j++) {
			struct msgbuf32 *mb = (void *) (uintptr_t) ptr + j;
			sprintf (p, "\tmtype=%u mtext=%x\n", mb->mtype,
				 mb->mtext[0]);
			p += strlen (p);
		}
		break;
	case MSGRCV:
		{
			unsigned version = call >> 16;
			unsigned msgtyp = fifth;
			if (!version) {
				struct ipc_kludge *uipck =
				    (struct ipc_kludge *) (uintptr_t) ptr;
				assert (ptr != 0);

				msgtyp = uipck->msgtyp;
			}

			sprintf (p, "msqid=%d msgsz=%d msgtyp=%d msgflg=%x\n",
				 first, second, msgtyp, third);
			p += strlen (p);
		}
		for (j = 0; j < second; j++) {
			struct msgbuf32 *mb = (void *) (uintptr_t) ptr + j;
			sprintf (p, "\tmtype=%u mtext=%x\n", mb->mtype,
				 mb->mtext[0]);
			p += strlen (p);
		}
		break;
	case MSGGET:
		sprintf (p, "key=%d msgflg=%x\n", first, second);
		p += strlen (p);
		break;
	case MSGCTL:
		ipc64 = (second & IPC_64) ? 1 : 0;
		cmd = ipc64 ? (second ^ IPC_64) : second;
		sprintf (p, "msqid=%d cmd=%d('%s%s')\n", first, second,
			 ipc64 ? "IPC_64 | " : "",
			 (cmd <= 12) ? msg_ctl_val[cmd] : "");
		p += strlen (p);
		break;
	case SHMAT:
		sprintf (p, "shmid=%d shmflag=%x shmaddr=%x\n", first, second,
			 ptr);
		p += strlen (p);
		break;
	case SHMDT:
		sprintf (p, "shmaddr=%x\n", ptr);
		p += strlen (p);
		break;
	case SHMGET:
		sprintf (p, "key=%d size=%x shmflg=%x\n", first, second, third);
		p += strlen (p);
		break;
	case SHMCTL:
		ipc64 = (second & IPC_64) ? 1 : 0;
		cmd = ipc64 ? (second ^ IPC_64) : second;
		sprintf (p, "shmid=%d cmd=%d('%s%s')\n", first, second,
			 ipc64 ? "IPC_64 | " : "",
			 (cmd <= 14) ? shm_ctl_val[cmd] : "");
		p += strlen (p);
		break;
	default:
		sprintf (p, "unknown call\n");
	}

	fprintf (g_log_file, "%s", buffer);
}

static void 
print_syscall_before (int syscall_id) 
{
	if (syscall_id == IA32_SYSCALL_KILL) {
		DBPRINT (TRUE,
			 "syscall kill pid=%d signo=%d\n",
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32));
	}
	if (syscall_id == IA32_SYSCALL_TKILL) {
		DBPRINT (TRUE,
			 "syscall tkill tid=%d signo=%d\n",
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32));
	}

	if (syscall_id == IA32_SYSCALL_OPEN) {
		DBPRINT (TRUE,
			 "syscall open file-name='%s' flags=%o mode=%o\n",
			 (char *) U2P (IA32_SYSCALL_PARAM1 (cur_thread_data->bc32)),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM3 (cur_thread_data->bc32));
	}

	if (syscall_id == IA32_SYSCALL_WRITE) {
		unsigned count = IA32_SYSCALL_PARAM3 (cur_thread_data->bc32);
		char *str = U2P (IA32_SYSCALL_PARAM2 (cur_thread_data->bc32));
		char wbuf[2048];
		unsigned j;
		for (j = 0; (j < sizeof (wbuf) - 1) && (j < count); j++) {
			if ((*(str + j) < ' ') || (*(str + j) > '~')) {
				break;
			} else {
				wbuf[j] = *(str + j);
			}
		}
		wbuf[j] = 0;
		DBPRINT (TRUE, "syscall write fd=%d string='%s'\n",
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32), wbuf);

	}

	if (syscall_id == IA32_SYSCALL_EXECVE) {
		btlib_thread_data_t *kuku UNUSED = cur_thread_data;
		char *file = U2P (IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));
		unsigned *argv = U2P (IA32_SYSCALL_PARAM2 (cur_thread_data->bc32));
		int j;
		DBPRINT (TRUE, "syscall execve file='%s'\n",
			 file);
		for (j = 0; *argv != 0; j++, *argv++) {
			DBPRINT (TRUE, "\targv[%d]='%s'\n",
				 j, (char *) (uintptr_t) * argv);
		}
	}
#if 0
	if (syscall_id == IA32_SYSCALL_IPC) {
		print_ipc_info (cur_thread_data->bc32);
	}
	if (syscall_id == IA32_SYSCALL_STAT64) {
		fprintf (g_log_file,
			 "(%d) syscall stat64 file-name='%s'\n",
			 cur_thread_data->tid, (char *) (uintptr_t)
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));

	}
	if (syscall_id == IA32_SYSCALL_EXECVE) {
		btlib_thread_data_t *kuku UNUSED = cur_thread_data;
		char *file = U2P (IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));
		unsigned *argv = U2P (IA32_SYSCALL_PARAM2 (cur_thread_data->bc32));
		int j;
		fprintf (g_log_file, "(%d) syscall execve file='%s'\n",
			 cur_thread_data->tid, file);
		for (j = 0; *argv != 0; j++, *argv++) {
			fprintf (g_log_file, "\targv[%d]='%s'\n",
				 j, (char *) (uintptr_t) * argv);
		}
	}
	if (syscall_id == 258 /* set_tid_address */ ) {
		fprintf (g_log_file,
			 "(%d) syscall set_tid_address tidptr=%p\n",
			 cur_thread_data->tid,
			 U2P (IA32_SYSCALL_PARAM1 (cur_thread_data->bc32)));
	}
	if (syscall_id == IA32_SYSCALL_STAT64) {
		fprintf (g_log_file,
			 "(%d) syscall stat64 file-name='%s'\n",
			 cur_thread_data->tid, (char *) (uintptr_t)
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));

	}
	if (syscall_id == IA32_SYSCALL_ACCESS) {
		fprintf (g_log_file,
			 "(%d) syscall access file-name='%s' flags=%d mode=%d\n",
			 cur_thread_data->tid, (char *) (uintptr_t)
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM3 (cur_thread_data->bc32));
	}
	if (syscall_id == IA32_SYSCALL_LSTAT64) {
		fprintf (g_log_file,
			 "(%d) syscall lstat file-name='%s'\n",
			 cur_thread_data->tid, (char *) (uintptr_t)
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));

	}
	if (syscall_id == IA32_SYSCALL_IPC) {
		print_ipc_info (cur_thread_data->bc32);
	}
	if (syscall_id == IA32_SYSCALL_LLSEEK) {
		fprintf (g_log_file,
			 "(%d) syscall llseek fd=%d off_hi=%x off_lo=%x orig=%d\n",
			 cur_thread_data->tid,
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM3 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM5 (cur_thread_data->bc32));
	}
	if (syscall_id == IA32_SYSCALL_WRITE) {
		unsigned count = IA32_SYSCALL_PARAM3 (cur_thread_data->bc32);
		char *str = U2P (IA32_SYSCALL_PARAM2 (cur_thread_data->bc32));
		char wbuf[2048];
		unsigned j;
		for (j = 0; (j < sizeof (wbuf) - 1) && (j < count); j++) {
			if ((*(str + j) < ' ') || (*(str + j) > '~')) {
				break;
			} else {
				wbuf[j] = *(str + j);
			}
		}
		wbuf[j] = 0;
		fprintf (g_log_file, "(%d) syscall write fd=%d string='%s'\n",
			 cur_thread_data->tid,
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32), wbuf);

	}
	if (syscall_id == IA32_SYSCALL_SOCKETCALL) {
		print_socketcall_info (cur_thread_data->bc32);
	}
	if (syscall_id == IA32_SYSCALL_CLOSE) {
		fprintf (g_log_file, "(%d) syscall close fd=%d\n",
			 cur_thread_data->tid,
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));
	}
	if (syscall_id == 10 /*unlink */ ) {
		fprintf (g_log_file,
			 "(%d) syscall unlink file-name='%s'\n",
			 cur_thread_data->tid, (char *)
			 U2P (IA32_SYSCALL_PARAM1 (cur_thread_data->bc32)));
	}
	if (syscall_id == IA32_SYSCALL_OPEN) {
		fprintf (g_log_file,
			 "(%d) syscall open file-name='%s' flags=%d mode=%d\n",
			 cur_thread_data->tid, (char *)
			 U2P (IA32_SYSCALL_PARAM1 (cur_thread_data->bc32)),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM3 (cur_thread_data->bc32));
	}
	if (syscall_id == IA32_SYSCALL_EPOLL_CREATE) {
		fprintf (g_log_file,
			 "(%d) syscall epoll_create size=%d\n",
			 cur_thread_data->tid,
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32));
	}
	if (syscall_id == IA32_SYSCALL_EPOLL_CTL) {
		fprintf (g_log_file,
			 "(%d) syscall epoll_ctl epfd=%d op=%d fd=%d\n",
			 cur_thread_data->tid,
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM3 (cur_thread_data->bc32));
	}
	if (syscall_id == IA32_SYSCALL_EPOLL_WAIT) {
		fprintf (g_log_file,
			 "(%d) syscall epoll_wait epfd=%d maxevents=%d timeout=%d\n",
			 cur_thread_data->tid,
			 (int) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM3 (cur_thread_data->bc32),
			 (int) IA32_SYSCALL_PARAM4 (cur_thread_data->bc32));
	}
#endif
}


static void 
print_syscall_after (int syscall_id, long retval) 
{
	if (syscall_id == IA32_SYSCALL_READ) {
		long count = retval;
		char *str = (char *) (uintptr_t) IA32_SYSCALL_PARAM2 (cur_thread_data->bc32);
		char rbuf[2048];
		unsigned j;
		for (j = 0; (j < sizeof (rbuf) - 1) && (j < count); j++) {
			if (((*(str + j) < ' ') || (*(str + j) > '~')) && (*(str + j) != '\n')) {
				break;
			} else {
				rbuf[j] = *(str + j);
			}
		}
		rbuf[j] = 0;
		DBPRINT (TRUE, "syscall read j=%d retval=%ld fd=%d string='%s'\n",
			 j, retval, IA32_SYSCALL_PARAM1 (cur_thread_data->bc32), rbuf);
	}
#if 0
	if (syscall_id == IA32_SYSCALL_GETCWD) {
		char *buf = (char *) (uintptr_t) IA32_SYSCALL_PARAM1 (cur_thread_data->bc32);
		unsigned len = IA32_SYSCALL_PARAM2 (cur_thread_data->bc32);

		if ((retval > 0) && (retval < len)) {
			fprintf (g_log_file, "(%d) syscall getcwd='%s'\n",
				 cur_thread_data->tid, buf);
		} else {
			fprintf (g_log_file,
				 "(%d) syscall getcwd: len=%u retval=%ld\n",
				 cur_thread_data->tid, len, retval);
		}
	}

	if (syscall_id == IA32_SYSCALL_GETDENTS64) {
		print_getdents (cur_thread_data->bc32);
	}

	if (syscall_id == IA32_SYSCALL_READ) {
		long count = retval;
		char *str =
		    (char *) (uintptr_t) IA32_SYSCALL_PARAM2 (cur_thread_data->
							      bc32);
		char rbuf[2048];
		unsigned j;
		for (j = 0; (j < sizeof (rbuf) - 1) && (j < count); j++) {
			if ((*(str + j) < ' ') || (*(str + j) > '~')) {
				break;
			} else {
				rbuf[j] = *(str + j);
			}
		}
		rbuf[j] = 0;
		fprintf (g_log_file, "(%d) syscall read fd=%d string='%s'\n",
			 cur_thread_data->tid,
			 IA32_SYSCALL_PARAM1 (cur_thread_data->bc32), rbuf);
	}
#endif
}

#endif	// DEBUG
