/*  
 * btlib_syscall.h
 *  
 *  
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _BTLIB_SYSCALL_H
#define _BTLIB_SYSCALL_H


#define LEGACY_RESTORER_SIZE 8ul

#define IA32_POP_EAX	 (unsigned char)0x58
#define IA32_MOV_EAX_IMM (unsigned char)0xb8
#define IA32_INT	 (unsigned char)0xcd

extern const unsigned char RESTORER_CODE[];
extern const unsigned char RT_RESTORER_CODE[];

extern long btl_kernel_page;
extern long btl_restorer;
extern long btl_rt_restorer;

int btlib_init_syscall_module(void);

/*
 --------------------------------------------------------------------------------
 * long system call handling
 --------------------------------------------------------------------------------
 */

static inline int
is_long_syscall (int syscall_id)
{
	return ((syscall_id == IA32_SYSCALL_READ) ||
		(syscall_id == IA32_SYSCALL_WRITE) ||
		(syscall_id == IA32_SYSCALL_READV) ||
		(syscall_id == IA32_SYSCALL_WRITEV) ||
		(syscall_id == IA32_SYSCALL_WAITPID) ||
		(syscall_id == IA32_SYSCALL_WAIT4) ||
		(syscall_id == IA32_SYSCALL_OPEN) ||
		(syscall_id == IA32_SYSCALL_SOCKETCALL) ||
		(syscall_id == IA32_SYSCALL_FCNTL) ||
		(syscall_id == IA32_SYSCALL_FCNTL64) ||
		(syscall_id == IA32_SYSCALL_PAUSE) ||
		(syscall_id == IA32_SYSCALL_SELECT) ||
		(syscall_id == IA32_SYSCALL_POLL) ||
		(syscall_id == IA32_SYSCALL_NEWSELECT) ||
		(syscall_id == IA32_SYSCALL_SIGSUSPEND) ||
		(syscall_id == IA32_SYSCALL_RT_SIGSUSPEND) ||
		(syscall_id == IA32_SYSCALL_IPC) ||
		(syscall_id == IA32_SYSCALL_NANOSLEEP) ||
		(syscall_id == IA32_SYSCALL_FUTEX) ||
		(syscall_id == IA32_SYSCALL_RT_SIGTIMEDWAIT) ||
		(syscall_id == IA32_SYSCALL_IO_GETEVENTS) ||
		(syscall_id == IA32_SYSCALL_EPOLL_WAIT) ||
		(syscall_id == IA32_SYSCALL_KILL) ||
		(syscall_id == IA32_SYSCALL_TKILL) ||
		(syscall_id == IA32_SYSCALL_TGKILL) ||
		(syscall_id == IA32_SYSCALL_IOCTL)) ? 1 : 0;
}

static inline int
syscall_need_restart (int syscall_id)
{
	return ((syscall_id == IA32_SYSCALL_READ) ||
		(syscall_id == IA32_SYSCALL_WRITE) ||
		(syscall_id == IA32_SYSCALL_READV) ||
		(syscall_id == IA32_SYSCALL_WRITEV) ||
		(syscall_id == IA32_SYSCALL_WAITPID) ||
		(syscall_id == IA32_SYSCALL_WAIT4) ||
		(syscall_id == IA32_SYSCALL_OPEN) ||
		(syscall_id == IA32_SYSCALL_SOCKETCALL) ||
		(syscall_id == IA32_SYSCALL_FCNTL) ||
		(syscall_id == IA32_SYSCALL_FCNTL64)) ? 1 : 0;
}

#endif				// _BTLIB_SYSCALL_H
