/*  
 * new_pe.h
 *  
 * 
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Doron Ziffrony
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#ifndef __PE_H__
#define __PE_H__


/*
 * PE format image signature 
 * It is a 4-byte field whose file offset specified at offset 0x3c
 */
#define PE_SIGNATURE                    0x00004550  /*  PE00   */
#define PE_OFFSETOF_SIGNATURE_ADDRESS   0x3c
typedef BT_U32 pe_signature_type;


/*
 * coff_header_type   --  This header immediately follows PE format signature.  
 *                        It includes section and symbol table information.
 */

typedef struct {
  BT_U16    machine_type;             /* Type of machine this exe is for */
  BT_U16    number_of_sections;       /* Number of sections in file */
  BT_U32    time_date_stamp;          /* Time of file creation */
  BT_U32    symbol_table_ptr;         /* File offset of COFF symbol table */
  BT_U32    number_of_symbols;        /* Size of symbol table */
  BT_U16    optional_header_size;     /* Size of optional header (present in PE files) */
  BT_U16    characteristics;          /* Flags indicating attributes of the file */
} coff_header_type;

#define PE_MACHINE_TYPE_IA64       0x200   /*  Intel(R) Itanium(R)-based computer */


/*
 * opt_header_type   --  This type describes the optional header for PE32+ format
 *                       Parts of the header are Windows specific fields.
 */

typedef struct {
  BT_U16    magic;                                /* PE32+ signature */
  BT_U8     linker_major_version;                 /* Linker major version number */
  BT_U8     linker_minor_version;                 /* Linker minor version number */
  BT_U32    code_size;                            /* Sum of the sizes of all the code (text) sections */
  BT_U32    size_of_init_data;                    /* Sum of the sizes of all initialized data sections */
  BT_U32    size_of_bss;                          /* Sum of the sizes of all the uninitialized data sections */
  BT_U32    entry_point;                          /* Address of entry point releative to image base */
  BT_U32    base_of_code;                         /* Address of beginning of code section */
  BT_U64    load_address;                         /* Image preferred load address */
  BT_U8     reserved[24];
  BT_U32    image_size;                           /* Size of image (including headers) */
  BT_U8     reserved2[48];
  BT_U32    number_of_directories;                /* Number of entries in the data directory */
} opt_header_type;

#define PE_32PLUS_SIGNATURE               0x20b  /* PE32+ format signature */


/*
 * Array of data directories immediately follow the optional header in the image file
 */

typedef struct {
  BT_U32   rva;          /* RVA of the directory data */
  BT_U32   size;         /* Size, in bytes, of the directory data  */
} pe_directory_entry_type;

#define PE_NUMOF_DIRECTORIES            16
#define PE_EXPORT_DIRECTORY_IDX         0 /* Index of the export directory entry */
#define PE_BASE_REALLOC_DIRECTORY_IDX   5 /* Index of the base reallocation directory entry */

/* 
 *  Export directory
 */

typedef struct {
    BT_U32   export_flags;                              /* Reserved */
    BT_U32   time_date_stamp;                           /* Creation date & time */
    BT_U16   major_version;                             /* User-defined major version */
    BT_U16   minor_version;                             /* User-defined minor version */
    BT_U32   name_rva;                                  /* RVA of ASCII name */
    BT_U32   ordinal_base;                              /* Base ordinal number for exports */
    BT_U32   number_of_functions;                       /* Number of entries in the export address table */
    BT_U32   number_of_names;                           /* Number of entries in the name pointer table */
    BT_U32   functions_rva;                             /* RVA of export address table */
    BT_U32   names_rva;                                 /* RVA of export name table */
    BT_U32   ordinals_rva;                              /* RVA of the ordinal table */
} export_directory_type;


/* 
 *  Base relocation directory
 */

typedef struct {
    BT_U32   page_rva;                                  /* Relative virtual address of page with relocation entries */
    BT_U32   size_of_block;                             /* Total number of bytes in fixup block */
} fixup_block_type;

typedef BT_U16 reloc_entry_type;

/* 
 *  Relocation types
 */
#define PE_RELOC_TYPE_ABSOLUTE              0
#define PE_RELOC_TYPE_HIGH                  1
#define PE_RELOC_TYPE_LOW                   2
#define PE_RELOC_TYPE_HIGHLOW               3
#define PE_RELOC_TYPE_HIGHADJ               4
#define PE_RELOC_TYPE_MIPS_JMPADDR          5
#define PE_RELOC_TYPE_IA64_IMM64            9
#define PE_RELOC_TYPE_DIR64                 10

#define GET_RELOCATION_OFFSET(field)          ((field) & 0x0fff)
#define GET_RELOCATION_TYPE(field)            (((field) >> 12) & 0xf)
#define NEXT_RELOC_ENTRY(reloc_entry_ptr)     ((BT_U64)((BT_U64)(reloc_entry_ptr) + (reloc_entry_ptr)->size_of_block))
#define FIXUP_PAGE_SIZE                       4096


/*
 * Section header - row in the section table that immediately follows the data directories
 */
typedef struct {
  char              section_name[8];              /* Section name (NULL padded if shorter than 8 bytes) */
  BT_U32            total_section_size;           /* Total size of section when loaded to memory */
  BT_U32            virtual_address;              /* Address of section relative to the image load address */
  BT_U32            size_of_raw_data;             /* Size of the initialized data on disk */
  BT_U32            pointer_to_raw_data;          /* File pointer to the section's data */
  BT_U32            pointer_to_relocations;       /* File pointer to the beginning of the section's relocations */
  BT_U32            pointer_to_line_numbers;      /* File pointer to the line-number entries for the section */
  BT_U16            number_of_relocations;        /* Number of relocation entires for the section */
  BT_U16            number_of_line_numbers;       /* Number of linje-number entries for the section */
  BT_U32            characteristics;              /* Section charectaristics */
} section_header_type, *psection_header_type;


/*
 * Section characteristics
 */


#define PE_SECTION_CODE                       0x00000020  /*  Section contains code. */
#define PE_SECTION_INITIALIZED_DATA           0x00000040  /*  Section contains initialized data. */
#define PE_SECTION_UNINITIALIZED_DATA         0x00000080  /*  Section contains uninitialized data. */

#define PE_SECTION_ALIGN_1BYTES               0x00100000  /*  */
#define PE_SECTION_ALIGN_2BYTES               0x00200000  /*  */
#define PE_SECTION_ALIGN_4BYTES               0x00300000  /*  */
#define PE_SECTION_ALIGN_8BYTES               0x00400000  /*  */
#define PE_SECTION_ALIGN_16BYTES              0x00500000  /*  Default alignment if no others are specified. */
#define PE_SECTION_ALIGN_32BYTES              0x00600000  /*  */
#define PE_SECTION_ALIGN_64BYTES              0x00700000  /*  */

#define PE_SECTION_MEM_DISCARDABLE            0x02000000  /*  Section can be discarded. */
#define PE_SECTION_MEM_NOT_CACHED             0x04000000  /*  Section is not cachable. */
#define PE_SECTION_MEM_NOT_PAGED              0x08000000  /*  Section is not pageable. */
#define PE_SECTION_MEM_SHARED                 0x10000000  /*  Section is shareable. */
#define PE_SECTION_MEM_EXECUTE                0x20000000  /*  Section is executable. */
#define PE_SECTION_MEM_READ                   0x40000000  /*  Section is readable. */
#define PE_SECTION_MEM_WRITE                  0x80000000  /*  Section is writeable. */

/*
 * IMM64 RELOCATION FIXUP
 * 
 * IMM64 reloc layout (please refer to ABI doc for details):
 * 
 *    6         5         4         3         2         1         0
 * 3210987654321098765432109876543210987654321098765432109876543210
 * ||_______________________________________|||___||_______||_____|
 * ||                                       |||   ||       ||     |
 * I                   IMM41                 Ic      IMM9d   IMM7b
 *                                             IMM5c
 * In bundle:
 * slot2:
 * 09876543210987654321098765432109876543210
 *     ||_______||___|| |_____|
 *     I  IMM9d  IMM5c|  IMM7b
 *                    Ic   
 * slot1:
 * 09876543210987654321098765432109876543210
 * |_______________________________________|
 *                   IMM41
 * 
 */

#define I_MASK	0x0800000000000000UL
#define I_SHIFT	4

/* IMM41 is cross the 64-bits boundary, we have to seperate it into 2 parts  */
#define IMM41_1_MASK	0x00000000007fffffUL
#define IMM41_1_SHIFT	40

#define IMM41_2_MASK	0xffffc00000000000UL
#define IMM41_2_SHIFT	24

#define IC_MASK		0x0000100000000000UL
#define IC_SHIFT	23

#define IMM5C_MASK	0x0003e00000000000UL
#define IMM5C_SHIFT	29

#define IMM9D_MASK	0x07fc000000000000UL
#define IMM9D_SHIFT	43

#define IMM7B_MASK	0x000007f000000000UL
#define IMM7B_SHIFT	36

#define INST_HIGHT64_MASK ( I_MASK \
				| IMM41_1_MASK \
				| IC_MASK \
				| IMM5C_MASK \
				| IMM9D_MASK \
				| IMM7B_MASK )

#define INST_LOW64_MASK	IMM41_2_MASK

#endif

