/*  
 * ia32x_utils.h
 *  
 * main header for the utility lib
 * 
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _LIBC_SUBST_H
#define _LIBC_SUBST_H

/* definitions / declarations needed by btlib */
#include <kernel_subst64.h>
#include <kernel_subst32.h>

#ifndef __ASSEMBLER__

/* prototypes of exports of sub-modules */
#include "ia64_syscalls.h"
#include "mutex.h"
#include "strings.h"
#include "malloc.h"
#include "stdio.h"
#include "setjmp.h"
#include "assert.h"

/* size of the thread descriptor allocated by the library */
#define TLS_PRE_TCB_SIZE 16

/* get the version string */
void lcs_get_ia32x_utils_version (char *version_str);
/* need to be called before fork */
extern int lcs_before_fork (void);
/* need to be called after fork */
extern int lcs_after_fork (long is_parent);
/* kill current thread-group */
extern void abort (void);
/* prototype of main function - to be implemented by the user app */
extern int main (int argc, char ** argv, char ** envp);
/* print an error message to the user and into adminstrator's log files */
extern void notify_error (int err);

/* manage tls */
struct tcbhead;
extern long allocate_and_init_dtv (struct tcbhead *);
extern long deallocate_dtv (struct tcbhead *);
extern void get_tls_static_info (unsigned long *sizep, unsigned long *alignp);

/* fd of the binary to be emulated */
extern int g_is_at_execfd;
extern unsigned long g_at_execfd;
/* the address of __kernel_syscall_via_epc() */
extern unsigned long g_at_sysinfo;
/* the page size */
extern unsigned long g_at_pagesize;
/* security info */
extern int g_is_at_secure;
extern unsigned long g_at_secure;
/* fd of log file */
extern int g_log_file;

enum {NO_BTGENERIC=1};

#define ROUNDUP(v,b) (((v) + (b) - 1) & ~((b) - 1))
#define MAX(a,b) ((a)>(b)) ? (a) : (b)
#define MIN(a,b) ((a)<(b) ? (a) : (b))

#define UL(i)                   ((unsigned long)(i))

extern unsigned long g_page_size;
#ifdef PAGE_SIZE 
    #undef PAGE_SIZE
#endif
#define PAGE_SIZE g_page_size

#ifdef PAGE_SHIFT
    #undef PAGE_SHIFT
#endif
#define PAGE_SHIFT              ((g_page_size==4*1024)?12:\
			                    ((g_page_size==8*1024)?13:\
			                    ((g_page_size==16*1024)?14:\
			                    16)\
			                    ))

#define PAGE_MASK               (~(PAGE_SIZE - 1))
#define PAGE_OFF(addr)          ((addr) & ~PAGE_MASK)
#define PAGE_START(addr)        (unsigned long)((addr) & PAGE_MASK)
#define PAGE_ALIGN(addr)        (((addr) + PAGE_SIZE - 1) & PAGE_MASK)

#define IA64_STACK_ALIGNMENT    16
#define IA64_STACK_MASK         (~(unsigned long)(IA64_STACK_ALIGNMENT - 1))
#define ALIGN_STACK(tos)        tos = (void*)((uintptr_t)(tos) & IA64_STACK_MASK)
#define IA64_STACK_SIZE         0x1000000ul	/* 1MB */
#define IA64_PAGE_SHIFT ((btl_page_size == 4 * 1024)? 12:         \
                         ((btl_page_size == 8 * 1024)? 13:        \
                           ((btl_page_size == 16 * 1024)? 14:16)  \
                         )                                        \
                        )

/* READ_IMPLIES_EXEC personality */
#define RX_PERSON(per)	((per) & READ_IMPLIES_EXEC)

#endif  /* end of !__ASSEMBLER__ */
	
#endif // _LIBC_SUBST_H

