/* 
 * ia64_syscalls.h
 * prototypes of native 64 bit system calls
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _IA64_SYSCALLS_H
#define _IA64_SYSCALLS_H

/* these declarations are from /usr/include/linux/sem.h in RHEL3 */
union semun {
	int val;			/* value for SETVAL */
	void *buf;		/* buffer for IPC_STAT & IPC_SET */
	unsigned short *array;		/* array for GETALL & SETALL */
	void *__buf;		/* buffer for IPC_INFO */
	void *__pad;
};
struct sembuf {
	unsigned short  sem_num;	/* semaphore index in array */
	short		sem_op;		/* semaphore operation */
	short		sem_flg;	/* operation flags */
};

/* the following constant is derived from glibc code - 
 * see for instance sysdeps/unix/sysv/linux/sigprocmask.c */
#define KERN_SIGSET_SIZE 8
typedef struct {
	unsigned long sig[KERN_SIGSET_SIZE / sizeof (unsigned long)];
} k_sigset_t;


extern long sys_accept(int fd, void *upeer_sockaddr, int *upeer_addrlen);
extern long sys_access(const char * filename, int mode);
extern long sys_acct(const char * filename);
extern long sys_acct(const char *name);
extern long sys_adjtimex(void *txc_p);
extern unsigned long sys_alarm(unsigned int seconds);
extern long sys_bdflush(int func, long data);
extern long sys_bind(int fd, void *umyaddr, int addrlen);
extern unsigned long sys_brk(unsigned long _brk);
/*
extern long sys_capget(cap_user_header_t header, cap_user_data_t dataptr);
extern long sys_capset(cap_user_header_t header, const cap_user_data_t data);
*/
extern long sys_chdir(const char * filename);
extern long sys_chmod(const char * filename, mode_t mode);
extern long sys_chown(const char * filename, uid_t user, gid_t group);
extern long sys_chown16(const char * filename, unsigned user, unsigned group);
extern long sys_chroot(const char * filename);
extern int sys_clone2(unsigned long flags, unsigned long ustack_base, unsigned long ustack_size, unsigned long child_tidptr, unsigned long parent_tidptr, unsigned long tls);
extern long sys_close(int fd);
extern long sys_connect(int fd, void *uservaddr, int addrlen);
extern long sys_creat(const char * pathname, int mode);
extern long sys_create_module(const char *name_user, size_t size);
extern long sys_delete_module(const char *name_user);
extern long sys_dup(unsigned int fildes);
extern long sys_dup2(unsigned int oldfd, unsigned int newfd);
extern long sys_epoll_create(int size);
extern long sys_epoll_ctl(int epfd, int op, int fd, void *event);
extern long sys_epoll_wait(int epfd, void *events, int maxevents, int timeout);
extern long sys_execve (char *filename, char **argv, char **envp);
extern void sys_exit(int error_code) __attribute__((__noreturn__));
extern void sys_exit_group(int error_code) __attribute__((__noreturn__));
extern long sys_fchdir(int fd);
extern long sys_fchmod(int fd, mode_t mode);
extern long sys_fchown(int fd, uid_t user, gid_t group);
extern long sys_fchown16(int fd, unsigned user, unsigned group);
extern long sys_fcntl(int fd, unsigned int cmd, unsigned long arg);
extern long sys_fcntl64(int fd, unsigned int cmd, unsigned long arg);
extern long sys_fdatasync(int fd);
extern ssize_t sys_fgetxattr(int fd, char *name, void *value, size_t size);
extern ssize_t sys_flistxattr(int fd, char *list, size_t size);
extern long sys_flock(int fd, unsigned int cmd);
extern long sys_fremovexattr(int fd, char *name);
extern long sys_fsetxattr(int fd, char *name, void *value, size_t size, int flags);
extern long sys_fstat(int fd, void * statbuf);
extern long sys_fstat64(unsigned long fd, void * statbuf, long flags);
extern long sys_fstatfs(int fd, void * buf);
extern long sys_fsync(int fd);
extern long sys_ftruncate(int fd, unsigned long length);
extern long sys_ftruncate64(int fd, long length);
extern long sys_futex(unsigned  *uaddr, int op, int val, void *utime, unsigned  *uaddr2);
extern long sys_fw_init (const char *args, int arglen);
extern long sys_get_kernel_syms(void *table);
extern long sys_getcwd(char *buf, unsigned long size);
extern long sys_getdents(int fd, void * dirent, unsigned int count);
extern long sys_getdents64(int fd, void * dirent, unsigned int count);
extern long sys_getegid(void);
extern long sys_getegid16(void);
extern long sys_geteuid(void);
extern long sys_geteuid16(void);
extern long sys_getgid(void);
extern long sys_getgid16(void);
extern long sys_getgroups(int gidsetsize, gid_t *grouplist);
extern long sys_getgroups16(int gidsetsize, unsigned *grouplist);
extern long sys_gethostname(char *name, int len);
extern long sys_getitimer(int which, void *value);
extern unsigned long sys_getpagesize (void);
extern long sys_getpeername(int fd, void *usockaddr, int *usockaddr_len);
extern long sys_getpgid(pid_t pid);
extern long sys_getpgrp(void);
extern int sys_getpid(void);
extern long sys_getppid(void);
extern long sys_getpriority(int which, int who);
extern long sys_getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);
extern long sys_getresgid16(unsigned *rgid, unsigned *egid, unsigned *sgid);
extern long sys_getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);
extern long sys_getresuid16(unsigned *ruid, unsigned *euid, unsigned *suid);
extern long sys_getrlimit(unsigned int resource, void *rlim);
extern long sys_getrusage(int who, void *ru);
extern long sys_getsid(pid_t pid);
extern long sys_getsockname(int fd, void *usockaddr, int *usockaddr_len);
extern long sys_getsockopt(int fd, int level, int optname, char *optval, int *optlen);
extern int sys_gettid(void);
extern long sys_gettimeofday(void *tv, void *tz);
extern long sys_getuid(void);
extern long sys_getuid16(void);
extern long sys_getunwind (void *buf, size_t buf_size);
extern long sys_getxattr(char *path, char *name, void *value, size_t size);
extern long sys_init_module(const char *name_user, void *mod_user);
extern long sys_io_cancel(unsigned long ctx_id, void *iocb, void *result);
extern long sys_io_destroy(unsigned long ctx);
extern long sys_io_getevents(unsigned long ctx_id, long min_nr, long nr, void *events, void *timeout);
extern long sys_io_setup(unsigned nr_reqs, unsigned long *ctxp);
extern long sys_io_submit(unsigned long ctx_id, long nr, void **iocbpp);
extern long sys_ioctl(int fd, unsigned int cmd, unsigned long arg);
extern long sys_kill(int pid, int sig);
extern long sys_lchown(const char * filename, uid_t user, gid_t group);
extern long sys_lchown16(const char * filename, unsigned user, unsigned group);
extern ssize_t sys_lgetxattr(char *path, char *name, void *value, size_t size);
extern long sys_link(const char * oldname, const char * newname);
extern long sys_listen(int fd, int backlog);
extern long sys_listxattr(char *path, char *list, size_t size);
extern long sys_llistxattr(char *path, char *list, size_t size);
extern long sys_llseek(int fd, unsigned long offset_high, unsigned long offset_low, long * result, unsigned int origin);
extern int sys_lookup_dcookie(unsigned long cookie64, char * buf, size_t len);
extern long sys_lremovexattr(char *path, char *name);
extern off_t sys_lseek(int fd, off_t offset, unsigned int origin);
extern long sys_lsetxattr(char *path, char *name, void *value, size_t size, int flags);
extern long sys_lstat(char * filename, void * statbuf);
extern long sys_lstat64(char * filename, void * statbuf, long flags);
extern long sys_madvise(unsigned long start, size_t len, int behavior);
extern long sys_mincore(unsigned long start, size_t len, unsigned char * vec);
extern long sys_mkdir(const char * pathname, int mode);
extern long sys_mknod(const char * filename, int mode, dev_t dev);
extern long sys_mlock(unsigned long start, size_t len);
extern long sys_mlockall(int flags);
extern unsigned long sys_mmap (unsigned long addr, unsigned long len, int prot, int flags, int fd, long off);
extern unsigned long sys_mmap2 (unsigned long addr, unsigned long len, int prot, int flags, int fd, long pgoff);
extern long sys_mount(char * dev_name, char * dir_name, char * type, unsigned long flags, void * data);
extern long sys_mprotect(unsigned long start, size_t len, unsigned prot);
extern unsigned long sys_mremap(unsigned long addr, unsigned long old_len, unsigned long new_len, unsigned flags, unsigned long new_addr);
extern long sys_msgctl (int msqid, int cmd, void *buf);
extern long sys_msgctl (int msqid, int cmd, void *buf);
extern long sys_msgget (key_t key, int msgflg);
extern long sys_msgget (key_t key, int msgflg);
extern long sys_msgrcv (int msqid, void *msgp, size_t msgsz, long msgtyp, int msgflg);
extern long sys_msgsnd (int msqid, void *msgp, size_t msgsz, int msgflg);
extern long sys_msgsnd (int msqid, void *msgp, size_t msgsz, int msgflg);
extern long sys_msync(unsigned long start, size_t len, int flags);
extern long sys_munlock(unsigned long start, size_t len);
extern long sys_munlockall(void);
extern long sys_munmap(unsigned long addr, size_t len);
extern long sys_nanosleep(void *rqtp, void *rmtp);
extern long sys_newfstat(int fd, void * statbuf);
extern long sys_newlstat(char * filename, void * statbuf);
extern long sys_newstat(char * filename, void * statbuf);
extern long sys_newuname(void * name);
extern long sys_nfsservctl(int cmd, void *argp, void *resp);
extern long sys_ni_syscall(void);
extern long sys_nice(int increment);
extern long sys_old_getrlimit(unsigned int resource, void *rlim);
extern long sys_open(const char * filename, int flags, int mode);
extern long sys_pciconfig_read (unsigned long bus, unsigned long dfn, unsigned long off, unsigned long len, void * buf);
extern long sys_pciconfig_write (unsigned long bus, unsigned long dfn, unsigned long off, unsigned long len, void * buf);
extern long sys_perfmonctl (int pid, int cmd, void *req, int count);
extern long sys_personality(u_long personality);
extern long sys_pipe (void);
extern long sys_pivot_root(const char *new_root, const char *put_old);
extern long sys_poll(void * ufds, unsigned int nfds, long timeout);
extern long sys_prctl(int option, unsigned long arg2);
extern ssize_t sys_pread(int fd, void * buf, size_t count, long pos);
extern long sys_ptrace (int request, pid_t pid, unsigned long addr, unsigned long data);
extern ssize_t sys_pwrite(int fd, const char * buf, size_t count, long pos);
extern long sys_query_module(const char *name_user, int which, char *buf, size_t bufsize, size_t * ret);
extern long sys_quotactl(unsigned int cmd, const char *special, unsigned id, caddr_t addr);
extern ssize_t sys_read(int fd, void * buf, size_t count);
extern ssize_t sys_readahead(int fd, long offset, size_t count);
extern long sys_readlink(const char * path, char * buf, int bufsiz);
extern ssize_t sys_readv(unsigned long fd, const void * vector, unsigned long count);
extern long sys_reboot(int magic1, int magic2, unsigned int cmd, void * arg);
extern long sys_recv(int fd, void * ubuf, size_t size, unsigned flags);
extern long sys_recvfrom(int fd, void * ubuf, size_t size, unsigned flags, void *addr, int *addr_len);
extern long sys_recvmsg(int fd, void *msg, unsigned int flags);
extern long sys_remap_file_pages(unsigned long start, unsigned long size, unsigned long __prot, unsigned long pgoff, unsigned long flags);
extern long sys_removexattr(char *path, char *name);
extern long sys_rename(const char * oldname, const char * newname);
extern long sys_rmdir(const char * pathname);
extern long sys_rt_sigaction(int sig, const void *act, void *oact, size_t sigsetsize);
extern long sys_rt_sigpending(k_sigset_t *set, size_t sigsetsize);
extern long sys_rt_sigprocmask(int how, k_sigset_t *set, k_sigset_t *oset, size_t sigsetsize);
extern long sys_rt_sigqueueinfo(int pid, int sig, void *uinfo);
extern long sys_rt_sigsuspend(k_sigset_t * set, unsigned long sigsetsize);
extern long sys_rt_sigtimedwait(const void *uthese, void *uinfo, const void *uts, size_t sigsetsize);
extern long sys_sched_get_priority_max(int policy);
extern long sys_sched_get_priority_min(int policy);
extern long sys_sched_getaffinity(pid_t pid, unsigned int len, unsigned long *user_mask_ptr);
extern long sys_sched_getparam(pid_t pid, void *param);
extern long sys_sched_getscheduler(pid_t pid);
extern long sys_sched_rr_get_interval(pid_t pid, void *interval);
extern long sys_sched_setaffinity(pid_t pid, unsigned int len, unsigned long *user_mask_ptr);
extern long sys_sched_setparam(pid_t pid, void *param);
extern long sys_sched_setscheduler(pid_t pid, int policy, void *param);
extern long sys_sched_yield(void);
extern long sys_select(int n, fd_set *inp, fd_set *outp, fd_set *__exp, void *tvp);
extern long sys_semctl (int semid, int semnum, int cmd, union semun arg);
extern long sys_semget (key_t key, int nsems, int semflg);
extern long sys_semop (int semid, void *sops, unsigned nsops);
extern long sys_semtimedop (int semid, void *tsops,  unsigned nsops, const void *timeout);
extern long sys_send(int fd, void * buff, size_t len, unsigned flags);
extern ssize_t sys_sendfile(int out_fd, int in_fd, off_t *offset, size_t count);
extern ssize_t sys_sendfile64(int out_fd, int in_fd, long *offset, size_t count);
extern long sys_sendmsg(int fd, void *msg, unsigned flags);
extern long sys_sendto(int fd, void * buff, size_t len, unsigned flags, void *addr, int addr_len);
extern long sys_set_tid_address(int *tidptr);
extern long sys_setdomainname(char *name, int len);
extern long sys_setfsgid(gid_t gid);
extern long sys_setfsgid16(unsigned gid);
extern long sys_setfsuid(uid_t uid);
extern long sys_setfsuid16(unsigned uid);
extern long sys_setgid(gid_t gid);
extern long sys_setgid16(unsigned gid);
extern long sys_setgroups(int gidsetsize, gid_t *grouplist);
extern long sys_setgroups16(int gidsetsize, unsigned *grouplist);
extern long sys_sethostname(char *name, int len);
extern long sys_setitimer(int which, void *value,  void *ovalue);
extern long sys_setpgid(pid_t pid, pid_t pgid);
extern long sys_setpriority(int which, int who, int niceval);
extern long sys_setregid(gid_t rgid, gid_t egid);
extern long sys_setregid16(unsigned rgid, unsigned egid);
extern long sys_setresgid(gid_t rgid, gid_t egid, gid_t sgid);
extern long sys_setresgid16(unsigned rgid, unsigned egid, unsigned sgid);
extern long sys_setresuid(uid_t ruid, uid_t euid, uid_t suid);
extern long sys_setresuid16(unsigned ruid, unsigned euid, unsigned suid);
extern long sys_setreuid(uid_t ruid, uid_t euid);
extern long sys_setreuid16(unsigned ruid, unsigned euid);
extern long sys_setrlimit(unsigned int resource, void *rlim);
extern long sys_setsid(void);
extern long sys_setsockopt(int fd, int level, int optname, char *optval, int optlen);
extern long sys_settimeofday(void *tv, void *tz);
extern long sys_setuid(uid_t uid);
extern long sys_setuid16(unsigned uid);
extern long sys_setxattr(char *path, char *name, void *value, size_t size, int flags);
extern long sys_sgetmask(void);
extern long sys_shmat (int shmid, char *shmaddr, int shmflg);
extern long sys_shmctl (int shmid, int cmd, void *buf);
extern long sys_shmdt (char *shmaddr);
extern long sys_shmget (key_t key, size_t size, int shmflag);
extern long sys_shutdown(int fd, int how);
extern long sys_sigaltstack (const void *uss, void *uoss);
extern long sys_signal(int sig, void * handler);
extern long sys_sigpending(unsigned long *set);
extern long sys_sigprocmask(int how, unsigned long *set, unsigned long *oset);
extern long sys_socket(int family, int type, int protocol);
extern long sys_socketcall(int call, unsigned long *args);
extern long sys_socketpair(int family, int type, int protocol, int usockvec[2]);
extern long sys_ssetmask(int newmask);
extern long sys_stat(char * filename, void * statbuf);
extern long sys_stat64(char * filename, void * statbuf, long flags);
extern long sys_statfs(const char * path, void * buf);
extern long sys_stime(int * tptr);
extern long sys_swapoff(const char * specialfile);
extern long sys_swapon(const char * specialfile, int swap_flags);
extern long sys_symlink(const char * oldname, const char * newname);
extern long sys_sync(void);
extern long sys__sysctl(void *args);
extern long sys_sysfs(int option, unsigned long arg1, unsigned long arg2);
extern long sys_sysinfo(void *info);
extern long sys_syslog(int type, char * buf, int len);
extern long sys_tgkill(int tgid, int pid, int sig);
extern long sys_times(void * tbuf);
extern long sys_tkill(int pid, int sig);
extern long sys_truncate(const char * path, unsigned long length);
extern long sys_truncate64(const char * path, long length);
/*
extern int sys_tux (unsigned int action, user_req_t *u_info);
*/
extern long sys_umask(int mask);
extern long sys_umount(char * name, int flags);
extern long sys_unlink(const char * pathname);
extern long sys_uselib(const char * library);
extern long sys_ustat(dev_t dev, void * ubuf);
extern long sys_utime(char * filename, void * times);
extern long sys_utimes(char * filename, void * utp);
extern long sys_vhangup(void);
extern long sys_vm86 (long arg0, long arg1, long arg2, long arg3);
extern long sys_wait4(pid_t pid,unsigned int * stat_addr, int options, void * ru);
extern ssize_t sys_write(int fd, const void * buf, size_t count);
extern ssize_t sys_writev(unsigned long fd, const void * vector, unsigned long count);
extern long sys_timer_create(int clockid, void* evp, void* timerid);
extern long sys_timer_settime(int timerid, int flags, void* value, void* ovalue);
extern long sys_timer_gettime(int timerid, void* value);
extern long sys_clock_settime(int clk_id, void* p);
extern long sys_clock_gettime(int clk_id, void* p);
extern long sys_clock_getres(int clk_id, void* p);
extern long sys_clock_nanosleep(int clk_id, int flags, void* qp, void* mp);
extern long sys_fstatfs64(unsigned int fd, size_t sz, void* buf);

# define SYSCALL(s,p) sys_##s p

#define SAFE_SYSCALL(syscall_name, syscall_params)                             \
({                                                                             \
	int _retval;                                                           \
	_retval = SYSCALL (syscall_name, syscall_params);                      \
	sys_err_chk((long)_retval, #syscall_name " syscall failed, aborting"); \
	_retval;                                                               \
})

#endif // _IA64_SYSCALLS_H
