/* 
 * kernel_subst32.h
 * macro / structure definitions on 32-bits linux
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _KERNEL_SUBST32_H
#define _KERNEL_SUBST32_H

/* some general definitions are defined in kernel_subst64.h,
 *  so this header file should not be included directly */
#ifndef _KERNEL_SUBST64_H
# error "Never include kernel_subst32.h without including <kernel_subst64.h>."
#endif

#ifndef __ASSEMBLER__

#define IA32_CLOCKS_PER_SEC     100 /* Cast in stone for IA32 Linux */
#define IA32_TICK(tick)         ((unsigned long long)(tick) * IA32_CLOCKS_PER_SEC / CLOCKS_PER_SEC)

#define IA32_LDT_ENTRIES        8192    /* Maximum number of LDT entries supported. */

#define _IA32_NSIG	            64
#define _IA32_NSIG_BPW	        32
#define _IA32_NSIG_WORDS        (_IA32_NSIG / _IA32_NSIG_BPW)

#define IA32_TASK_UNMAPPED_BASE IA32_MAX_USER_ADDR/3
#define IA32_PAGE_SHIFT         12  /* 4KB pages */
#define IA32_PAGE_SIZE          (1UL << IA32_PAGE_SHIFT)
#define IA32_PAGE_MASK          (~(IA32_PAGE_SIZE - 1))
#define IA32_PAGE_START(addr)   (unsigned long)((addr) & IA32_PAGE_MASK)
#define IA32_PAGE_ALIGN(addr)   (((addr) + IA32_PAGE_SIZE - 1) & IA32_PAGE_MASK)
#define IA32_PAGE_OFF(offset)   ((offset) & ~IA32_PAGE_MASK)

#define IA32_IOBASE             0x2000000000000000 /* Virtual address for I/O space */

#define MAX_IA32_UINT   0xffffffff

#define IA32_MAX_USER_ADDR  0xC0000000

#define IA32_MIN_KERNEL_ADDR	IA32_MAX_USER_ADDR
#define IA32_MAX_KERNEL_ADDR	0x100000000
/* address for memory mapping in IA-32 processes (TASK_SIZE/3) */
#define IA32_MAPPING_BASE    (IA32_MAX_USER_ADDR / 3)
/* highest address for IA32 users */
#define IA32_STACK_TOP  IA32_MAX_USER_ADDR

/* page shift of IA-32 processes */
#define IA32_PAGE_SHIFT         12	/* 4KB pages */

struct user_regs_struct_32 {
	unsigned ebx, ecx, edx, esi, edi, ebp, eax;
	unsigned short ds, __ds, es, __es;
	unsigned short fs, __fs, gs, __gs;
	unsigned orig_eax, eip;
	unsigned short cs, __cs;
	unsigned eflags, esp;
	unsigned short ss, __ss;
};

typedef __kernel_fsid_t	__kernel_fsid_t32;
typedef int	__clock_t32;
typedef int	__time_t32;
typedef unsigned short __uid_t32;
typedef unsigned int __size_t32;
typedef unsigned int __uid32_t32;
typedef unsigned short __gid_t32;
typedef unsigned int __gid32_t32;
typedef unsigned short __mode_t32;
typedef unsigned short __ipc_pid_t32;
typedef int __pid_t32;

/* from bits/fcntl.h */
struct flock32 {
       short l_type; 
       short l_whence;
       int l_start;
       int l_len;
       int l_pid;
};

/* from bits/stat.h */
struct stat32 {
       unsigned short st_dev;
       unsigned short __pad1;
       unsigned int st_ino;
       unsigned short st_mode;
       unsigned short st_nlink;
       unsigned short st_uid;
       unsigned short st_gid;
       unsigned short st_rdev;
       unsigned short __pad2;
       unsigned int  st_size;
       unsigned int  st_blksize;
       unsigned int  st_blocks;
       unsigned int  st_atim;
       unsigned int  __unused1;
       unsigned int  st_mtim;
       unsigned int  __unused2;
       unsigned int  st_ctim;
       unsigned int  __unused3;
       unsigned int  __unused4;
       unsigned int  __unused5;
};

struct stat64_32 {
	unsigned long st_dev;
	unsigned char	__pad0[4];
	unsigned int	__st_ino;
	unsigned int	st_mode;
	unsigned int	st_nlink;
	unsigned int	st_uid;
	unsigned int	st_gid;
	unsigned long	st_rdev;
	unsigned char	__pad3[4];
	unsigned int	st_size_lo;
	unsigned int	st_size_hi;
	unsigned int	st_blksize;
	unsigned int	st_blocks;	/* Number 512-byte blocks allocated. */
	unsigned int	__pad4;		/* future possible st_blocks high bits */
	unsigned int	st_atim;
	unsigned int	st_atim_nsec;
	unsigned int	st_mtim;
	unsigned int	st_mtim_nsec;
	unsigned int	st_ctim;
	unsigned int	st_ctim_nsec;
	unsigned int	st_ino_lo;
	unsigned int	st_ino_hi;
};

struct statfs32 {
       int f_type;
       int f_bsize;
       int f_blocks;
       int f_bfree;
       int f_bavail;
       int f_files;
       int f_ffree;
       __kernel_fsid_t32 f_fsid;
       int f_namelen; 
       int f_spare[6];
};

struct statfs64_32 {
	__u32 f_type;
	__u32 f_bsize;
	__u64 f_blocks;
	__u64 f_bfree;
	__u64 f_bavail;
	__u64 f_files;
	__u64 f_ffree;
	__kernel_fsid_t32 f_fsid;
	__u32 f_namelen;
	__u32 f_frsize;
	__u32 f_spare[5];
} __attribute__((packed));

/* from sys/uio.h */
struct iovec32 {
	__u32 iov_base;
	__s32 iov_len;
};

struct dirent32 {
	__u32	d_ino;
	__u32	d_off;
	__u16	d_reclen;
	char	d_name[256];
};

/* from bits/socket.h */
struct msghdr32 {
	__u32 msg_name;	/* Socket name			*/
	__s32 msg_namelen;	/* Length of name		*/
	__u32 msg_iov;	/* Data blocks			*/
	__u32 msg_iovlen;	/* Number of blocks		*/
	__u32 msg_control;	/* Per protocol magic (eg BSD file descriptor passing) */
	__u32 msg_controllen;	/* Length of cmsg list */
	__u32 msg_flags;
};

struct cmsghdr32 {
	__u32 cmsg_len;
	__s32 cmsg_level;
	__s32 cmsg_type;
};

/* macros to access cmsghdr32 structure */
#define CMSG32_ALIGN(len)	((__size_t32)((len)+sizeof(__size_t32)-1) & (__size_t32)~(sizeof(__size_t32)-1))
#define CMSG32_DATA(cmsg32)	((unsigned char *) ((struct cmsghdr32 *) (cmsg32) + 1))
#define CMSG32_FIRSTHDR(mhdr32) \
  ((__size_t32)(mhdr32)->msg_controllen >= sizeof(struct cmsghdr32)                 \
   ? (struct cmsghdr32 *)(mhdr32)->msg_control : (struct cmsghdr32 *)NULL)
#define CMSG_DIFF	(CMSG_ALIGN(sizeof(struct cmsghdr)) - CMSG32_ALIGN(sizeof(struct cmsghdr32)))
#define CMSG32_LEN(cmsg32)	((cmsg32).cmsg_len)

/* the following type are derived from glibc code
 * e.g. - sysdeps/unix/sysv/linux/i386/sigaction.c */
#define KERN_SIGSET_SIZE 8
typedef struct {
	unsigned sig[KERN_SIGSET_SIZE / sizeof (unsigned)];
} sigset32_t;

/* from /usr/include/asm/siginfo.h */

/* note that SI_PAD_SIZE / SIGEV_PAD_SIZE is different for 64 bit archs and 32 bit archs */
#define SI_PAD_SIZE32 ((SI_MAX_SIZE/sizeof(int)) - 3)
#ifndef SIGEV_MAX_SIZE
#define SIGEV_MAX_SIZE 64
#endif
#define SIGEV_PAD_SIZE32 ((SIGEV_MAX_SIZE/sizeof(int)) - 3)

typedef union sigval32 {
	int sival_int;
	unsigned int sival_ptr;
} sigval_t32;

typedef struct siginfo32 {
	int si_signo;
	int si_errno;
	int si_code;

	union {
		int _pad[SI_PAD_SIZE32];
		/* kill() */
		struct {
			unsigned int _pid;	/* sender's pid */
			unsigned int _uid;	/* sender's uid */
		} _kill;
		/* POSIX.1b timers */
		struct {
			unsigned int _timer1;
			unsigned int _timer2;
		} _timer;
		/* POSIX.1b signals */
		struct {
			unsigned int _pid;	/* sender's pid */
			unsigned int _uid;	/* sender's uid */
			sigval_t32 _sigval;
		} _rt;
		/* SIGCHLD */
		struct {
			unsigned int _pid;	/* which child */
			unsigned int _uid;	/* sender's uid */
			int _status;		/* exit code */
			__clock_t32 _utime;
			__clock_t32 _stime;
		} _sigchld;
		/* SIGILL, SIGFPE, SIGSEGV, SIGBUS */
		struct {
			unsigned int _addr;	/* faulting insn/memory ref. */
		} _sigfault;
		/* SIGPOLL */
		struct {
			int _band;	/* POLL_IN, POLL_OUT, POLL_MSG */
			int _fd;
		} _sigpoll;
	} _sifields;
} siginfo_t32;

typedef struct sigevent32 {
	sigval_t32 sigev_value;
	int sigev_signo;
	int sigev_notify;
	union {
		int _pad[SIGEV_PAD_SIZE32];
		struct {
			__u32 _function;
			__u32 _attribute;
			} _sigev_thread;
	} _sigev_un;
} sigevent_t32;

/* from asm/signal.h */
typedef struct sigaltstack32 {
	unsigned ss_sp;
	int ss_flags;
	unsigned ss_size;
} stack32_t;

/* sysdeps/unix/sysv/linux/kernel_sigaction.h */
typedef unsigned legacy_sigset32_t;
typedef struct legacy_sigaction32 {
	unsigned handler;
	legacy_sigset32_t mask;
	unsigned long flags;
	unsigned restorer;
} legacy_sigaction32_t;

/* sysdeps/unix/sysv/linux/kernel_sigaction.h */
typedef struct sigaction32 {
	unsigned handler;
	int flags;
	unsigned restorer;
	sigset32_t mask;
} sigaction32_t;

/* from asm/ucontext.h */
struct ucontext32 {
	unsigned _flags;
	unsigned _link;
	stack32_t _stack;
	sigcontext32_t _mcontext;
	sigset32_t _sigmask;
};

/* from linux/time.h */
struct timeval32 {
	int tv_sec, tv_usec;
};

struct timespec32 {
	int	tv_sec;
	int	tv_nsec;
};

struct itimerspec32 {
	struct timespec32 it_interval;
	struct timespec32 it_value;
};

/* needed by async io */
typedef unsigned int  aio_ctx32_t;

/* from asm/sigcontext.h */
struct _fpstate_ia32 {
	/* legacy fpstate */
	unsigned cw;
	unsigned sw;
	unsigned tag;
	unsigned ipoff;
	unsigned cssel;
	unsigned dataoff;
	unsigned datasel;
	unsigned _st[20];
	unsigned short status;

	/* indication of whether the new state is used */
	unsigned short magic;	/* 0xffff = regular FPU data only 
				 * 0x0000 = xmm data available
				 */

	/* new fp state */
	unsigned short xcwd __attribute__ ((aligned (16)));
	unsigned short xswd;
	unsigned short xtwd;
	unsigned short xfip;
	unsigned xfop;
	unsigned xfcs;
	unsigned xfoo;
	unsigned xfos;
	unsigned mxcsr;
	unsigned reserved;
	unsigned _fxsr_st[32];
	unsigned _xmm[32];
	unsigned padding[56];
};

/* from linux/socket.h */
struct sockaddr32 {
	unsigned short sa_family;
	unsigned char sa_data[14];
};

/* form linux/kernel.h */
struct sysinfo32 {
	__s32 uptime;		/* Seconds since boot */
	__u32 loads[3];		/* 1, 5, and 15 minute load averages */
	__u32 totalram;		/* Total usable main memory size */
	__u32 freeram;		/* Available memory size */
	__u32 sharedram;	/* Amount of shared memory */
	__u32 bufferram;	/* Memory used by buffers */
	__u32 totalswap;	/* Total swap space size */
	__u32 freeswap;		/* swap space still available */
	unsigned short procs;	/* Number of current processes */
	__u32 totalhigh;	/* Total high memory size */
	__u32 freehigh;		/* Available high memory size */
	__u32 mem_unit;		/* Memory unit size in bytes */
	char _f[8];		/* Padding: libc5 uses this.. */
};

/* from bits/resource.h */
struct rusage32 {
	struct timeval32 ru_utime;
	struct timeval32 ru_stime;
	int    ru_maxrss;
	int    ru_ixrss;
	int    ru_idrss;
	int    ru_isrss;
	int    ru_minflt;
	int    ru_majflt;
	int    ru_nswap;
	int    ru_inblock;
	int    ru_oublock;
	int    ru_msgsnd;
	int    ru_msgrcv;
	int    ru_nsignals;
	int    ru_nvcsw;
	int    ru_nivcsw;
};

#endif  /* end of !__ASSEMBLER__ */
	
#endif //_KERNEL_SUBST32_H

