/*  
 * ia32x_utils.c
 *  
 * main file for the utility lib
 * 
*/
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include "../ia32x_utils.h"
#include "../include/ia32x_utils_internal.h"

#ifndef AT_SYSINFO
#define AT_SYSINFO 32
#endif // AT_SYSINFO
#ifndef AT_SECURE
#define AT_SECURE 23
#endif // AT_SECURE


int g_log_file = 2; /* stderr */
/* fd of the binary to be emulated */
int g_is_at_execfd = 0;
unsigned long g_at_execfd = 0;
/* the address of __kernel_syscall_via_epc() */
unsigned long g_at_sysinfo = 0;
/* program headers */
void * g_at_phdr = NULL;
unsigned long g_at_phnum = 0;
/* the page size */
unsigned long g_at_pagesize = 0;
/* security info */
int g_is_at_secure = 0;
unsigned long g_at_secure = 0;

static char *error_message[] = {"",
		"For IA-32 application compatability you should install the IA32 Execution Layer from the installation CD"
};

static long analyze_at_vector (int * argc_addr, char ** argv, char ** envp);
static long lib_init (int * argc_addr, char ** argv, char ** envp);
/* not really needed - just silent the compiler warning */
long before_main (int argc, char ** argv, char ** envp);

/*****************************************************************************/
/*****************************************************************************/
long before_main (int argc, char ** argv, char ** envp)
{
	long ret;
	lib_init (&argc, argv, envp);
	ret = main (argc, argv, envp);
	return ret;
}
/*****************************************************************************/
/*****************************************************************************/
static long lib_init (int * argc_addr, char ** argv, char ** envp)
{
	long ret = 0;
	ret = analyze_at_vector (argc_addr, argv, envp);
	if (ret) 
		goto out;
	ret = mutex_module_init ();
	if (ret) 
		goto out;
	ret = malloc_module_init ();
	if (ret) 
		goto out;

	ret = allocate_initial_tls ();
	if (ret)
		goto out;

out:
	return ret;
}
/*****************************************************************************/
/*****************************************************************************/
static long analyze_at_vector (int * argc_addr, char ** argv, char ** envp)
{
	char ** p = NULL;
	unsigned long * elf_info = NULL;


	/* skip the vector of pointers to env strings */
	if (envp != NULL) {
		for (p = envp; *p != NULL; p ++);
		elf_info = (void*)++p;
	} else if (argv != NULL) {
		/* skip the vector of pointers to args */
		for (p = argv; *p != NULL; p ++);
		elf_info = (void*)++p;
	} else {
		/* there might be alignment issue here - 
		 * it's not performance critical, so let the kernel handle it */
		elf_info = (unsigned long*)(void*)(argc_addr + 1);
	}

	while (*(elf_info) != AT_NULL) {
		unsigned long id, val;
		id = *elf_info;
		val = *(elf_info+1);
		switch (id) {
			case AT_PHDR:
				g_at_phdr = (void*)val;
				break;
			case AT_PHNUM:
				g_at_phnum = val;
				break;
			case AT_SYSINFO:
				g_at_sysinfo = val;
				break;
			case AT_EXECFD:
				g_is_at_execfd = 1;
				g_at_execfd = val;
				break;
			case AT_PAGESZ:
				g_at_pagesize = val;
				break;
			case AT_SECURE:
				g_is_at_secure = 1;
				g_at_secure = val;
				break;
		}
		elf_info += 2;
	}
	return 0;
}
/*****************************************************************************/
/*****************************************************************************/
int lcs_before_fork ()
{
	return malloc_lock ();
}
/*****************************************************************************/
/*****************************************************************************/
int lcs_after_fork (long is_parent __attribute__((unused)))
{
	return malloc_unlock ();
}
/*****************************************************************************/
/* kill current thread-group */
/*****************************************************************************/
void abort ()
{
	SYSCALL (kill, (0, 9/*SIGKILL*/));
	/* dummy - supress the compiler warning about a noreturn function
	 * that does return */
	SYSCALL (exit_group, (0));
}
/*****************************************************************************/
/*****************************************************************************/
void lcs_get_ia32x_utils_version (char *version_str)
{
	sprintf(version_str,"%s", BUILD_NO);
	return;
}

/*****************************************************************************/
/* print the error message to the user and into adminstrator's log files     */
/*****************************************************************************/

void notify_error (int err)
{
	int fd;
	long retval;
	struct sockaddr sock;

	/* print the error message to the user */
	fprintf (stderr, "%s\n", error_message[err]);

	/* print the error message into the administrator's log files */
	fd = SYSCALL (socket, (AF_UNIX, SOCK_DGRAM, 0));
	if (IS_SYS_ERROR(fd)) {
		goto out;
	}
	retval = SYSCALL (fcntl, (fd, F_SETFD, (long)1));
	if (IS_SYS_ERROR(retval)) {
		goto out;
	}
	sock.sa_family = AF_UNIX;
	sprintf (sock.sa_data, "%s", PATH_LOG);		
	retval = SYSCALL (connect, (fd, &sock, (socklen_t)sizeof(sock)));
	if (IS_SYS_ERROR(retval)) {
		goto out;
	}
	retval = SYSCALL (send, (fd, (void *)error_message[err], strlen(error_message[err]) + 1, 0));
out:
	if (fd >= 0) {
		retval = SYSCALL (close, (fd));
	}
	return;
}







